/*

CPUDATA

  THIS MODULE PROVIDES MANAGEMENT OF 2+N-DIMENSIONAL DATA IN CPU MEMORY
  THIS INCLUDES DEALING WITH DIFFERENT KINDS OF IMAGE REPRESENTATIONS
  AND STREAMING OF IMAGES FROM VARIOUS SOURCES (VIDEO FILES, IMAGE FILES,
  HDF5 FILES, AND VARIOUS CAMERAS) AND TO VIDEO FILES

COPYRIGHT (C)

  FRAUNHOFER FIT-LIFE
  SCHLOSS BIRLINGHOVEN
  53754 ST. AUGUSTIN
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  Fraunhofer-Institut für Angewandte Informationstechnik FIT 
  http://fit.fraunhofer.de

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
/*

GPUIMAGEPROCESSING

  THIS MODULE PROVIDES A SIMPLE API FOR IMAGE OPERATIONS ON
  A GRAPHICAL PROCESSING UNIT (GPU) BY MAKING USE OF CUDA 
  TECHNOLOGY.

COPYRIGHT (C)

  ULM UNIVERSITY 
  INSTITUTE OF NEURAL INFORMATION PROCESSING (NEUROINFORMATIK)
  89069 ULM
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  INSTITUTE OF NEURAL INFORMATION PROCESSING:
  http://www.uni-ulm.de/en/in/institute-of-neural-information-processing.html

  ULM UNIVERSITY 
  HTTP://WWW.UNI-ULM.DE/

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
#ifndef BAUMERIMAGESEQUENCE_H
#define BAUMERIMAGESEQUENCE_H


#include "HostImageSequence.h"
#include "simple_fifo.hpp"

#include <string>
#include <vector>

using namespace std;



#include <bgapi.hpp>
using namespace BGAPI;

#define DEFAULT_EXPOSURE    5000 //30000
#define DEFAULT_GAIN        0.0f  //4.75f
#define DEFAULT_IMAGEFORMAT     3//3  // full frame, 15fps ( 2 is HQ, 8fps)
#define DEFAULT_PIXELFORMAT     0//5  // Mono 8 (but the format depends on Colorspace)
#define DEFAULT_BUFFERLENGTH    4

#define HW_TIMER_FACTOR       1.0 / (31.275*1000000.0)


#define USE_EXTERNAL_BUFFER
//#define USE_TRIGGER

/**
 * This class gets images from a Baumer camera and stores frames in a 
 * (circular) buffer from which the current frame can be grabbed
 */
class BaumerImageSequence : public HostImageSequence
{
public:
	struct singletonCamera
	{
		BGAPI::Camera*   pCamera;
		bool             isFree;
		std::string      modelName;
		std::string      serialNumber;
	};

	static const HostImage::ColorSpace DEFAULT_SPACE = HostImage::LUM;
	/**
	 * This constructor creates an enviroment BGAPI for camera access and
	 * connects to the first free camera 
	 */
	BaumerImageSequence( HostImage::ColorSpace C = DEFAULT_SPACE, int imageformat = -1, int pixelformat = -1, int gain = -1, float exposure = -1.f, int bufferlength = -1, bool write_log = false );
	/**
	 * This constructor creates an enviroment BGAPI for camera access and
	 * connects to the camera given by string
	 */
	BaumerImageSequence(std::string camera_name, HostImage::ColorSpace C = DEFAULT_SPACE, int imageformat = -1, int pixelformat = -1, int gain = -1, float exposure = -1.f, int bufferlength = -1, bool write_log = false );
	/**
	 * This constructor creates an enviroment BGAPI for camera access and
	 * connects to the camera given by index
	 */
	BaumerImageSequence(int camera_number, HostImage::ColorSpace C = DEFAULT_SPACE, int imageformat = -1, int pixelformat = -1, int gain = -1, float exposure = -1.f, int bufferlength = -1, bool write_log = false );
	
	
	
	/**
	 * frees the memory of all read images
	 */
	virtual ~BaumerImageSequence();
	virtual HostImage getNextImage();
    virtual HostImage getCurrentImage();
	virtual bool hasNextImage();
	virtual int getWidth();
	virtual int getHeight();
	virtual void reset();
	
	
	
	void setImageFormat( int index );
	void setImageFormat( std::string name );
	std::vector<BGAPIX_CameraImageFormat> const& getImageFormats( bool force_check=false );
    std::string getImageFormatStr();
	int getCurrentImageFormatIndex( bool force_check=false );
	BGAPIX_CameraImageFormat const& getCurrentImageFormat( bool force_check=false );
	
	
	void setPixelFormat( int index );
	void setPixelFormat( std::string name );
	int getPixelFormatByName( std::string name );
	std::vector<BGAPIX_CameraPixelFormat> const& getPixelFormats( bool force_check=false );
	std::string getPixelFormatStr();
	
    int getCurrentPixelFormatIndex( bool force_check=false );
	BGAPIX_CameraPixelFormat const& getCurrentPixelFormat( bool force_check=false );
	void setIP( unsigned char* ip, unsigned char* subnetmask, unsigned char* gateway );

	
	void setDestinationPixelFormat( int index );
	void setDestinationPixelFormat( std::string name );
	std::vector<BGAPIX_CameraPixelFormat> const& getDestinationPixelFormats( bool force_check=false );
	int getCurrentDestinationPixelFormatIndex( bool force_check=false );
	BGAPIX_CameraPixelFormat const& getCurrentDestinationPixelFormat( bool force_check=false );


	// there is no set and get for variable space, because it mustn't be set except by the constructor
	
	float getGain( bool force_check=false );
	void setGain( float gain );
	int getExposure( bool force_check=false );
	void setExposure( int exposure );
	void setPacketDelay( int ipg );
	void setPacketSize( int packetsize );
	void setConstantFps( float fps );
    
    
private:
	void init( int camera_number = 0, int imageformat = -1, int pixelformat = -1);
	void init_test();
	void clear();
	bool startCamera();
	bool stopCamera();

	HostImage                            _current_image;

	HostImage::ColorSpace                _space;
	int                                  _frame_number;	
	int                                  _time_stamp;
	bool                                 _camera_running;
	int									 _sleep_time;
	int                                  _camera_number;
    std::string                          _camera_name;
    
	int                                  _height;
	int                                  _width;
	int                                  _depth;

	/// ********* Baumer enviroment *************
	static BGAPI::System*                initBGAPI( ); 
	static void                          releaseBGAPI();
	static void                          createCamera( std::string camera_name );
	static int                           getCameraByString( std::string name );
	static int                           getSystemCount();
	static int                           getCameraCount();
	static int                           getGlobalCameraCount();
	static BGAPI::System*                getSystem();
	static bool                          has_file( std::string filename );
	/// ********* global variables  *************
public:
    static  std::string                  g_library_path;
	static int                           g_active_users_count;
	static BGAPI::System*                g_pSystem;
	static int                           g_system_count;
	static std::vector< singletonCamera > g_pCameras;
	
	
	/// ********* local variables   *************
private:
    BGAPI::System*                       pSystem;

	BGAPIX_SystemInfo                    sysdeviceinfo;	
	BGAPI::Camera*                       pCamera;
	BGAPI_RESULT                         res;
	BGAPI_FeatureState                   state;
	BGAPIX_TypeRangeINT	                 Datalength;
	BGAPIX_CameraInfo                    camdeviceinfo;
	
	//Feature ImageFormat (chapter 4.3.1.)
	std::vector<BGAPIX_CameraImageFormat> ImageFormats;   
	int									 ImageFormatIndex;
	
	//Variables for PixelFormat (chapter 4.3.2.)
	std::vector<BGAPIX_CameraPixelFormat> PixelFormats;
	int									 PixelFormatIndex;
	
	//Variables for Destination PixelFormat
	std::vector<BGAPIX_CameraPixelFormat> DestinationPixelFormats;
	int									 DestinationPixelFormatIndex;
	
	
	//Variables for Exposure (chapter 4.3.3.)
	BGAPIX_TypeRangeINT                  exposurerange;
	int                                  exposurevalue;
	
	//Variables for Gain (chapter 4.3.4.)
	BGAPIX_TypeRangeFLOAT                gainrange;
	float                                gainvalue;
	
	// length of incoming image buffer
	int                                  bufferlength;
	
    // write log files to disk
    bool                                 write_log;


  //Variables for Trigger control (chapters 4.4.1. - 4.4.3.)
	BGAPIX_TypeListINT                   TriggerSourceList;
	BGAPIX_TypeListINT                   TriggerActivation;
	//*****************************************
};


#endif
