/*

CPUDATA

  THIS MODULE PROVIDES MANAGEMENT OF 2+N-DIMENSIONAL DATA IN CPU MEMORY
  THIS INCLUDES DEALING WITH DIFFERENT KINDS OF IMAGE REPRESENTATIONS
  AND STREAMING OF IMAGES FROM VARIOUS SOURCES (VIDEO FILES, IMAGE FILES,
  HDF5 FILES, AND VARIOUS CAMERAS) AND TO VIDEO FILES

COPYRIGHT (C)

  FRAUNHOFER FIT-LIFE
  SCHLOSS BIRLINGHOVEN
  53754 ST. AUGUSTIN
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  Fraunhofer-Institut für Angewandte Informationstechnik FIT 
  http://fit.fraunhofer.de

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
#ifndef NEWCPUDATA_HPP_HETXABHM
#define NEWCPUDATA_HPP_HETXABHM

#include "CPUDataBlock.hpp"

#include <stdexcept>
#include <vector>
#include <typeinfo>
#include <boost/proto/proto.hpp>

#include "CPUDataNDBase.hpp"
#include "CPUDataNDExpressions.hpp"

template<typename T>
class CPUDataND
  : public cpudata_expr< typename proto::terminal< CPUDataND_<T> >::type >
{

public:

  typedef cpudata_expr< typename proto::terminal< CPUDataND_<T> >::type > Base;
  typedef T num_type;
  typedef CPUDataBlock<T> DataBlockT;
  typedef CPUDataBlockReference<T> DataBlockReferenceT;
  typedef CPUDataND<T> This;
  typedef boost::proto::literal< CPUDataND<T> > Literal;

 friend std::ostream &operator <<(std::ostream &sout, const CPUDataND<T> & data )
 {
   const CPUDataND<T> * ptr = (&data);
   sout << "CPUDataND" << ptr << "<" << typeid(T).name() << ">";
   sout << "(";
   for( size_t i = 0; i < data.rank(); ++i )
     {
       sout << data.extent(i);
       if ( i + 1 < data.rank() ) sout << "x";
     }
   sout << ")";
   return sout;
 }

  CPUDataND( size_t n1 )
  {
    size_t shape[1] = { n1 };
    impl().init( 1, shape );
  }

  CPUDataND( size_t n1, size_t n2 )
  {
    size_t shape[2] = { n1, n2 };
    impl().init( 2, shape );
  }

  CPUDataND( size_t n1, size_t n2, size_t n3 )
  {
    size_t shape[3] = { n1, n2, n3 };
    impl().init( 3, shape );
  }

  CPUDataND()
  {
    impl().init();
  }

  CPUDataND( size_t rank, size_t * shape )
  {
    impl().init( rank, shape );
  }
  
  void clear()
  {
	impl().clear();
  }

  T * data() { return impl().data(); }
  const T * data() const { return impl().data(); }
  T * data_end() { return impl().data() + impl().size(); }
  const T * data_end() const { return impl().data() + impl().size(); }

  operator T()
  {
#ifdef DEBUG
    if ( rank() > 0 ) throw std::runtime_error("trying to cast array to singelton");
#endif
    return *( data() + offset() );
  }
  
  size_t rank() const 
  { 
	  return impl().dims().size(); 
  }

  ptrdiff_t offset() const 
  {
	  return impl().offset();
  }

  size_t extent( size_t index ) const
  { 
	  return impl().dim(index).extent;
  }

  ptrdiff_t stride( size_t index ) const
  { 
	  return impl().dim(index).stride;
  }

  bool contiguous() const 
  { 
	  return impl().contiguous();
  }

  size_t size() const 
  {
    return impl().size();
  }

private:
  
  typedef CPUDataND_<T> Impl;

  Impl & impl() 
  { 
	  return Base::base_type::proto_base().child0;
  }

  const Impl & impl() const
  { 
	  return Base::base_type::proto_base().child0;
  }

};

#endif /* end of include guard: NEWCPUDATA_HPP_HETXABHM */
