/*

CPUDATA

  THIS MODULE PROVIDES MANAGEMENT OF 2+N-DIMENSIONAL DATA IN CPU MEMORY
  THIS INCLUDES DEALING WITH DIFFERENT KINDS OF IMAGE REPRESENTATIONS
  AND STREAMING OF IMAGES FROM VARIOUS SOURCES (VIDEO FILES, IMAGE FILES,
  HDF5 FILES, AND VARIOUS CAMERAS) AND TO VIDEO FILES

COPYRIGHT (C)

  FRAUNHOFER FIT-LIFE
  SCHLOSS BIRLINGHOVEN
  53754 ST. AUGUSTIN
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  Fraunhofer-Institut für Angewandte Informationstechnik FIT 
  http://fit.fraunhofer.de

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
/*

GPUIMAGEPROCESSING

  THIS MODULE PROVIDES A SIMPLE API FOR IMAGE OPERATIONS ON
  A GRAPHICAL PROCESSING UNIT (GPU) BY MAKING USE OF CUDA 
  TECHNOLOGY.

COPYRIGHT (C)

  ULM UNIVERSITY 
  INSTITUTE OF NEURAL INFORMATION PROCESSING (NEUROINFORMATIK)
  89069 ULM
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  INSTITUTE OF NEURAL INFORMATION PROCESSING:
  http://www.uni-ulm.de/en/in/institute-of-neural-information-processing.html

  ULM UNIVERSITY 
  HTTP://WWW.UNI-ULM.DE/

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
#ifndef HOSTIMAGESEQUENCE_H_
#define HOSTIMAGESEQUENCE_H_

#include "HostImage.h"
//#include "GPUImage.h"

/**
 * This class is a simple abstract interface to get a sequenace of images.
 * A simple implementation of this abstract interface, which reads images
 * from file is SimpleHostImageSequence
 */
class HostImageSequence
{
public:
    /**
    * Simple Constructor to initialize the member variables defined in HostImageSequence
    *
    **/
  HostImageSequence(float step = 1.0f):frame_stepsize(step){}; 

	/**
	 * Returns the next image (the crucial data is reference counted).
	 *
	 * This HostImageSequence remains responsible for the returned HostImage
	 * and its allocated memory. Any implementation of this function garanties
	 * that the returned HostImage object remains vailid at least until the
	 * next image (the image following this one) is returned by this function.
	 *
	 * This function is only garanteed to work if hasNextImage() returns true
	 */
	virtual HostImage getNextImage() = 0;

  /**
   *  Returns the image the frame_number is pointing to.
   *  In contrast to getNextImage the frame number is NOT being increased!
  */
  virtual HostImage getCurrentImage() = 0;


	/**
	 * This function simply copies the current HostImage to the given GPUImage
	 * object by calling the GPUImage::loadImage(const HostImage&) function
	 * on the given GPUImage object.
	 * The current Host image is got by calling getNextImage() on this object.
	 *
	 * This function is only garanteed to work if hasNextImage() returns true
	 */
  //virtual void copyNextImage(GPUImage& gData);

	/**
	 * This function retuns true if a new image is present to be retuned by
	 * getNextImage() or copied by copyNextImage().
	 */
	virtual bool hasNextImage() = 0;

	/**
	 * returning the width of returned images
	 */
	virtual int getWidth() = 0;

	/**
	 * returning the height of returned images
	 */
	virtual int getHeight() = 0;

	virtual void reset() {}

  /**
  * Sets the stepsize for image grabbing, default is 1
  */
  virtual void setFrameStepsize(float stepsize);

  ///vrrtual destructor neded for derived class destruction!!
  virtual ~HostImageSequence() {}
protected:
  float frame_stepsize;
};


#endif /*HOSTIMAGESEQUENCE_H_*/
