/*

CPUDATA

  THIS MODULE PROVIDES MANAGEMENT OF 2+N-DIMENSIONAL DATA IN CPU MEMORY
  THIS INCLUDES DEALING WITH DIFFERENT KINDS OF IMAGE REPRESENTATIONS
  AND STREAMING OF IMAGES FROM VARIOUS SOURCES (VIDEO FILES, IMAGE FILES,
  HDF5 FILES, AND VARIOUS CAMERAS) AND TO VIDEO FILES

COPYRIGHT (C)

  FRAUNHOFER FIT-LIFE
  SCHLOSS BIRLINGHOVEN
  53754 ST. AUGUSTIN
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  Fraunhofer-Institut für Angewandte Informationstechnik FIT 
  http://fit.fraunhofer.de

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
#include "IDXImageSequence.h"

IDXImageSequence::IDXImageSequence( const std::string & path )
: _file ( path ), _i(0)
{
  if ( _file.rank() > 3 )
	throw std::runtime_error("IDXImageSequence cannot handle images with more than 3 dimensions");
  _n = _file.extent(0);
  _h = _file.extent(1);
  _w = _file.extent(2);  
  //std::cout << " idxseq n = " << _n << " w = " << _w << " h = " << _h << std::endl;
  if ( _file.type() == IDXFile::UBYTE )
	_current.resize( _w, _h, HostImage::UBYTE, HostImage::LUM );
  else
	_current.resize( _w, _h, HostImage::FLOAT, HostImage::LUM );
}

HostImage IDXImageSequence::getNextImage()
{
  if ( ! hasNextImage() )
	throw std::runtime_error( "IDXImageSequence: no next image available" );

  if ( _file.type() == IDXFile::UBYTE )
	_file.read_elements( _current.cpu_data_ubyte().data(), _w * _h );
  else
	_file.read_elements( _current.cpu_data_float().data(), _w * _h );

  return _current;
}

HostImage IDXImageSequence::getCurrentImage()
{
  return _current;
}

bool IDXImageSequence::hasNextImage()
{
  return _i < _n;
}

int IDXImageSequence::getWidth()
{
  return _w;
}

int IDXImageSequence::getHeight()
{
  return _h;
}

void IDXImageSequence::reset() 
{
  _file.reset();
}
