/*

CPUDATA

  THIS MODULE PROVIDES MANAGEMENT OF 2+N-DIMENSIONAL DATA IN CPU MEMORY
  THIS INCLUDES DEALING WITH DIFFERENT KINDS OF IMAGE REPRESENTATIONS
  AND STREAMING OF IMAGES FROM VARIOUS SOURCES (VIDEO FILES, IMAGE FILES,
  HDF5 FILES, AND VARIOUS CAMERAS) AND TO VIDEO FILES

COPYRIGHT (C)

  FRAUNHOFER FIT-LIFE
  SCHLOSS BIRLINGHOVEN
  53754 ST. AUGUSTIN
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  Fraunhofer-Institut für Angewandte Informationstechnik FIT 
  http://fit.fraunhofer.de

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
#ifndef _INDEXEXPRESSIONS_HPP_
#define _INDEXEXPRESSIONS_HPP_

#include <boost/proto/proto.hpp>

#include <boost/mpl/set.hpp>
#include <boost/mpl/int.hpp>
#include <boost/mpl/joint_view.hpp>
#include <boost/preprocessor.hpp>

namespace mpl = boost::mpl;

#define HAVE_INDEX_GRAMMAR 1
#define INDEX_PLACEHOLDER_MAX 6
#define PLACEHOLDER_NAMES (_i,_j,_k,_l,_m,_n)
#define N_PLACEHOLDER_NAMES 6
#define INDEX_BINARY_OP_NAMES (plus,minus,multiplies,divides)
#define N_INDEX_BINARY_OP_NAMES 4

namespace proto = boost::proto;

// all the index stuff is really just the calculator example with ptrdiff_t as type

template<unsigned int I>
struct index_placeholder
{
  static const int value = I;
  friend std::ostream &operator <<(std::ostream &sout, index_placeholder)
  {
	return sout << "index_placeholder<" << I << ">";
  }
};

template<typename Expr>
struct index_expr;

// define the context to do math with index_placeholders
template<int N>
struct index_context
  : proto::callable_context< index_context<N> const >
{
    // the values for the indices
    ptrdiff_t args[N];
    
    // we work with ptrdiff_t's as index values
    typedef ptrdiff_t result_type;
    
    // evaluator for the terminals, everything else uses proto defaults
    template<unsigned int I>
    result_type operator()(proto::tag::terminal, index_placeholder<I>) const
    {
        return I < N ? args[I] : result_type(0);
    }

    // constructors
    index_context( )
    {}    

#define CONSUME_ARG(Z,I,D) BOOST_PP_CAT(*pargs++ = a,I);
#define BOOST_PP_LOCAL_LIMITS (1, INDEX_PLACEHOLDER_MAX+1 )
#define BOOST_PP_LOCAL_MACRO(I)                                      \
    index_context( BOOST_PP_ENUM_PARAMS(I, ptrdiff_t a) )            \
    {                                                                \
      ptrdiff_t * pargs = args;                                      \
      BOOST_PP_REPEAT( I, CONSUME_ARG, NIL )                         \
    }
#include BOOST_PP_LOCAL_ITERATE()
#undef CONSUME_ARG

};

#if HAVE_INDEX_GRAMMAR

struct index_placeholder_grammar
: proto::or_<
#define BOOST_PP_LOCAL_LIMITS (0, INDEX_PLACEHOLDER_MAX)
#define BOOST_PP_LOCAL_MACRO(I)                                      \
         proto::terminal< index_placeholder<I> >                     \
  	     BOOST_PP_COMMA_IF(BOOST_PP_LESS(I, INDEX_PLACEHOLDER_MAX))
#include BOOST_PP_LOCAL_ITERATE()
>
{};

// matches index placeholder terminals up to order 5
struct index_terminal_grammar
: proto::or_<
              index_placeholder_grammar,
              proto::terminal< ptrdiff_t >,
              proto::terminal< int >
              > {};

// Define the grammar of index expressions
struct index_grammar;
struct index_grammar_cases
{
  template<typename Tag > struct case_ : proto::not_<proto::_>  {};
};
template<> struct index_grammar_cases::case_< proto::tag::terminal > : index_terminal_grammar {};

// binary ops
#define BOOST_PP_LOCAL_LIMITS (0, N_INDEX_BINARY_OP_NAMES-1)
#define OP(I) BOOST_PP_TUPLE_ELEM(N_INDEX_BINARY_OP_NAMES,I,INDEX_BINARY_OP_NAMES)
#define BOOST_PP_LOCAL_MACRO(N) \
template<> struct index_grammar_cases::case_< proto::tag:: OP(N) > : \
proto:: OP(N)< index_grammar, index_grammar > {};
#include BOOST_PP_LOCAL_ITERATE()
#undef OP

struct index_grammar
  : proto::switch_<index_grammar_cases> {};

// Define index domain using index_expr as a wrapper and index_grammar as a grammar
struct index_domain
  : proto::domain< proto::pod_generator<index_expr>, index_grammar >
{};

#else

struct index_domain
  : proto::domain< proto::pod_generator<index_expr> >
{};

#endif

// index expression wrapper that retains pod-ness
// so that we can statically construct index placeholders
template<typename Expr>
struct index_expr
{

  BOOST_PROTO_BASIC_EXTENDS(Expr, index_expr<Expr>, index_domain);

  typedef ptrdiff_t result_type;

#define BOOST_PP_LOCAL_LIMITS (0, INDEX_PLACEHOLDER_MAX+1 )
#define BOOST_PP_LOCAL_MACRO(I)                                          \
    result_type operator()( BOOST_PP_ENUM_PARAMS(I, ptrdiff_t a) ) const \
    {                                                                    \
      index_context<I> ctx( BOOST_PP_ENUM_PARAMS(I, a) );                \
      return proto::eval(*this, ctx );                                   \
    }
#include BOOST_PP_LOCAL_ITERATE()

};

// pre-define some index_placeholders
#define BOOST_PP_LOCAL_LIMITS (0, N_PLACEHOLDER_NAMES-1)
#define BOOST_PP_LOCAL_MACRO(I)                      \
index_expr< proto::terminal< index_placeholder<I> >::type > \
const BOOST_PP_TUPLE_ELEM(N_PLACEHOLDER_NAMES,I,PLACEHOLDER_NAMES) = {{{}}};
#include BOOST_PP_LOCAL_ITERATE()



template<int I>
struct Constant : proto::callable
{
  typedef int result_type;
  result_type operator()() const
  {
    return I;
  }
};

#define BINARY_OP_CALLABLE(OP,NAME)               \
struct NAME : proto::callable                     \
{                                                 \
  typedef int result_type;                        \
  result_type operator()(int i1, int i2) const    \
  {                                               \
    return i1 OP i2;                              \
  }                                               \
};
BINARY_OP_CALLABLE(*,Multiply)
BINARY_OP_CALLABLE(+,Add)
BINARY_OP_CALLABLE(-,Subtract)
#undef BINARY_OP_CALLABLE

// calcs the stride (differential) an index exression has in a given index
// only constant differentials allowed (i.e. the expr must be linear)
template<unsigned int I>
struct IndexStride
  : proto::or_<
      // the addressed index contributes 1
      proto::when< 
        proto::terminal< index_placeholder<I> >,
        Constant<1>()
      >
      ,
      // non-addressed indices don't contribute to the stride
      proto::when< 
        index_placeholder_grammar,
        Constant<0>()
      >
      ,
      proto::when< 
        proto::multiplies< proto::terminal< int >, IndexStride<I> > ,
        Multiply( proto::_value( proto::_left ), IndexStride<I>( proto::_right ) )
      >
      ,
      proto::when< 
        proto::multiplies< IndexStride<I>, proto::terminal< int > > ,
        Multiply( IndexStride<I>( proto::_left ), proto::_value( proto::_right ) )
      >
      ,
      proto::when< 
        proto::terminal< int >,
        Constant<0>()
      >
      ,
      proto::when<
        proto::plus< IndexStride<I>, IndexStride<I> > ,
        Add( IndexStride<I>( proto::_left ), IndexStride<I>( proto::_right ) )
        >
      ,
      proto::when<
        proto::minus< IndexStride<I>, IndexStride<I> > ,
        Subtract( IndexStride<I>( proto::_left ), IndexStride<I>( proto::_right ) )
        >
    >
    {};

template<typename placeholder> struct IndexPlaceholderValue;

template<int I>
struct IndexPlaceholderValue< index_placeholder<I> >
{
  static const int value = I;
};

struct IndexExprIndexSet
  : proto::or_<
      proto::or_<

#define BOOST_PP_LOCAL_LIMITS (0, INDEX_PLACEHOLDER_MAX)
#define BOOST_PP_LOCAL_MACRO(I)                      \
         proto::when<                                \
          proto::terminal< index_placeholder<I> >,   \
  				mpl::set< mpl::int_<I> >()                 \
  	     >                                           \
  	     BOOST_PP_COMMA_IF(BOOST_PP_LESS(I, INDEX_PLACEHOLDER_MAX))
#include BOOST_PP_LOCAL_ITERATE()
      >
      ,
      proto::when< proto::terminal<proto::_>,
				   mpl::set<>()
			>
      , 
      proto::when< proto::unary_expr<proto::_,IndexExprIndexSet>,
            IndexExprIndexSet(proto::_child)
      >
      , 
      proto::when< proto::binary_expr<proto::_,IndexExprIndexSet, IndexExprIndexSet>,
				   mpl::joint_view<IndexExprIndexSet(proto::_left),IndexExprIndexSet(proto::_right)>::type()
        >


  >
{};

struct IndexOffset
  : proto::or_<
      proto::when< 
        index_placeholder_grammar,
        Constant<0>()
      >
      ,
      proto::when< 
        proto::multiplies< index_placeholder_grammar, IndexOffset > ,
        Constant<0>()
      >
      ,
      proto::when< 
        proto::multiplies< IndexOffset, index_placeholder_grammar > ,
        Constant<0>()
      >
      ,
      proto::when< 
        proto::negate< proto::terminal< int > >,
        proto::negate< proto::_value >
      >
      ,
      proto::when< 
        proto::terminal< int >,
        proto::_value
      >
      ,
      proto::when<
        proto::plus< IndexOffset, IndexOffset > ,
        Add( IndexOffset( proto::_left ), IndexOffset( proto::_right ) )
        >
      ,
      proto::when<
        proto::multiplies< IndexOffset, IndexOffset > ,
        Multiply( IndexOffset( proto::_left ), IndexOffset( proto::_right ) )
        >
      ,
      proto::when<
        proto::minus< IndexOffset, IndexOffset > ,
        Subtract( IndexOffset( proto::_left ), IndexOffset( proto::_right ) )
        >
    >
    {};

#endif /* _INDEXEXPRESSIONS_HPP_ */
