/*

CPUDATA

  THIS MODULE PROVIDES MANAGEMENT OF 2+N-DIMENSIONAL DATA IN CPU MEMORY
  THIS INCLUDES DEALING WITH DIFFERENT KINDS OF IMAGE REPRESENTATIONS
  AND STREAMING OF IMAGES FROM VARIOUS SOURCES (VIDEO FILES, IMAGE FILES,
  HDF5 FILES, AND VARIOUS CAMERAS) AND TO VIDEO FILES

COPYRIGHT (C)

  FRAUNHOFER FIT-LIFE
  SCHLOSS BIRLINGHOVEN
  53754 ST. AUGUSTIN
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  Fraunhofer-Institut für Angewandte Informationstechnik FIT 
  http://fit.fraunhofer.de

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
#ifndef SIMPLEHOSTIMAGESEQUENCE_H
#define SIMPLEHOSTIMAGESEQUENCE_H
#include "HostImageSequence.h"

#include <list>
#include <string>

#include <boost/scoped_ptr.hpp>

class image_fetcher;

/**
 * This class can gradually read files using the read method of HostImage
 */
class SimpleHostImageSequence : public HostImageSequence
{

private:

  std::string  _pattern;
  int _start, _end;
  size_t _n_prefetch;
  float _step_size;

  boost::scoped_ptr<image_fetcher> _image_fetcher;
  bool _image_available;
  HostImage _current_image;

public:

	/**
	 * This constructor creates a list of image names by passing the parameter
	 * 'fileExpr' to printf() with every integers within the given range
	 * [start , end] (including start and end).
	 * All the images are read to HostImage objects and stored in a list.
	 */
	SimpleHostImageSequence(const char* fileExpr, int start, int end, int numbersToLoadA = 5, float frame_stepsizeA=1.f);

	virtual ~SimpleHostImageSequence();
	virtual HostImage getNextImage();
    virtual HostImage getCurrentImage();
	virtual bool hasNextImage();
	//virtual void copyNextImage(GPUImage& gData);
	virtual int getWidth();
	virtual int getHeight();
	virtual void reset();


};

#endif
