/*

CPUDATA

  THIS MODULE PROVIDES MANAGEMENT OF 2+N-DIMENSIONAL DATA IN CPU MEMORY
  THIS INCLUDES DEALING WITH DIFFERENT KINDS OF IMAGE REPRESENTATIONS
  AND STREAMING OF IMAGES FROM VARIOUS SOURCES (VIDEO FILES, IMAGE FILES,
  HDF5 FILES, AND VARIOUS CAMERAS) AND TO VIDEO FILES

COPYRIGHT (C)

  FRAUNHOFER FIT-LIFE
  SCHLOSS BIRLINGHOVEN
  53754 ST. AUGUSTIN
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  Fraunhofer-Institut für Angewandte Informationstechnik FIT 
  http://fit.fraunhofer.de

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
#ifndef VIDEOIMAGESEQUENCE_H
#define VIDEOIMAGESEQUENCE_H

#include "HostImageSequence.h"


#include <list>
#include <string>


class AVFormatContext;
class AVCodecContext;
class AVCodec;
class AVPacket;
class AVFrame;
struct SwsContext;

class VideoImageSequence : public HostImageSequence
{
public:

	VideoImageSequence( const char* filename );
	virtual ~VideoImageSequence();


	/**
	 * Returns a pointer to the next image.
	 *
	 * This HostImageSequence remains responsible for the returned HostImage
	 * and its allocated memory. Any implementation of this function garanties
	 * that the returned HostImage object remains vailid at least until the
	 * next image (the image following this one) is returned by this function.
	 *
	 * This function is only garanteed to work if hasNextImage() returns true
	 */
	virtual HostImage getNextImage();

	virtual HostImage getCurrentImage();

	/**
	 * This function retuns true if a new image is present to be retuned by
	 * getNextImage() or copied by copyNextImage().
	 */
	virtual bool hasNextImage();

	/**
	 * returning the width of returned images
	 */
	virtual int getWidth();

	/**
	 * returning the height of returned images
	 */
	virtual int getHeight();

protected:

	bool GetNextFrame();

    AVFormatContext *pFormatCtx;
    int             videoStream;
    AVCodecContext  *pCodecCtx;
    AVCodec         *pCodec;
    AVFrame         *pFrame;
    AVFrame         *pFrameRGB;
    int             numBytes;
    uint8_t         *buffer;

    bool            hasImageInBuffer;
    int             frame;
    float           frame_next;

	std::string _path;

	struct SwsContext * _img_convert_ctx;

private:
};
#endif
