/*

CPUDATA

  THIS MODULE PROVIDES MANAGEMENT OF 2+N-DIMENSIONAL DATA IN CPU MEMORY
  THIS INCLUDES DEALING WITH DIFFERENT KINDS OF IMAGE REPRESENTATIONS
  AND STREAMING OF IMAGES FROM VARIOUS SOURCES (VIDEO FILES, IMAGE FILES,
  HDF5 FILES, AND VARIOUS CAMERAS) AND TO VIDEO FILES

COPYRIGHT (C)

  FRAUNHOFER FIT-LIFE
  SCHLOSS BIRLINGHOVEN
  53754 ST. AUGUSTIN
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  Fraunhofer-Institut für Angewandte Informationstechnik FIT 
  http://fit.fraunhofer.de

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
/*

GPUIMAGEPROCESSING

  THIS MODULE PROVIDES A SIMPLE API FOR IMAGE OPERATIONS ON
  A GRAPHICAL PROCESSING UNIT (GPU) BY MAKING USE OF CUDA 
  TECHNOLOGY.

COPYRIGHT (C)

  ULM UNIVERSITY 
  INSTITUTE OF NEURAL INFORMATION PROCESSING (NEUROINFORMATIK)
  89069 ULM
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  INSTITUTE OF NEURAL INFORMATION PROCESSING:
  http://www.uni-ulm.de/en/in/institute-of-neural-information-processing.html

  ULM UNIVERSITY 
  HTTP://WWW.UNI-ULM.DE/

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
#ifndef _VIDEOOUTPUTFILE_HPP_
#define _VIDEOOUTPUTFILE_HPP_

extern "C" {
#ifdef HAVE_FFMPEG_LIB_PREFIXES
	#include <libavcodec/avcodec.h>
	#include <libavformat/avformat.h>
	#include <libswscale/swscale.h>
#else
	#include <avcodec.h>
	#include <avformat.h>
	#include <swscale.h>
#endif
};

#include <boost/scoped_ptr.hpp>
#include <boost/mem_fn.hpp>
#include <boost/thread.hpp>

#include "HostImage.h"
#include "concurrent_queue.hpp"

class VideoOutputFile
{
  
public:

  void write_video_frame( HostImage image );
  
  VideoOutputFile( const std::string & sfilename, size_t width, size_t height, HostImage::ColorSpace space = HostImage::RGB,
                   float fps = 25, size_t rate = 100000, CodecID codec_id =  CODEC_ID_MJPEG );
  VideoOutputFile()
  :   _av_format(NULL)
    , _av_format_context(NULL)
    , _img_convert_ctx(NULL)
    , _av_stream(NULL)
    , _picture(NULL)
    , _picture_buf(NULL)
    , _video_outbuf(NULL)
  {
    throw std::runtime_error(" default constructing videooutputfile " );
  }

   ~VideoOutputFile();
   
    void close();         

  // thread entry point
  void operator()()
  {
    while ( true )
    {
      HostImage image;
      if ( _output_queue.pop( image, true ) )
      {
		do_write_video_frame( image );
      }
      else break;
    }
  }

protected:
  
  void do_write_video_frame( HostImage image );

    void open( const std::string & sfilename, size_t width, size_t height, HostImage::ColorSpace space,
               float fps, size_t rate, CodecID codec_id );
    
    AVOutputFormat  * _av_format;
    AVFormatContext * _av_format_context;
    SwsContext      * _img_convert_ctx;
    AVStream        * _av_stream;
    AVFrame         * _picture;
    AVFrame         * _picture_buf;
    uint8_t         * _video_outbuf;
    size_t            _frame_count;
    size_t            _video_outbuf_size;
    PixelFormat       _pixel_format;
    HostImage::ColorSpace _color_space;
    bool              _open;
  
  concurrent_queue< HostImage > _output_queue;
  boost::scoped_ptr<boost::thread> _thread;

};

#endif /* _VIDEOOUTPUTFILE_HPP_ */
