/*

CPUDATA

  THIS MODULE PROVIDES MANAGEMENT OF 2+N-DIMENSIONAL DATA IN CPU MEMORY
  THIS INCLUDES DEALING WITH DIFFERENT KINDS OF IMAGE REPRESENTATIONS
  AND STREAMING OF IMAGES FROM VARIOUS SOURCES (VIDEO FILES, IMAGE FILES,
  HDF5 FILES, AND VARIOUS CAMERAS) AND TO VIDEO FILES

COPYRIGHT (C)

  FRAUNHOFER FIT-LIFE
  SCHLOSS BIRLINGHOVEN
  53754 ST. AUGUSTIN
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  Fraunhofer-Institut für Angewandte Informationstechnik FIT 
  http://fit.fraunhofer.de

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
// arch-tag: dc0a48cf-c241-4480-b722-6e181cbd9fca 
/* 
* (C) Copyright Neal D. Becker (2004) 
* Permission to copy, use, modify, sell and distribute this software 
* is granted provided this copyright notice appears in all copies. 
* This software is provided "as is" without express or implied 
* warranty, and with no claim as to its suitability for any purpose. 
*/ 

#ifndef strided_iterator_hpp 
#define strided_iterator_hpp 

#include <boost/iterator/iterator_adaptor.hpp> 
#include <iterator> 

namespace boost { 

template<typename BaseIterator> 
class strided_iterator : public boost::iterator_adaptor< 
strided_iterator<BaseIterator>, 
BaseIterator> { 


friend class iterator_core_access; 

public: 

typedef typename boost::iterator_adaptor<strided_iterator<BaseIterator>,BaseIterator> super_t; 
typedef typename std::iterator_traits<BaseIterator>::difference_type difference_type; 

strided_iterator() {} 

explicit strided_iterator (BaseIterator _base, size_t _stride) : 
super_t (_base), stride (_stride) {} 

void increment() 
{ this->base_reference() += stride; } 

void decrement() 
{ this->base_reference() -= stride; } 

void advance(difference_type n) 
{ this->base_reference() += n*stride; } 

difference_type 
distance_to(strided_iterator<BaseIterator> const& y) const 
{ return (y.base_reference()-this->base_reference())/stride; } 


private: 

const int stride; 
}; 

template<typename BaseIterator> 
strided_iterator<BaseIterator> make_strided_iterator(BaseIterator const& begin, int stride) { 
return strided_iterator<BaseIterator> (begin, stride); 
} 



} // namespace boost 

#endif 
