#include <iostream>
#include <boost/mpl/next_prior.hpp>
#include <boost/proto/proto.hpp>

namespace mpl = boost::mpl;
namespace proto = boost::proto;
namespace fusion = boost::fusion;
using proto::_;

// The array implementation
template<typename T, typename Dim>
struct array_
{
   typedef T value_type;
   typedef Dim dimension_type;

   friend std::ostream &operator <<(std::ostream &sout, array_)
   {
       return sout << "array<" << typeid(T).name() << "," << Dim::value << ">";
   }
};

// array<int, 1> is a 1-D array
// array<int, 2> is a 2-D array
// array<int, 3> is a 3-D array
template<typename T, int Dim>
struct array
 : proto::extends<
       typename proto::terminal<array_<T, mpl::int_<Dim> > >::type
     , array<T, Dim>
   >
{
   array() {}
};

template<typename Dim>
struct index_
{
   typedef Dim dimension_type;
   friend std::ostream &operator <<(std::ostream &sout, index_)
   {
       return sout << "index_<" << Dim::value << ">";
   }
};

template<typename T>
struct dimension_of
{
   typedef typename T::dimension_type type;
};

proto::terminal<index_<mpl::int_<1> > >::type const i = {{}};
proto::terminal<index_<mpl::int_<2> > >::type const j = {{}};
proto::terminal<index_<mpl::int_<3> > >::type const k = {{}};

// The grammar for valid tensor expressions, with transforms
// that calculate the dimension of the expression
struct dimension
 : proto::or_<
       proto::when<
           proto::terminal<array_<_, _> >
         , dimension_of<proto::_value>()
       >
     , proto::when<
           proto::terminal<_>
         , mpl::int_<0>()
       >
     , proto::when<
           proto::subscript<dimension, _>
         , mpl::prior<dimension(proto::_left)>()
       >
     , proto::when<
           proto::unary_expr<_, dimension>
         , dimension(proto::_child)
       >
     , proto::when<
           proto::and_<
               proto::binary_expr<_, dimension, dimension>
             , proto::if_<mpl::equal_to<dimension(proto::_left), dimension(proto::_right)>()>
           >
         , dimension(proto::_left)
       >
   >
{};

struct sum_
{
   friend std::ostream &operator <<(std::ostream &sout, sum_)
   {
       return sout << "sum_";
   }
};

proto::terminal<sum_>::type const sum = {{}};

template<typename Expr>
void test_expr(Expr const &expr)
{
   proto::display_expr(expr);
   BOOST_MPL_ASSERT((proto::matches<Expr, dimension>));
   std::cout << "dimension = " << dimension()(expr) << std::endl;
}

int main()
{
   array<int, 2> a; // a 2-D array
   array<int, 1> b; // 1-D arrays ...
   array<int, 1> c;
   array<int, 3> d; // a 3-D array

   test_expr(
       a[i][j] = b[i] * ( c[j] - sum( (d[i][j][k], k = (i-1,1)) ) )
   );
}