/*

GPUDATA

  THIS MODULE PROVIDES MULTI-DIMENSIONAL MATRIX STORAGE 
  AND OPERATIONS ON GRAPHICAL PROCESSING UNITS (GPU)
  BY MAKING USE OF CUDA TECHNOLOGY.

COPYRIGHT (C)

  ULM UNIVERSITY 
  INSTITUTE OF NEURAL INFORMATION PROCESSING (NEUROINFORMATIK)
  89069 ULM
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  INSTITUTE OF NEURAL INFORMATION PROCESSING:
  http://www.uni-ulm.de/en/in/institute-of-neural-information-processing.html

  ULM UNIVERSITY 
  HTTP://WWW.UNI-ULM.DE/

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
#include <cuda.h>
#include "cutil.h"

__global__ void dFillSpecial4D( const int ix , const int sx,
                                    const int iy , const int sy,
                                    const int iu , const int su,
                                    const int iv , const int sv,  float* dData,
                                    const float val)
{
        // Block index
        const int bIndexX = blockIdx.x;
        const int bIndexY = blockIdx.y;

        // no of threads
        const int noOfThreadsX = blockDim.x;
        const int noOfThreadsY = blockDim.y;

        // Thread index
        const int tIndexX = threadIdx.x;
        const int tIndexY = threadIdx.y;

        // index Offset
        const int x  = (bIndexX* noOfThreadsX) + tIndexX; // origin
        const int y  = (bIndexY* noOfThreadsY) + tIndexY;  // origin
        const int dxy= (y* iy) + (x* ix); // destination

        if (x<sx && y<sy) {
			int dd = dxy;
			int u,v;
			for (v=0; v<sv; v++) {
				int d = dd;
				for (u=0; u<su; u++) {

							dData[d] = val;

							d +=  iu;
				}
				dd +=  iv;
			}
        }

}

extern "C" void doFillSpecial4D(const int dix , const int dsx,
			                const int diy , const int dsy,
			                const int diu , const int dsu,
			                const int div , const int dsv,  float* dData,
							const float val)
{
        dim3 block(16,8);
        dim3 grid(iDivUp(dsx,block.x),iDivUp(dsy,block.y));
        dFillSpecial4D<<<grid,block>>>(dix , dsx, diy , dsy, diu , dsu, div ,   dsv,  dData, val);
}
