/*

GPUDATA

  THIS MODULE PROVIDES MULTI-DIMENSIONAL MATRIX STORAGE 
  AND OPERATIONS ON GRAPHICAL PROCESSING UNITS (GPU)
  BY MAKING USE OF CUDA TECHNOLOGY.

COPYRIGHT (C)

  ULM UNIVERSITY 
  INSTITUTE OF NEURAL INFORMATION PROCESSING (NEUROINFORMATIK)
  89069 ULM
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  INSTITUTE OF NEURAL INFORMATION PROCESSING:
  http://www.uni-ulm.de/en/in/institute-of-neural-information-processing.html

  ULM UNIVERSITY 
  HTTP://WWW.UNI-ULM.DE/

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
/*
 * local_contigous_array.cu
 *
 *  Created on: 04-Mar-2009
 *      Author: feisenkeil
 */

#include <cuda.h>
//#include <iostream>

inline int iDivUp(int a, int b){
  return (a % b != 0) ? (a / b + 1) : (a / b);
}

#define KERNEL_CONT(name,func)  													\
__global__ void name(float *d_in, int N, float *d_out){								\
const int bx = (blockIdx.x * blockDim.x); 											\
const int i  = (bx + threadIdx.x); 													\
	if (i<N) {																		\
		d_out[i] = func(d_in[i]);													\
	}																				\
}

#define KERNEL_1D(name,func)  														\
__global__ void name(float *d_in, int inc1, int N, float *d_out){					\
const int bx = (blockIdx.x * blockDim.x); 											\
const int i1  = (bx + threadIdx.x); 												\
const int i  = 	i1*inc1; 															\
	if (i1<N) {																		\
		d_out[i]=func(d_in[i1]);													\
	}																				\
}

#define KERNEL_2D(name,func)  														\
__global__ void name(float *d_in, int inc1, int inc2, int N1, int N2, float *d_out){\
const int bx = (blockIdx.x * blockDim.x); 											\
const int by = (blockIdx.y * blockDim.y); 											\
const int i1  = (bx + threadIdx.x); 												\
const int i2  = (by + threadIdx.y); 												\
const int i_out  = 	i1*inc1+i2*inc2; 												\
const int i = i1*N2+i2;																\
	if (i1<N1 && i2<N2) {															\
		d_out[i_out] = func(d_in[i]);												\
	}																				\
}

#define KERNEL_3D(name,func)  														\
__global__ void name(float *d_in,int inc1, int inc2, int inc3, int N1, int N2, int N3, float *d_out){	\
const int bx = (blockIdx.x * blockDim.x); 											\
const int by = (blockIdx.y * blockDim.y); 											\
const int i1  = (bx + threadIdx.x); 												\
const int i2  = (by + threadIdx.y); 												\
int i_out = i1*inc1+i2*inc2;	 													\
int i = i1+N1*i2;																	\
if (i1<N1 && i2<N2)																	\
for (int i3=0; i3<N3; i3++){														\
	d_out[i_out] = func(d_in[i+i3*(N1*N2)]);										\
	i_out += inc3;																	\
}																					\
}

// BUGGY!
#define KERNEL_4D(name,func)  														\
__global__ void name(float *d_in,int inc1, int inc2, int inc3, int inc4, int N1, int N2, int N3, int N4, float *d_out){ \
const int bx = (blockIdx.x * blockDim.x);											\
const int by = (blockIdx.y * blockDim.y);											\
const int i1  = (bx + threadIdx.x);													\
const int i2  = (by + threadIdx.y);													\
int i_out = i1*inc1+i2*inc2;															\
int i = i1+i2*N1;																\
if (i1<N1 && i2<N2){																\
	int index1 = i_out;																	\
	for (int i4=0; i4<N4; i4++){													\
		int index2 = index1;														\
		for(int i3=0; i3<N3; i3++){													\
			d_out[index2] = func(d_in[i]);										\
			i += N1*N2;															\
			index2 += inc3;															\
		}																		\
		index1 += inc4;																\
	}																			\
}																				\
}

#define KERNEL_ALL(name,function)        \
KERNEL_CONT(name##_cont,function)       \
KERNEL_1D(name##_1d,function)           \
KERNEL_2D(name##_2d,function)           \
KERNEL_3D(name##_3d,function)           \
KERNEL_4D(name##_4d,function)

#define EXTERN_C(name, kernelname)																			\
extern "C" void name(float* data, int inc1, int inc2, int inc3, int inc4, int N1, int N2, int N3, int N4, float* return_data) {\
	dim3 dimBlock2d(16*2,16);																				\
	dim3 dimGrid2d (iDivUp(N1,dimBlock2d.x),iDivUp(N2,dimBlock2d.y));										\
	dim3 dimBlock1d(512);																					\
	dim3 dimGrid1d(iDivUp(N1,dimBlock1d.x),1);																\
	/* Decision which kernel is used by size of strides	*/													\
	if(N2==1 && N3==1 && N4==1 && inc1==1){																	\
		kernelname##_cont<<<dimGrid1d, dimBlock1d>>>(return_data, N1, data);								\
	} else if(N2==1 && N3==1 && N4==1){																		\
		kernelname##_1d<<<dimGrid1d, dimBlock1d>>>(return_data, inc1, N1, data);							\
	} else if(N3==1 && N4==1) {																				\
		kernelname##_2d<<<dimGrid2d, dimBlock2d>>>(return_data, inc1, inc2, N1, N2, data);					\
	} else if(N4==1){																						\
		kernelname##_3d<<<dimGrid2d, dimBlock2d>>>(return_data, inc1, inc2, inc3, N1, N2, N3, data);		\
	} else {																								\
		kernelname##_4d<<<dimGrid2d, dimBlock2d>>>(return_data, inc1, inc2, inc3, 1, N1, N2, N3, N4, data);	\
	}																										\
}


KERNEL_ALL(dcos,cosf);
KERNEL_ALL(dsin,sinf);
KERNEL_ALL(dacos,acosf);
KERNEL_ALL(dasin,asinf);
KERNEL_ALL(dexp,expf);
KERNEL_ALL(dexp2,exp2f);
KERNEL_ALL(dexp10,exp10f);
KERNEL_ALL(dsqrt,sqrtf);
KERNEL_ALL(dlog,logf);
KERNEL_ALL(dlog2,log2f);
KERNEL_ALL(dabs,fabsf);
KERNEL_ALL(dround,roundf);
KERNEL_ALL(drintf,rintf);
KERNEL_ALL(disnan,isnan);

EXTERN_C(doLocal_cos_ret,dcos);
EXTERN_C(doLocal_sin_ret,dsin);
EXTERN_C(doLocal_acos_ret,dacos);
EXTERN_C(doLocal_asin_ret,dasin);
EXTERN_C(doLocal_exp_ret,dexp);
EXTERN_C(doLocal_exp2_ret,dexp2);
EXTERN_C(doLocal_exp10_ret,dexp10);
EXTERN_C(doLocal_sqrt_ret,dsqrt);
EXTERN_C(doLocal_log_ret,dlog);
EXTERN_C(doLocal_log2_ret,dlog2);
EXTERN_C(doLocal_abs_ret,dabs);
EXTERN_C(doLocal_round_ret,dround);
EXTERN_C(doLocal_drintf_ret,drintf);
EXTERN_C(doLocal_sinan_ret,disnan);
