/*

GPUDATA

  THIS MODULE PROVIDES MULTI-DIMENSIONAL MATRIX STORAGE 
  AND OPERATIONS ON GRAPHICAL PROCESSING UNITS (GPU)
  BY MAKING USE OF CUDA TECHNOLOGY.

COPYRIGHT (C)

  ULM UNIVERSITY 
  INSTITUTE OF NEURAL INFORMATION PROCESSING (NEUROINFORMATIK)
  89069 ULM
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  INSTITUTE OF NEURAL INFORMATION PROCESSING:
  http://www.uni-ulm.de/en/in/institute-of-neural-information-processing.html

  ULM UNIVERSITY 
  HTTP://WWW.UNI-ULM.DE/

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
#include <cuda.h>
#include "cutil.h"

__global__ void medianOnBinary(float* res, const float* src, int range, int sizeL) {

	// grid,block

	// Block index
	const int bIndex = blockIdx.x+blockIdx.y*gridDim.x;

	// number of threads per block
	const int noOfThreads = blockDim.x;

	// Thread index
	const int tIndex = threadIdx.x;

	// index Offset
	const int i = ((bIndex * noOfThreads) + tIndex);

	if (i<sizeL) {
		// only do inversion here for a test
		res[i]=1-src[i];
	}

	// traverse spatial neighborhood up to range L


}

__global__ void medianOnBinary2D(float* res, const float* src, int range_x, int range_y, int size_x, int size_y) {

	const int pix_idx_x = blockIdx.x*blockDim.x+threadIdx.x;
	const int pix_idx_y = blockIdx.y*blockDim.y+threadIdx.y;


	float acu = 0;
	for (int y=-range_y; y<=range_y; ++y) {

		if (pix_idx_y+y<size_y && pix_idx_y+y>=0) 
    {

			for (int x=-range_x; x<=range_x; ++x) 
      {

				if (pix_idx_x+x<size_x && pix_idx_x+x>=0) 
        {

					acu += (src[(pix_idx_y+y)*size_x+(pix_idx_x+x)]>0.5f) ? 1.0f : -1.0f;

				}
			}

		}

	}
	res[pix_idx_y*size_x+pix_idx_x] = acu>0 ? 1.0f : 0.0f;
}
extern "C" {

void do_medianOnBinary(float* res, const float* src, int range, int sizeL) {
	dim3 dimBlock(128);
	int s  = iDivUp(sizeL,dimBlock.x);
	int s2 = 1;
	while (s>=65535) {
		s2 *= 2;
		s   = iDivUp(s,2);
	}
	dim3 dimGrid(s,s2);
	medianOnBinary<<<dimGrid, dimBlock>>>(res, src, range, sizeL);
}

void do_medianOnBinary2D(float* res, const float* src, int range_x, int range_y, int size_x, int size_y) {
	dim3 dimBlock(8,4);
	dim3 dimGrid(iDivUp(size_x,dimBlock.x),iDivUp(size_y,dimBlock.y));

	medianOnBinary2D<<<dimGrid, dimBlock>>>(res, src, range_x, range_y, size_x, size_y);
}

}
