/*

GPUDATA

  THIS MODULE PROVIDES MULTI-DIMENSIONAL MATRIX STORAGE 
  AND OPERATIONS ON GRAPHICAL PROCESSING UNITS (GPU)
  BY MAKING USE OF CUDA TECHNOLOGY.

COPYRIGHT (C)

  ULM UNIVERSITY 
  INSTITUTE OF NEURAL INFORMATION PROCESSING (NEUROINFORMATIK)
  89069 ULM
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  INSTITUTE OF NEURAL INFORMATION PROCESSING:
  http://www.uni-ulm.de/en/in/institute-of-neural-information-processing.html

  ULM UNIVERSITY 
  HTTP://WWW.UNI-ULM.DE/

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
#ifndef GPUDATABASE_H_2QMUKB9T
#define GPUDATABASE_H_2QMUKB9T

#include <cstddef>     // For size_t
#include <stdio.h>

//class cudaArray;

struct GPUDataBase
{
  /// rank of the array
  size_t ndim;
  /// extents of the array dimensions
  size_t * dims;
  /// strides along the array dimensions
  int* strides;
  /// buffered tag indicating whether there are gaps in the data storage
  bool contiguous;
  /**
   * Pointer to the beginning of the data.
   * It can be different of \c inner-data if you want
   * to start from a given offset in the allocated memory.
   */
  float* _data;
  /**
   * Index to the beginning of the data in constant memory.
   * This should be -1 if no const copy of this data exists.
   */
  int constData;
  /**
   *  Pointer to data in Array (texture memory)
   * */
  //cudaArray* _array;


  /// directly get a given extent, and give implicit values for extents out of range
  unsigned extent( size_t i ) const { return i < ndim ? dims[i] : 1; }
  /// directly get a given stride, and give implicit values for strides out of range
  int stride( size_t i ) const { return i < ndim ? strides[i] : 1; }
  /** Return the total number of elements in the array. */
  size_t size() const
  {
    size_t n = 1;
    for ( size_t i = 0; i < ndim; ++i )
      n *= dims[i];
    return n;
  }
  /** Return all the dimensions of the array. */
  const size_t* getDimensions() const  { return dims; }
  /** Return all the dimensions of the array. */
  const size_t* getSizes() const  { return dims; }
  /** Return all the srides of the array. */
  const int* getStrides() const  { return strides; }
  /// get the data pointer usable for cuda algorithms
  float* getDataPointer() const { return _data; }
  /// get the data pointer usable for cuda algorithms
  float* data() const { return _data; }
  float* getPtr() const { return _data; }
};

#endif /* end of include guard: GPUDATABASE_H_2QMUKB9T */
