/*

GPUDATA

  THIS MODULE PROVIDES MULTI-DIMENSIONAL MATRIX STORAGE 
  AND OPERATIONS ON GRAPHICAL PROCESSING UNITS (GPU)
  BY MAKING USE OF CUDA TECHNOLOGY.

COPYRIGHT (C)

  ULM UNIVERSITY 
  INSTITUTE OF NEURAL INFORMATION PROCESSING (NEUROINFORMATIK)
  89069 ULM
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  INSTITUTE OF NEURAL INFORMATION PROCESSING:
  http://www.uni-ulm.de/en/in/institute-of-neural-information-processing.html

  ULM UNIVERSITY 
  HTTP://WWW.UNI-ULM.DE/

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/

#include "GPUTextureData1D.hpp"
#include "cutil.h"
#include <cuda_runtime.h>
#include <CPUData.hpp>
#include <GPUData.h>



GPUTextureData1D::GPUTextureData1D() {
	innerData = new InnerTexture1DData();
}

GPUTextureData1D::GPUTextureData1D(int size) {
	innerData = new InnerTexture1DData(size);
}

GPUTextureData1D::GPUTextureData1D(const GPUTextureData1D& d) {
	innerData=0;
//	std::cout << "use copy constructor\n";
//	std::cout << "   copied inner data has size" << d.innerData->size << "\n";
//	std::cout << "   copied inner data has counter " << d.innerData->refCount<< "\n";
	clear();
	this->innerData = d.innerData;
	if (this->innerData!=0) this->innerData->increaseCounter();
	std::cout << "   copied inner data has counter " << d.innerData->refCount<< "\n";
	std::cout.flush();
}

void GPUTextureData1D::clear() {
	if (innerData!=0) {
		if (innerData->decreaseCounter()==0) {
			delete innerData;
			innerData=0;
		}
	}
}

GPUTextureData1D::~GPUTextureData1D() {
	clear();
}

GPUTextureData1D& GPUTextureData1D::operator=(const GPUTextureData1D& d) {
//	std::cout << "use = operator\n" << std::endl;
	if (this==&d) return *this;
	clear();
	this->innerData = d.innerData;
	if (this->innerData!=0) this->innerData->increaseCounter();
	return *this;
}

int GPUTextureData1D::getSize() {
	if (innerData!=0) return innerData->size;
	else return 0;
}

void GPUTextureData1D::fill(const float val){
	if(this->innerData!=0){
		std::cout << " write value " << std::endl;
		this->innerData->fill(val);
	}
}

GPUData GPUTextureData1D::read(){
	GPUData dat(this->getSize());
	if(this->innerData!=0){
//		std::cout << " get texture array" << std::endl;
		this->innerData->read(dat.getDataPointer());
	}
	return dat;
}

void GPUTextureData1D::bindTexture(){
	if(this->innerData!=0){
		this->innerData->bindTexture();
	}
}
void GPUTextureData1D::writeGPUData(const GPUData val){
	if(this->innerData!=0){
		this->innerData->writeGPUData(val.getDataPointer(), val.size());
	}
}
