/*

GPUDATA

  THIS MODULE PROVIDES MULTI-DIMENSIONAL MATRIX STORAGE 
  AND OPERATIONS ON GRAPHICAL PROCESSING UNITS (GPU)
  BY MAKING USE OF CUDA TECHNOLOGY.

COPYRIGHT (C)

  ULM UNIVERSITY 
  INSTITUTE OF NEURAL INFORMATION PROCESSING (NEUROINFORMATIK)
  89069 ULM
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  INSTITUTE OF NEURAL INFORMATION PROCESSING:
  http://www.uni-ulm.de/en/in/institute-of-neural-information-processing.html

  ULM UNIVERSITY 
  HTTP://WWW.UNI-ULM.DE/

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
#ifndef GPUTEXTUREDATA2D_HPP_
#define GPUTEXTUREDATA2D_HPP_

#include <GPUDataBase.h>
#include <cutil.h>

struct InnerTexture2DData {
	InnerTexture2DData();
	InnerTexture2DData(int height, int width);
	~InnerTexture2DData();
	cudaArray *cuArray;
	cudaChannelFormatDesc channelDesc;
	int size;
	size_t height;
	size_t width;
	int refCount;
	int increaseCounter();
	int decreaseCounter();
	void fill(const float val);
	void read(float* d_data);
	void bindTexture( cudaTextureFilterMode filtermode = cudaFilterModePoint ) const;
	void bindTexture2() const;
	void writeGPUData(const float* val, const size_t* size);
} ;

struct lens_distortion_params
{
  float x_c, y_c; // principal point
  float f_x, f_y; // focal lengths
  float K1, K2, K3;
  float P1, P2; //distortion
  float alpha_c; // skew
};

class GPUTextureData2D {
public:
  
  GPUTextureData2D();
  GPUTextureData2D(int width, int height);
  GPUTextureData2D( const GPUTextureData2D & d );
  GPUTextureData2D( const GPUDataBase & d      );
  
  GPUTextureData2D& operator=(const GPUTextureData2D& d);
  void reference(const GPUTextureData2D& d);
  
  virtual ~GPUTextureData2D();
  
  int getSize();
  void fill(const float val);
  //GPUData read(void);
  void bindTexture( void) const;
  void writeGPUData(const GPUDataBase& val);
  
  size_t width() const { return _innerData ? _innerData->width : 0; }
  size_t height() const { return _innerData ? _innerData->height : 0; }
  
  void conv2D(const GPUDataBase& kernel, GPUDataBase& dest);
  void conv2D(const GPUTextureData2D& kernel, GPUDataBase& dest);
  void conv2D3(const GPUTextureData2D& kernel, int depth, GPUDataBase& dest);
  void affineTransform( const GPUDataBase & target, const float A[3][3] ) const;
  void lensDistort( const GPUDataBase & target, const lens_distortion_params P ) const;
  void scale( const GPUDataBase & target, float xscale, float yscale ) const;

protected:
  
  void clear();
  
  InnerTexture2DData* _innerData;
  
};

#endif /* GPUTEXTUREDATA1D_HPP_ */
