/*

GPUDATA

  THIS MODULE PROVIDES MULTI-DIMENSIONAL MATRIX STORAGE 
  AND OPERATIONS ON GRAPHICAL PROCESSING UNITS (GPU)
  BY MAKING USE OF CUDA TECHNOLOGY.

COPYRIGHT (C)

  ULM UNIVERSITY 
  INSTITUTE OF NEURAL INFORMATION PROCESSING (NEUROINFORMATIK)
  89069 ULM
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  INSTITUTE OF NEURAL INFORMATION PROCESSING:
  http://www.uni-ulm.de/en/in/institute-of-neural-information-processing.html

  ULM UNIVERSITY 
  HTTP://WWW.UNI-ULM.DE/

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
#include "cuda_context_handler.hpp"
#include <boost/any.hpp>
#include <boost/shared_ptr.hpp>

boost::thread_specific_ptr<cuda_context_handler> cuda_context_handler::_tls_instance;

typedef std::list<cuda_context_specific_resource_base*> cuda_context_specific_resource_list;
cuda_context_specific_ptr<cuda_context_specific_resource_list> cuda_context_specific_resources;

// resources are pushed and popped to and from a stack, i.e. first in, last out

cuda_context_specific_resource_base::~cuda_context_specific_resource_base() {}

void delete_cuda_context_sepcific_resources()
{
  cuda_context_specific_resource_list * rlist = cuda_context_specific_resources.get();
  if ( rlist )
	{
	  // todo: make list unique w.r.t. the contained pointers first...
	  while ( !rlist->empty() )
		{
		  delete rlist->back();
		  rlist->pop_back();
		}
	  // this is delicate: we remove the list itself from management and then delete it
	  // otherwise the removal would reference the list...
	  cuda_context_specific_resources.set( 0 );
	  delete rlist;
	}
}

void remove_cuda_context_specific_resource( void * p )
{
  cuda_context_specific_resource_list * rlist = cuda_context_specific_resources.get();
  if ( rlist )
	{  
	  for ( cuda_context_specific_resource_list::iterator i = rlist->begin(), e = rlist->end();
			i != e; ++i )
		{
		  if ( (*i)->get_ptr() == p )
			{
			  rlist->erase( i );
			  // important: we can only do this once
			  // otherwise: use remove_if...
			  break;
			}
		}
	}
}

void add_cuda_context_specific_resource_internal( cuda_context_specific_resource_base * p )
{
  cuda_context_specific_resource_list * rlist = cuda_context_specific_resources.get();
  if ( rlist == 0 )
	{
	  // we dont want the list itself to be managed as it is part of the management itself
	  cuda_context_specific_resources.set( rlist = new cuda_context_specific_resource_list(), false );
	}
  rlist->push_back( p );
}

  
