/*

GPUDATA

  THIS MODULE PROVIDES MULTI-DIMENSIONAL MATRIX STORAGE 
  AND OPERATIONS ON GRAPHICAL PROCESSING UNITS (GPU)
  BY MAKING USE OF CUDA TECHNOLOGY.

COPYRIGHT (C)

  ULM UNIVERSITY 
  INSTITUTE OF NEURAL INFORMATION PROCESSING (NEUROINFORMATIK)
  89069 ULM
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  INSTITUTE OF NEURAL INFORMATION PROCESSING:
  http://www.uni-ulm.de/en/in/institute-of-neural-information-processing.html

  ULM UNIVERSITY 
  HTTP://WWW.UNI-ULM.DE/

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
#ifndef CUFFTKERNELKEY_H
#define CUFFTKERNELKEY_H
#include <boost/unordered_map.hpp>
#include <string>
extern "C" 
{
	#include <cufft.h>
}
struct plan_key;
#include "GPUData.h"

/**
 * a key which contains all cufft plan parameters so that it can be stored
 */
struct plan_key {
	plan_key();
	~plan_key();
	plan_key(const plan_key& k);
	plan_key(cufftType_t type_, int dim_, int* dims_);
	plan_key(cufftType_t type_, int z, int y, int x);
	plan_key(cufftType_t type_, int h, int w);
	plan_key(cufftType_t type_, int size);

	cufftType_t type;
	int dim;
	int* dims;

	/**
	 * factory method
	 */
	cufftHandle* create_plan();

	void destroy_plan(cufftHandle* p);

	bool operator==(const plan_key& k) const;

	plan_key& operator=(const plan_key& k);

    std::string dump() const;

};

size_t hash_value(const plan_key& k);



#endif /*CUFFTKERNELKEY_H*/
