/*

GPUDATA

  THIS MODULE PROVIDES MULTI-DIMENSIONAL MATRIX STORAGE 
  AND OPERATIONS ON GRAPHICAL PROCESSING UNITS (GPU)
  BY MAKING USE OF CUDA TECHNOLOGY.

COPYRIGHT (C)

  ULM UNIVERSITY 
  INSTITUTE OF NEURAL INFORMATION PROCESSING (NEUROINFORMATIK)
  89069 ULM
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  INSTITUTE OF NEURAL INFORMATION PROCESSING:
  http://www.uni-ulm.de/en/in/institute-of-neural-information-processing.html

  ULM UNIVERSITY 
  HTTP://WWW.UNI-ULM.DE/

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
#ifndef CUTIL_H_
#define CUTIL_H_

#include <cuda.h>
#include <cuda_runtime_api.h>

//#ifdef __cplusplus
//extern "C" {
//#endif

  extern void set_gpu_block_size( int n );

  extern int GPUData_max_block_dim_size;
  extern int GPUData_max_grid_dim_size;

  extern dim3 GPUData_dimBlock1d;
  extern dim3 GPUData_dimBlock2d;
  extern dim3 GPUData_dimBlock3d;

  inline int iDivUp(int a, int b)
  {
	return (a % b != 0) ? (a / b + 1) : (a / b);
  }

  //Round a / b to nearest lower integer value
  inline int iDivDown(int a, int b)
  {
    return a / b;
  }

  //Align a to nearest higher multiple of b
  inline int iAlignUp(int a, int b)
  {
    return (a % b != 0) ?  (a - a % b + b) : a;
  }

  //Align a to nearest lower multiple of b
  inline int iAlignDown(int a, int b)
  {
    return a - a % b;
  }

  inline int ggt(int a, int b) {
	if (a<b) {
	  const int temp = a;
	  a = b;
	  b = temp;
	}
	int rest = a % b;
	while (rest!=0) {
	  a = b;
	  b = rest;
	  rest = a % b;
	}
	return b;
  }

  inline dim3 make_2d_gridDim_for_1d_blockDim(int block_dim_x, int n) {
	int sx = iDivUp(n,block_dim_x);
	int sy = 1;
	if (sx<=GPUData_max_grid_dim_size)
	  return dim3(sx,sy);
	int t = ggt(sx,GPUData_max_grid_dim_size);
	while (sx/t>GPUData_max_grid_dim_size) {
	  t*=2;
	}
	sx = iDivUp(sx,t);
	sy = t;
	return dim3(sx,sy);
  }

//#ifdef __cplusplus
//}
//#endif

#endif  // #ifndef CUTIL_H_
