/*

GPUDATA

  THIS MODULE PROVIDES MULTI-DIMENSIONAL MATRIX STORAGE 
  AND OPERATIONS ON GRAPHICAL PROCESSING UNITS (GPU)
  BY MAKING USE OF CUDA TECHNOLOGY.

COPYRIGHT (C)

  ULM UNIVERSITY 
  INSTITUTE OF NEURAL INFORMATION PROCESSING (NEUROINFORMATIK)
  89069 ULM
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  INSTITUTE OF NEURAL INFORMATION PROCESSING:
  http://www.uni-ulm.de/en/in/institute-of-neural-information-processing.html

  ULM UNIVERSITY 
  HTTP://WWW.UNI-ULM.DE/

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
//
// cpu median3x3 implementation for reference purposes
//
#include "median3x3_cpu.hpp"
#include <cstdlib>


inline void
compare_tuple (float *r0, float *r1, float f1, float f2)
{
	if (f1 < f2) {
		*r0 = f2;
		*r1 = f1;
	}
	else {
		*r0 = f1;
		*r1 = f2;
	}
}


inline void
compare_triple (float *r0, float *r1, float *r2, 
                float f1, float f2, float f3)
{

	float c10, c11;
	float c20, c21;
	float c30, c31;

	compare_tuple(&c10, &c11, 
	              f2, f3);

	compare_tuple(&c20, &c21, 
	              f1, c11);

	compare_tuple(&c30, &c31, 
	              c20, c10);

	*r0 = c21;
	*r1 = c31;
	*r2 = c30;
}


inline float
get_median (float f1, float f2, float f3, 
            float f4, float f5, float f6, 
            float f7, float f8, float f9)
{
	float tmp1, tmp2;
	float t10, t11, t12;
	float t20, t21, t22;
	float t30, t31, t32;
	float d0, d1;

	compare_triple(&t10, &t11, &t12, 
			       f1, f2, f3);

	compare_triple(&t20, &t21, &t22,
	               f4, f5, f6);

	compare_triple(&t30, &t31, &t32, 
	               f7, f8, f9);


	compare_tuple(&d0, &d1, 
	              t10, t20);
	tmp1 = d0;
	compare_tuple(&d0, &d1, 
	              tmp1, t30);
	tmp1 = d0;

	compare_tuple(&d0, &d1, 
	              t22, t32);
	tmp2 = d1;
	compare_tuple(&d0, &d1, 
	              t12, tmp2);
	tmp2 = d1;

	compare_triple(&t10, &t11, &t12, 
	               t11, t21, t31);
	compare_triple(&t10, &t11, &t12,
	               tmp1, t11, tmp2);
	return t11;
}


void 
median3x3_cpu (float *res, float *src, int width, int height)
{	
	bool isFirstRow;
	bool isLeftColumn;
	bool isRightColumn;
	bool isLastRow;
	long int count = width * height;

	// edge-handling: border expansion
	for (int i = 0; i < count; i++) {

		// determine where the calculation is at
		isFirstRow = (i - width) < 0;
		isLeftColumn = (i % width) == 0;
		isRightColumn = ((i+1) % width) == 0;
		isLastRow = (count - i - 1) < width;

		//
		// coordinate 'calculation'. filter points are located at:
		//
		//   f1 f2 f3
		//   f4 f5 f6
		//   f7 f8 f9
		//
		// so f5 is the point the current operation is centered on.
		//
		float f1, f2, f3, f4, f5, f6, f7, f8, f9;
	
		// f5 can be taken easily, as this point should always be there.
		// otherwise the caller messed up something.
		f5 = src[i];

		f4 = isLeftColumn ? f5 : src[i - 1];
		f6 = isRightColumn ? f5 : src[i + 1];

		// calculation of f1-f3
		if (isFirstRow) {
			f2 = f5;
			f1 = isLeftColumn ? f5 : f4;
			f3 = isRightColumn ? f5 : f6;
		}
		else {
			f2 = src[i-width];
			f1 = isLeftColumn ? f2 : src[i - width - 1];
			f3 = isRightColumn ? f2 : src[i - width + 1];
		}

		// calculation of f7-f9
		if (isLastRow) {
			f8 = f5;
			f7 = isLeftColumn ? f5 : f4;
			f9 = isRightColumn ? f5 : f6;
		}
		else {
			f8 = src[i+width];
			f7 = isLeftColumn ? f8 : src[i + width - 1];
			f9 = isRightColumn ? f8 : src[i + width + 1];
		}

		res[i] = get_median(f1, f2, f3, f4, f5, f6, f7, f8, f9);
	}
}
