/*

GPUDATA

  THIS MODULE PROVIDES MULTI-DIMENSIONAL MATRIX STORAGE 
  AND OPERATIONS ON GRAPHICAL PROCESSING UNITS (GPU)
  BY MAKING USE OF CUDA TECHNOLOGY.

COPYRIGHT (C)

  ULM UNIVERSITY 
  INSTITUTE OF NEURAL INFORMATION PROCESSING (NEUROINFORMATIK)
  89069 ULM
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  INSTITUTE OF NEURAL INFORMATION PROCESSING:
  http://www.uni-ulm.de/en/in/institute-of-neural-information-processing.html

  ULM UNIVERSITY 
  HTTP://WWW.UNI-ULM.DE/

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
//
// Just a simple PGM reader and writer. At the moment they are just for testing
// purposes and loading possibly only works with lena_noise.pgm in ASCII format.
//
#include "pgm.hpp"
#include <fstream>
#include <cstdlib>
#include <iostream>
#ifdef _WIN32
#include <string>	// added for VC2008 compatibility (getline())
#endif

using namespace std;
namespace pgm {

float *
read_pgm_file (const char *fname)
{
	string line;
	ifstream f;
	f.open(fname);
	if (!f) {
		fprintf(stderr, "Could not open data/lena.pgm");
		exit(EXIT_FAILURE);
	}
	
	// ignore first two lines
	getline(f, line); getline(f, line);

	// get width, height
	getline(f, line);
	char *pEnd;
	long int width = strtol(line.c_str(), &pEnd, 10);
	long int height = strtol(line.c_str(), &pEnd, 10);
	float *result = (float*)calloc(width*height, sizeof(float));

	// ignore next line
	getline(f, line);

	// read values
	long int i = 0;
	while (getline(f, line)) {
		result[i] = atof(line.c_str());
		i++;
	}

	f.close();
	return result;
}

void 
write_pgm_file (const char *fname, const float *data, const int width, const int height)
{
	string line;
	ofstream f;
	f.open(fname, ios::out | ios::trunc);

	f << "P2" << endl;
	f << "# created by median test application" << endl;
	// f << size[0] << " " << size[1] << endl << get_max_value(data, size[0]*size[1]) << endl;
	f << width << " " << height << endl << 255 << endl;


	for (int i = 0; i < (width*height); i++)
		f << (int)data[i] << endl;

	f.flush();
	f.close();
}

} // namespace pgm
