#include <cppunit/extensions/HelperMacros.h>
#include <cppunit/ui/text/TestRunner.h>
#include <cmath>
#include <GPUData.h>
#include <CPUData.hpp>
#include <GPUTextureData1D.hpp>
#include <GPUTextureData2D.hpp>
//#include <CudaDispatcher.hpp>

#include "GPUDataDebugging.hpp"
//#include "DeclareCudaKernel.hpp"
#include <iostream>
#include <median3x3_cpu.hpp>
#include <pgm.hpp>

void fillLinSpace(GPUData& d, double start_A, double end_A) 
{
	float* hData = new float[d.size()];
	const  size_t s = d.size();

	for (size_t i=0; i<s;++i) {
		hData[i] = float(start_A+(i*(end_A-start_A))/(s-1.0));
	}

	d.copyFromHostMemory(hData);

	delete[] hData;
}

void fillLinSpaceColMajor(GPUData& d, double start_A, double end_A, const size_t w, const size_t h) 
{
	float* hData = new float[d.size()];
	const  size_t s = d.size();

	for (size_t i=0; i<w; ++i) {
		for (size_t j=0; j<h; ++j) {
			hData[i*h+j] = float(start_A+((i+j*w)*(end_A-start_A))/(s-1.0));
		}
	}
	
	d.copyFromHostMemory(hData);

	delete[] hData;
}

/// this declares the cuda kernel and its arguments for use with CALL_CUDA_KERNEL
// CUDA_KERNEL( void, dAddScaledNumbers, float* dest, float* origin, int inc, int N, float v );

class GPUCudaTest : public CppUnit::TestFixture {
	CPPUNIT_TEST_SUITE(GPUCudaTest);
	
	CPPUNIT_TEST(testCreation);
	CPPUNIT_TEST(testDeviceExchanges);
	CPPUNIT_TEST(testDimReduce);
	CPPUNIT_TEST(testView);
	CPPUNIT_TEST(testView2);
	CPPUNIT_TEST(test5DDliceAndCopy);
	CPPUNIT_TEST(testCopyFromView1);
	CPPUNIT_TEST(testCopyFromView2);
	CPPUNIT_TEST(testLocalCos);
	CPPUNIT_TEST(testMakeUnique);
	CPPUNIT_TEST(testCopy1dSpecial);
	CPPUNIT_TEST(testSlice);
	CPPUNIT_TEST(testFill);
	CPPUNIT_TEST(testMul);
	CPPUNIT_TEST(testScale);
	CPPUNIT_TEST(testDiv);
	CPPUNIT_TEST(testSum);
	CPPUNIT_TEST(testSumOverLastDimension);
	CPPUNIT_TEST(testAdd);
	CPPUNIT_TEST(testAddRepeated);
	CPPUNIT_TEST(testMinMax);
	CPPUNIT_TEST(testMinMax2);
	CPPUNIT_TEST(testHeaviside);
	CPPUNIT_TEST(testModulo);
	CPPUNIT_TEST(testSetIfInBetween);
	CPPUNIT_TEST(testRect);
	CPPUNIT_TEST(testClamp);
	CPPUNIT_TEST(testSqr);
	CPPUNIT_TEST(testShiftDim);
	CPPUNIT_TEST(testLog);
	CPPUNIT_TEST(testPyth);
	CPPUNIT_TEST(testSqrt);
	CPPUNIT_TEST(testnakarushton2);
	CPPUNIT_TEST(testatan2);
	CPPUNIT_TEST(testsincos);
	CPPUNIT_TEST(testReduceDimensionality);
	CPPUNIT_TEST(testRemoveSingletonDimensions);
	CPPUNIT_TEST(testExtendDimensionality);
	//	CPPUNIT_TEST(testCopyGPUData2GPUData);
	CPPUNIT_TEST(testCopyToConst);
	CPPUNIT_TEST(testGeneralConvolution1Din4D);
	CPPUNIT_TEST(testDGLExplicitEuler);
	CPPUNIT_TEST(testDGLExplicitEulerColMaj);
	CPPUNIT_TEST(testDGLExplicitEuler2);
	CPPUNIT_TEST(testDGLImplicitEuler);
	CPPUNIT_TEST(testDGLImplicitEuler2);
	CPPUNIT_TEST(testDGLSteadyState);
	CPPUNIT_TEST(testDGLSteadyState2);
	CPPUNIT_TEST(testMatVecMul);
	CPPUNIT_TEST(testTexDataObject);
	
	CPPUNIT_TEST(testMedian3x3);
	//	CPPUNIT_TEST(testCudaDispatcher);
	CPPUNIT_TEST(testArray);
	CPPUNIT_TEST_SUITE_END();

public:

	/*
	void testCudaDispatcher()
	{
		size_t dims[] = {1};
		GPUData a(1,dims);
		GPUData b(1,dims);
		a.fill(1);
		b.fill(2);

		CALL_CUDA_KERNEL( dim3(16,16,1), dim3(16,16,1), dAddScaledNumbers, a.data(), b.data(), a.size(), 1.5 );

		CPPUNIT_ASSERT( a[0] == 4 );
	}
	*/

	void testCreation() {
		size_t dims[] = {12, 23};
		GPUData a(2, dims);
		CPPUNIT_ASSERT(! a.isEmpty());
		CPPUNIT_ASSERT(a.isContiguous());
	}

	void testDeviceExchanges()
	{
		// Test the 1D
		size_t dims[] = {12, 23};
		GPUData a(1, dims);
		a[3] = 567.;
		CPPUNIT_ASSERT(a[3] == 567.);

		// Test 2d
		GPUData b(2, dims);
		for ( size_t i = 0; i < dims[0]; ++i )
			for ( size_t j = 0; j < dims[1]; ++j )
				b[i][j] = i * b.stride(0) + j * b.stride(1);
		for ( size_t i = 0; i < dims[0]; ++i )
			for ( size_t j = 0; j < dims[1]; ++j )
				CPPUNIT_ASSERT( b[i][j] == i * b.stride(0) + j * b.stride(1) );

		// Test 2d via CPUData
		CPUData<float> cb;
		b.copyTo( cb );
		for ( size_t i = 0; i < dims[0]; ++i )
			for ( size_t j = 0; j < dims[1]; ++j )
				CPPUNIT_ASSERT( cb(i,j,0) == i * b.stride(0) + j * b.stride(1) );

		// from views
		/*
		int offsets[] = { 3, 3 };
		size_t sizes[] = { 5, 11 };
		GPUData bview( b.view( offsets, sizes ) );
		CPUData<float> cbview;
		bview.copyTo( cbview );
		for ( size_t i = 0; i < bview.extent(0); ++i )
			for ( size_t j = 0; j < bview.extent(1); ++j )
				CPPUNIT_ASSERT( cbview(i,j,0) == bview[i][j] );
		*/
	}

	void testLocalCos() {

		GPUData contiguous_small(145);
		fillLinSpace(contiguous_small,0.0f,1.0f);
		GPUData contiguous_large(10000);
		fillLinSpace(contiguous_large,0.0f,1.0f);
		int offs[3]  = {0};
		size_t sizes[3] = {10      ,   10, 10};
		int stride[3] = {10*22+4 , 20+2,  2};

		GPUData non_contiguous_medium(contiguous_large.view(offs,3,sizes,stride));

		contiguous_small.cos();
		contiguous_large.cos();

		CPPUNIT_ASSERT(contiguous_small[0]==cos(0.0f));
		CPPUNIT_ASSERT(fabs(contiguous_small[145/2]-cos(0.5f)) <= 4*std::numeric_limits<float>::epsilon());
		CPPUNIT_ASSERT(fabs(contiguous_small[144]-cos(1.0f)) <= 4*std::numeric_limits<float>::epsilon());

		// Testing 4D

		GPUData testdata(10,12,15,17);
		testdata.fill(1.0f);

		size_t dim[4] = {4,11,4,3};
		GPUData data(testdata.view(dim));
		data.fill(4.0f);

		data.cos();

		int count_cos = 0;
		for(int l=0; l<17; l++){
			for(int k=0; k<15; k++){
				for(int i=0; i<12; i++){
					for(int j=0; j<10; j++){
						if(fabs(data[j][i][k][l]-cos(4.0f)) <= 4*std::numeric_limits<float>::epsilon()){
							count_cos++;
						}
					}
				}
			}
		}
		CPPUNIT_ASSERT(count_cos == 3*4*11*4);

		data = data[2];
		count_cos = 0;
		for(int k=0; k<15; k++){
			for(int i=0; i<12; i++){
				for(int j=0; j<10; j++){
					if(fabs(data[j][i][k]-cos(4.0f)) <= 4*std::numeric_limits<float>::epsilon()){
						count_cos++;
					}
				}
			}
		}
		CPPUNIT_ASSERT(count_cos == 4*10*3);


		GPUData testdata2(10,12,15);
		testdata2.fill(1.0f);

		size_t dim2[3] = {4,7,8};
		GPUData data2(testdata2.view(dim2));
		data2.fill(4.0f);

		data2.cos();
		data2 = data2[1];

		count_cos = 0;
		for(int i=0; i<12; i++){
			for(int j=0; j<10; j++){
				if(fabs(data2[j][i]-cos(4.0f)) <= 4*std::numeric_limits<float>::epsilon()){
					count_cos++;
				}
			}
		}
		CPPUNIT_ASSERT(count_cos == 7*8);

	}

	void testDimReduce() {
		size_t dims[] = {12, 23};
		GPUData a(2, dims);
		a[7][11] = 123.;
		GPUData b = a[7];
		CPPUNIT_ASSERT(b[11] == 123.);
		b[10] = 567.;
		CPPUNIT_ASSERT(a[7][10] == 567.);
	}

	void testView() {
		size_t dims[] = {13, 21, 57};

		// 1D
		GPUData a(1, dims);
		a[6] = 123;

		int index[] = {3};
		size_t size[] = {8};
		GPUData b = a.view(index, size);
		CPPUNIT_ASSERT(b[3] == 123.0);

		// 3D
		GPUData d(3, dims);
		d[7][4][2] = 123.;

		int index2[] = {7, 4, 1};
		size_t size2[] = {1, 2, 2};
		GPUData e = d.view(index2, size2);
		CPPUNIT_ASSERT(e[0][0][1] == 123.);
	}

	void testView2() {
		size_t dim1DSize = 11*11*11*11;
		GPUData a(1,&dim1DSize);
		for (size_t i = 0; i<dim1DSize; ++i)
			{
				a[i] = i;
			}

		size_t dims[] = {11, 11, 11, 11};
		int inde[] = { 0 };
		int stri[] = { 11*11*11 , 11*11 , 11 , 1};
		GPUData b = a.view(inde,4,dims,stri);
		int n = 0;
		for (size_t l = 0; l<11; l++) {
		for (size_t k = 0; k<11; k++) {
		for (size_t j = 0; j<11; j++) {
		for (size_t i = 0; i<11; i++) {
			CPPUNIT_ASSERT(b[l][k][j][i] == n++);
		}
		}
		}
		}

	}

	void testCopyFromView1()
	{
		size_t dims[] = {10,14,11};
		GPUData a(3,dims);
		for ( size_t i = 0; i < dims[0]; ++i )
		for ( size_t j = 0; j < dims[1]; ++j )
		for ( size_t k = 0; k < dims[2]; ++k )
			{
				a[i][j][k] = i * a.stride(0) + j * a.stride(1) + k * a.stride(2);
			}
		size_t dims2[] = {5,7,11};
		int strides2[] = {a.stride(0)*2,a.stride(1)*2,a.stride(2)};
		int offsets2[] = {1,1,0};
		GPUData b( 3, dims2 );
		GPUData aview( a.view( offsets2, 3, dims2, strides2 ) );
		b.copyFrom(aview);
		for ( size_t i = 0; i < dims2[0]; ++i )
		for ( size_t j = 0; j < dims2[1]; ++j )
		for ( size_t k = 0; k < dims2[2]; ++k )
			{
				CPPUNIT_ASSERT( b[i][j][k] == aview[i][j][k] );
				CPPUNIT_ASSERT( b[i][j][k] == (1+i*2)*a.stride(0) + (1+j*2)*a.stride(1) + k*a.stride(2) );
			}
	}

	void testCopyFromView2()
	{
		size_t dims[] = {11,14,12};
		GPUData a(3,dims);
		for ( size_t i = 0; i < dims[0]; ++i )
		for ( size_t j = 0; j < dims[1]; ++j )
		for ( size_t k = 0; k < dims[2]; ++k )
			{
				a[i][j][k] = i * a.stride(0) + j * a.stride(1) + k * a.stride(2);
			}
		size_t dims2[] = {11,7,6};
		int strides2[] = {a.stride(0),a.stride(1)*2,a.stride(2)*2};
		int offsets2[] = {0,1,1};
		GPUData b( 3, dims2 );
		GPUData aview( a.view( offsets2, 3, dims2, strides2 ) );
		b.copyFrom(aview);
		for ( size_t i = 0; i < dims2[0]; ++i )
		for ( size_t j = 0; j < dims2[1]; ++j )
		for ( size_t k = 0; k < dims2[2]; ++k )
			{
				CPPUNIT_ASSERT( b[i][j][k] == aview[i][j][k] );
				CPPUNIT_ASSERT( b[i][j][k] == i*a.stride(0) + (1+j*2)*a.stride(1) + (1+k*2)*a.stride(2) );
			}
	}

void testMakeUnique()
  {
    size_t dims[] = {12, 23};
    GPUData b(2, dims);
    for ( size_t i = 0; i < dims[0]; ++i )
      for ( size_t j = 0; j < dims[1]; ++j )
        b[i][j] = i * b.stride(0) + j * b.stride(1);

    GPUData c;
    c.copyFrom( b, true );
    c.make_unique();
    b.fill( 0.f );

    for ( size_t i = 0; i < dims[0]; ++i )
      for ( size_t j = 0; j < dims[1]; ++j )
        CPPUNIT_ASSERT( c[i][j] == i * c.stride(0) + j * c.stride(1) );



  }

	void testSlice()
	{
		size_t dims[] = {11,14,12};
		GPUData a(3,dims);
		for ( size_t i = 0; i < dims[0]; ++i )
		for ( size_t j = 0; j < dims[1]; ++j )
		for ( size_t k = 0; k < dims[2]; ++k )
			{
				a[i][j][k] = i * a.stride(0) + j * a.stride(1) + k * a.stride(2);
			}
		GPUData b( a.slice( 2, 3 ) );
		GPUData c( b.getNoOfDimensions(), b.getDimensions() );
		c.copyFrom( b );
		for ( size_t i = 0; i < b.extent(0); ++i )
			for ( size_t j = 0; j < b.extent(1); ++j )
			{
				CPPUNIT_ASSERT( b[i][j] == i*a.stride(0) + j*a.stride(1) + 3*a.stride(2) );
				CPPUNIT_ASSERT( b[i][j] == c[i][j] );
			}
	}

	void testFill() {
		size_t dim1DSize = 11*11*11*11;
		GPUData a(1,&dim1DSize);
		for (size_t i=0; i<dim1DSize; ++i) {
			a[i] = 0.0f;
		}
		a.fill(2.345f);

		size_t dims[] = {10, 10, 10, 10};
		int inde[] = { 0 };
		int stri[] = { 11*11*11 , 11*11 , 11 , 1};
		GPUData b = a.view(inde,4,dims,stri);
		for (size_t l = 0; l<10; l++) {
		for (size_t k = 0; k<10; k++) {
		for (size_t j = 0; j<10; j++) {
		for (size_t i = 0; i<10; i++) {
			CPPUNIT_ASSERT(b[l][k][j][i] == 2.345f);
		}
		}
		}
		}

		a.fill(0.0f);
		for (size_t l = 0; l<10; l++) {
		for (size_t k = 0; k<10; k++) {
		for (size_t j = 0; j<10; j++) {
		for (size_t i = 0; i<10; i++) {
			CPPUNIT_ASSERT(b[l][k][j][i] == 0.0f);
		}
		}
		}
		}

		{
			GPUData c(5,5,5,5);
			c.fill(1.0f);
			size_t dims[] = {2,2,2,2};
			int offs[] = {1,1,1,1};
			GPUData d(c.view(offs,dims));
			d.fill(2.0f);
			CPPUNIT_ASSERT(c[0][0][0][0] == 1.0f);
			CPPUNIT_ASSERT(c[4][2][2][2] == 1.0f);
			CPPUNIT_ASSERT(c[2][2][2][2] == 2.0f);
		}
	}

	void testScale()
	{
		GPUData data(3,3,3);
		GPUData data2 = data.slice( 0, 0 );

		data.fill(1.0f);
		data.mul(2.5f);

		CPPUNIT_ASSERT( data[0][0][0] == 2.5f );
		CPPUNIT_ASSERT( data[1][1][1] == 2.5f );
		CPPUNIT_ASSERT( data[2][2][2] == 2.5f );

		data2.mul(2.0f);

		CPPUNIT_ASSERT( data[0][0][0] == 5.0f );
		CPPUNIT_ASSERT( data[1][1][1] == 2.5f );
		CPPUNIT_ASSERT( data[2][2][2] == 2.5f );

		CPPUNIT_ASSERT( data2[2][0] == 5.0f );
		CPPUNIT_ASSERT( data2[1][1] == 5.0f );
		CPPUNIT_ASSERT( data2[2][2] == 5.0f );

	}

	void testMul() {
		size_t sizes[] = {5,4,3};
		GPUData a(3,sizes);
		GPUData b(3,sizes);
		a.fill(2.25f);
		b.copyFrom(a);
		b[2][0][0] = 1.0f;

		a.mul(b);

		CPPUNIT_ASSERT(a[0][0][0] == 2.25f*2.25f);
		CPPUNIT_ASSERT(a[2][0][0] == 1.00f*2.25f);

		a.mul(b,33.0f);
		CPPUNIT_ASSERT(a[0][0][0] == 2.25f*2.25f*2.25f*33.0f);
		CPPUNIT_ASSERT(a[2][0][0] == 2.25f*1.00f*1.00f*33.0f);

		a.mul(0.4f);
		CPPUNIT_ASSERT(a[0][0][0] == 2.25f*2.25f*2.25f*33.0f*0.4f);
		CPPUNIT_ASSERT(a[2][0][0] == 2.25f*1.00f*1.00f*33.0f*0.4f);

		size_t sizesA[] = {125,1024};
		size_t sizesB[] = {125,512};
		int offs[] = {0,0};
		int stri[] = {1024,2};
		GPUData c(2,sizesA);
		GPUData d = c.view(offs,2,sizesB,stri);
		GPUData e(2,sizesB);

		c.fill(2.0f);
		d.fill(3.0f);
		e.fill(4.0f);
		d.mul(e);

		CPPUNIT_ASSERT(d[1][0]==3.0f*4.0f);
		CPPUNIT_ASSERT(d[1][1]==3.0f*4.0f);
		CPPUNIT_ASSERT(c[1][0]==3.0f*4.0f);
		CPPUNIT_ASSERT(c[1][1]==2.0f);

		a.fill(1.4142136f);
		a.mul(a);

		CPPUNIT_ASSERT(a[0][0][0]==1.4142136f*1.4142136f);
		CPPUNIT_ASSERT(a[4][3][2]==1.4142136f*1.4142136f);
		CPPUNIT_ASSERT(a[2][1][1]==1.4142136f*1.4142136f);
		CPPUNIT_ASSERT(a[1][1][0]==1.4142136f*1.4142136f);
		CPPUNIT_ASSERT(a[0][2][2]==1.4142136f*1.4142136f);
		CPPUNIT_ASSERT(a[4][3][2]==1.4142136f*1.4142136f);
	}

	void testDiv() {
		size_t sizes[] = {5,4,3};
		GPUData a(3,sizes);
		GPUData b(3,sizes);
		a.fill(2.25f);
		b.copyFrom(a);
		b[2][0][0] = 1.0f;

		a.div(b);

		CPPUNIT_ASSERT(a[0][0][0] == 2.25f/2.25f);
		CPPUNIT_ASSERT(a[2][0][0] == 2.25f/1.00f);
	}

	void testShiftDim()
	{

		GPUData a(3,5,4,2);
		GPUData b = a.shiftDim(1);
		GPUData c;

		CPPUNIT_ASSERT(a.extent(0) == b.extent(1));
		CPPUNIT_ASSERT(a.extent(2) == b.extent(3));
		CPPUNIT_ASSERT(a.extent(3) == b.extent(0));

		a = GPUData(3,3);
		fillLinSpace(a,0.0f,1.0f);

		CPPUNIT_ASSERT(a[0][0]==0.0f);
		CPPUNIT_ASSERT(a[0][1]==1.0f/8);
		CPPUNIT_ASSERT(a[0][2]==2.0f/8);
		CPPUNIT_ASSERT(a[2][2]==1.0f);

		a = a.shiftDim(1);

		CPPUNIT_ASSERT(a[0][0]==0.0f);
		CPPUNIT_ASSERT(a[1][0]==1.0f/8);
		CPPUNIT_ASSERT(a[2][0]==2.0f/8);
		CPPUNIT_ASSERT(a[2][2]==1.0f);

		b = GPUData(3,4,7);
		fillLinSpace(b,0.0f,0.5f); b[1][1][1] = 1.0f;
		a = b.shiftDim(-2).copy();
		c = b.shiftDim(+1).copy();

		GPUDataIsEqualTo tester;

		CPPUNIT_ASSERT(tester(a,c));
	}


	void testSqrt() {
		size_t sizes[] = {5,4,3};
		GPUData a(3,sizes);
		GPUData b(3,sizes);
		a.fill(9.0f);
		b.fill(2.0f);

		a.sqrt();
		b.sqrt();
		//printf("%f %f %f",float(a[1][1][1]),float(fabs(a[1][1][1]-3.0f)),float(b[1][1][1]));
		CPPUNIT_ASSERT(fabs(a[1][1][1]-3.0f) <= 0.00001f);
		CPPUNIT_ASSERT(fabs(b[1][1][1] * b[1][1][1] - 2.0f)<= 0.000001f);
	}

	void testSqr() {
			size_t sizes[] = {5,4,3};
			GPUData a(3,sizes);
			GPUData b(3,sizes);
			a.fill(9.0f);
			b.fill(2.0f);

			a.sqr();
			b.sqr();
			//printf("%f %f %f",float(a[1][1][1]),float(fabs(a[1][1][1]-3.0f)),float(b[1][1][1]));
			CPPUNIT_ASSERT(fabs(a[1][1][1]-81.0f) <= 0.00001f);
			CPPUNIT_ASSERT(fabs(b[1][1][1] - 4.0f)<= 0.000001f);



			GPUData aa(2,sizes);
			GPUData bb(2,sizes);
			aa.fill(9.0f);
			bb.fill(2.0f);


			int offset[] = {0,2};
			size_t sizes_new[] = {5};
			int strides_new[] = {4};

			GPUData aaa(aa.view(offset,1,sizes_new,strides_new));
			GPUData bbb(bb.view(offset,1,sizes_new,strides_new));


//			printf("------------ %d ---------------\n",aaa.getDimensionsCount());
//
//
			aaa.sqr();
			bbb.sqr();
			CPPUNIT_ASSERT(fabs(aaa[1]-81.0f) <= 0.00001f);
			CPPUNIT_ASSERT(fabs(bbb[1] - 4.0f) <= 0.000001f);
			CPPUNIT_ASSERT(fabs(aa[1][1] - 9.0f)<= 0.000001f);
			CPPUNIT_ASSERT(fabs(aa[1][2] -81.0f)<= 0.000001f);
			CPPUNIT_ASSERT(fabs(bb[1][1] - 2.0f)<= 0.000001f);
			CPPUNIT_ASSERT(fabs(bb[2][2] - 4.0f)<= 0.000001f);

	}

	void testLog() {
		size_t sizes[] = {5,4,3};
		GPUData a(3,sizes);
		GPUData b(3,sizes);
		a.fill(9.0f);
		b.fill(2.0f);

		a.log();
		b.log();
		//printf("%f %f %f",float(a[1][1][1]),float(fabs(a[1][1][1]-3.0f)),float(b[1][1][1]));
		CPPUNIT_ASSERT(fabs(a[1][1][1] - logf(9.0)) <= 0.00001f);
		CPPUNIT_ASSERT(fabs(b[1][1][1] - logf(2.0)) <= 0.00001f);
	}

	void testPyth() {
		size_t sizes[] = {5,10,10};
		int offs[] = {2,0,0};
		GPUData a(10,10,10);
		GPUData a2(a.view(offs,sizes));
		GPUData b(10,10,10);
		GPUData b2(b.view(offs,sizes));
		GPUData c(10,10,10);
		GPUData c2(3,sizes);

		a.fill(9.0f);
		b.fill(2.0f);
		c.fill(0.0f);

		b[3][2][2] = 5.0f;

		c.pythagoras(a,b);
//		printf("%f ",float(c[1][1][1]));
		CPPUNIT_ASSERT(fabs(c[1][1][1]-9.21954446f) <= 0.00001f);
		CPPUNIT_ASSERT(fabs(c[2][2][2]-9.21954446f) <= 0.00001f);
		CPPUNIT_ASSERT(fabs(c[3][2][2]-10.2956301f) <= 0.00001f);

		c.fill(0.0f);
		CPPUNIT_ASSERT(fabs(c[3][2][2])<=0.000001f);

		c2.pythagoras(a2,b2);
		CPPUNIT_ASSERT(fabs(c2[1][1][1]-9.21954446f) <= 0.00001f);
		CPPUNIT_ASSERT(fabs(c2[2][2][2]-9.21954446f) <= 0.00001f);
		CPPUNIT_ASSERT(fabs(c2[1][2][2]-10.2956301f) <= 0.00001f);
	}



	void testnakarushton2() {
		size_t sizes[] = {100,10};
		GPUData a(1,sizes);
		GPUData b(1,sizes);

		a.fill(2.0f);
		a[0]  = -4.5f;
		a[99] =  100.0f;

		a.nakarushton2(b,4.5f,0.5f);
		a.nakarushton2(4.5f);

		//printf("%f",float(a[0]));
		CPPUNIT_ASSERT(fabs(a[0]  + 4.5f*4.5f/(4.5f*4.5f*2)) <= 0.000001f);
		CPPUNIT_ASSERT(fabs(a[1]  - 2.0f*2.0f/(2.0f*2.0f+4.5f*4.5f)) <= 0.000001f);
		CPPUNIT_ASSERT(fabs(a[99] - 100.0f*100.0f/(100.0f*100.0f+4.5f*4.5f)) <= 0.000001f);

		CPPUNIT_ASSERT(fabs(b[0]*2  + 4.5f*4.5f/(4.5f*4.5f*2)) <= 0.000001f);
		CPPUNIT_ASSERT(fabs(b[1]*2  - 2.0f*2.0f/(2.0f*2.0f+4.5f*4.5f)) <= 0.000001f);
		CPPUNIT_ASSERT(fabs(b[99]*2 - 100.0f*100.0f/(100.0f*100.0f+4.5f*4.5f)) <= 0.000001f);
	}

	void testatan2() {
		size_t sizes[] = {1000,10};
		GPUData a(2,sizes);
		GPUData b(2,sizes);
		GPUData c(2,sizes);

		a.fill(2.0f);
		b.fill(1.0f);
		c.fill(0.0f);

		c.atan2(a,b);
		//printf("%f",float(c[1][1]));
		CPPUNIT_ASSERT(fabs(c[0][0]-1.10714872f) <= 0.001f);
		CPPUNIT_ASSERT(fabs(c[0][9]-1.10714872f) <= 0.001f);
	}

	void testsincos() {
		size_t sizes[] = {1000,10};
		GPUData a(2,sizes);
		GPUData b(2,sizes);
		GPUData c(2,sizes);

		size_t sizes2[] = {10,10};
		GPUData d(2,sizes2);
		GPUData e(2,sizes2);
		GPUData f(2,sizes2);

		a.fill(2.0f);
		b.fill(1.0f);
		c.fill(0.0f);

		for (size_t i=0; i<10; i++)
		for (size_t j=0; j<10; j++)
		{
			d[i][j]=3.1415927f*i/10.0f;
		}

		a.sin();
		b.cos();
		c.sin();

		e.sin(d);
		f.cos(d);

		//printf("%f",float(c[1][1]));
		CPPUNIT_ASSERT(fabs(a[0][0]-0.909297f) <= 0.0001f);
		CPPUNIT_ASSERT(fabs(b[0][9]-0.540302f) <= 0.0001f);
		CPPUNIT_ASSERT(fabs(c[100][3]-0.0f) <= 0.0001f);
		//printf("%f ",float(e[7][7]));
		CPPUNIT_ASSERT(fabs(e[7][7]-0.809016994f) <= 0.0001f);
		CPPUNIT_ASSERT(fabs(f[7][1]+0.587785252f) <= 0.0001f);
	}

	void testSum() {
		size_t dims[] = {30,30};
		GPUData a(2,dims);
		a.fill(3.0f);
		a[2][2]=-1.5f;

		float target = 30 * 30 * 3 - 3 - 1.5f;

		CPPUNIT_ASSERT(a.sum()==target);

		target += 2.0f * 1.5f;

		CPPUNIT_ASSERT(a.absSum()==target);
	}

	void testSumOverLastDimension() {
		size_t dims[] = {30,30};
		GPUData a(2,dims);
		a.fill(3.0f);
		a[2][2]=-1.5f;

		GPUData b(30);

		a.sumOverDeepestDimension(b);

		CPPUNIT_ASSERT(b.sum()==a.sum());

		CPPUNIT_ASSERT(b[0]==30*3);
		CPPUNIT_ASSERT(b[1]==30*3);
		CPPUNIT_ASSERT(b[2]==29*3-1.5f);
		CPPUNIT_ASSERT(b[3]==30*3);
	}
	void testAddRepeated() {
		GPUData a(20,10);
		GPUData b(10,10);
		a.fill(0.0f);
		b.fill(1.0f);
		b[0].fill(2.0f);

		a.add_repeated(b);

		CPPUNIT_ASSERT(fabs(a.sum()-(20*10+2*10))<=0.00000001);
		CPPUNIT_ASSERT(fabs(a[0][0]-2)<=0.00000001);
		CPPUNIT_ASSERT(fabs(a[0][3]-2)<=0.00000001);
		CPPUNIT_ASSERT(fabs(a[0][9]-2)<=0.00000001);
		CPPUNIT_ASSERT(fabs(a[1][0]-1)<=0.00000001);
		CPPUNIT_ASSERT(fabs(a[1][3]-1)<=0.00000001);
		CPPUNIT_ASSERT(fabs(a[1][9]-1)<=0.00000001);
//		printf("value = %f\n",float(a[10][0]));
		CPPUNIT_ASSERT(fabs(a[10][0]-2)<=0.00000001);
		CPPUNIT_ASSERT(fabs(a[10][3]-2)<=0.00000001);
		CPPUNIT_ASSERT(fabs(a[10][9]-2)<=0.00000001);
		CPPUNIT_ASSERT(fabs(a[11][0]-1)<=0.00000001);
		CPPUNIT_ASSERT(fabs(a[11][3]-1)<=0.00000001);
		CPPUNIT_ASSERT(fabs(a[11][9]-1)<=0.00000001);
	}

	void testAdd() {
		size_t dims[] = {200,200};
		GPUData a(2,dims);
		a.fill(0.0f);
		a[100][100] = 3.0f;
		a[100][101] = 5.0f;
		a[100][102] = 7.0f;

		GPUData b(2,dims);
		b.fill(-5.0f);
		b[100][103]=0.0f;
		a.add(b);

		CPPUNIT_ASSERT(a[100][ 99]==-5.0f);
		CPPUNIT_ASSERT(a[100][100]==-2.0f);
		CPPUNIT_ASSERT(a[100][101]== 0.0f);
		CPPUNIT_ASSERT(a[100][102]== 2.0f);
		CPPUNIT_ASSERT(a[100][103]== 0.0f);

		a.add(b,-1.0f);

		CPPUNIT_ASSERT(a[100][ 99]== 0.0f);
		CPPUNIT_ASSERT(a[100][100]== 3.0f);
		CPPUNIT_ASSERT(a[100][101]== 5.0f);
		CPPUNIT_ASSERT(a[100][102]== 7.0f);
		CPPUNIT_ASSERT(a[100][103]== 0.0f);

		a.add(2.35f);

		CPPUNIT_ASSERT(a[100][100]== 3.0f+2.35f);
		CPPUNIT_ASSERT(a[100][103]== 2.35f);

		a.sub(2.35f);

		CPPUNIT_ASSERT(a[100][100]== 3.0f);
		CPPUNIT_ASSERT(a[100][103]== 0.0f);
		{
			GPUData data(100,10); data.fill(0.5f);
			GPUData data2(100);  data2.fill(1.0f);

			data2.add(data.slice(1,4));
			CPPUNIT_ASSERT(data2[5]==1.5f);
		}
	}
	void testMinMax() {
		size_t size = 2*16*1024+4;
		GPUData a(1,&size);
		a.fill(0.1f);
		a[0] = -2.0f;
		a[2] =  111.0f;
		a[2*16*1024+4-1] = -2.1f;
		a[1*16*1024+2]=-222.0f;
		CPPUNIT_ASSERT(a.max()==111.0f);
		CPPUNIT_ASSERT(a.absMax()==222.0f);

		CPPUNIT_ASSERT(a.min()==-222.0f);
		CPPUNIT_ASSERT(a.absMin()==0.1f);
	}

	void testMinMax2() {
		size_t size = 2*16*1024+4;
		GPUData a(1,&size);
		a.fill(0.1f);

		GPUData b(1,&size);
		b.fill(0.2f);

		a[0] = -2.0f;
		a[2] =  111.0f;
		a[2*16*1024+4-1] = -2.1f;
		a[1*16*1024+2]=-222.0f;

		a.max(b);

		CPPUNIT_ASSERT(a.max()==111.0f);
		CPPUNIT_ASSERT(a.absMax()==111.0f);

		CPPUNIT_ASSERT(a[0]==0.2f);
		CPPUNIT_ASSERT(a[1]==0.2f);
		CPPUNIT_ASSERT(a[2]==111.0f);
		CPPUNIT_ASSERT(a[2*16*1024+4-1]==0.2f);
		CPPUNIT_ASSERT(a[1*16*1024+2]==0.2f);
	}

	void testHeaviside() {
		size_t sizes[] = {225,225};
		GPUData a(2,sizes);
		a.fill(0.0f);
		a[2][4]=-22.1f;
		a[3][4]= 0.199999f;
		a[4][4]= 0.2f;
		a[5][4]= 0.200001f;
		a[6][4]= 22.1f;

		a.heavisideStepFunction(0.2f);

		CPPUNIT_ASSERT(a[2][4]==0.0f);
		CPPUNIT_ASSERT(a[3][4]==0.0f);
		CPPUNIT_ASSERT(a[4][4]==0.5f);
		CPPUNIT_ASSERT(a[5][4]==1.0f);
		CPPUNIT_ASSERT(a[6][4]==1.0f);
	}

	void testModulo()
	{
		size_t sizes[] = {225,225};
		GPUData a(2,sizes);
		a.fill(0.0f);
		a[2][4] = -22.1f;
		a[3][4] =  0.199999f;
		a[4][4] =  0.2f;
		a[5][4] =  0.200001f;
		a[6][4] =  22.1f;

		a.modulo(0.2f);


		//printf("%f",float(a[6][4]));
		/*
		std::cout << std::endl;
		std::cout << a[2][4]       << "," << (-22.1f+0.2f) << std::endl;
		std::cout << a[3][4]       << "," << (0.199999f) << std::endl;
		std::cout << a[4][4]       << "," << (0.0f) << std::endl;
		std::cout << a[5][4]       << "," << (0.200001f-0.2f) << std::endl;
		std::cout << fabs(a[6][4]-0.1f) << "," << (0.0000001f) << std::endl;
		std::cout << a[1][1]       << "," << (0.0f) << std::endl;
		*/
		/* daniel: I dont understand this test, the correct result for fmodf is -0.1f
		CPPUNIT_ASSERT(a[2][4]==-22.1f+0.2f);
		*/
		CPPUNIT_ASSERT(a[3][4]==0.199999f);
		CPPUNIT_ASSERT(a[4][4]==0.0f);
		CPPUNIT_ASSERT(a[5][4]==0.200001f-0.2f);
		CPPUNIT_ASSERT(fabs(a[6][4]-0.1f)<=0.0000001f);
		CPPUNIT_ASSERT(a[1][1]==0.0f);
	}

	void testSetIfInBetween() {
		size_t sizes[] = {225,225};
		GPUData a(2,sizes);
		GPUData b(2,sizes);
		a.fill(0.0f);
		b.fill(1.0f);
		a[2][4]=-22.1f;
		a[3][4]= 0.199999f;
		a[4][4]= 0.2f;
		a[5][4]= 0.200001f;
		a[6][4]= 22.1f;

		a.setIfBetween(0.1f,0.22f,99.0f,b);
		a.setIfBetween(0.1f,0.22f,99.0f);

		//printf("%f",float(b[3][4]));

		CPPUNIT_ASSERT(a[2][4]==-22.1f);
		CPPUNIT_ASSERT(a[3][4]==99.0f);
		CPPUNIT_ASSERT(a[4][4]==99.0f);
		CPPUNIT_ASSERT(a[5][4]==99.0f);
		CPPUNIT_ASSERT(a[6][4]==22.1f);
		CPPUNIT_ASSERT(a[1][1]==0.0f);

		CPPUNIT_ASSERT(b[2][4]==1.0f);
		CPPUNIT_ASSERT(b[3][4]==99.0f);
		CPPUNIT_ASSERT(b[4][4]==99.0f);
		CPPUNIT_ASSERT(b[5][4]==99.0f);
		CPPUNIT_ASSERT(b[6][4]==1.0f);
		CPPUNIT_ASSERT(b[1][1]==1.0f);
	}

	void testRect() {
		size_t sizes[] = {225,225};
		GPUData a(2,sizes);
		a.fill(0.0f);
		a.add(3.0f);
		int ofs[] = {100,100};
		size_t siz[] = {100,100};

		a.rect();

		CPPUNIT_ASSERT(a.sum()==3.0f*225*225);

		GPUData b = a.view(ofs,siz);
		b.fill(-3.0f);

		a.rect();
		CPPUNIT_ASSERT(a[100][100]==0);
		CPPUNIT_ASSERT(a[100][99]==3);
		CPPUNIT_ASSERT(a.sum()==3.0f*(225*225-100*100));
	}

void testClamp() {
    size_t sizes[] = {225,225};
    GPUData a(2,sizes);
    a.fill(0.0f);
    a.add(3.0f);
    a.clamp(0.f, 1.f);

    CPPUNIT_ASSERT(a.sum()==1.0f*225*225);

  }

	void testReduceDimensionality() {
		size_t dims[] = { 3 , 5 , 5 , 9 };
		GPUData a(4,dims);
		a.fill(3.35f);
		a[1][2][2][4] = -1.0f;
		a[0][0][1][1] = -2.0f;
		GPUData b = a.reduceDimensionality();

		size_t dimC = 3;
		GPUData c = a.reduceDimensionality(dimC);
		CPPUNIT_ASSERT(dimC == 1);

		size_t dimD = 1;
		GPUData d = a.reduceDimensionality(dimD);
		CPPUNIT_ASSERT(dimD == 1);

		size_t dimE = 0;
		GPUData e = a.reduceDimensionality(dimE);
		CPPUNIT_ASSERT(dimE == 0);

		size_t dimF1 = 2;
		size_t dimF2 = 3;
		GPUData f = a.reduceDimensionality(dimF1,dimF2);
		CPPUNIT_ASSERT(dimF1 == 1);
		CPPUNIT_ASSERT(dimF2 == 2);

		size_t dimG1 = 0;
		size_t dimG2 = 3;
		GPUData g = a.reduceDimensionality(dimG1,dimG2);
		CPPUNIT_ASSERT(dimG1 == 0);
		CPPUNIT_ASSERT(dimG2 == 2);

		CPPUNIT_ASSERT(b.getDimensionsCount()==1);
		CPPUNIT_ASSERT(b.size()==a.size());
		CPPUNIT_ASSERT(b[(3*5*5*9)/2]==-1.0f);
		CPPUNIT_ASSERT(b[1*(9)+1*(1)]==-2.0f);

		CPPUNIT_ASSERT(c.getDimensionsCount()==2);
		CPPUNIT_ASSERT(c.size()==a.size());
		CPPUNIT_ASSERT(c[(3*5*5)/2][4]==-1.0f);
		CPPUNIT_ASSERT(c[1][1]==-2.0f);

		CPPUNIT_ASSERT(d.getDimensionsCount()==3);
		CPPUNIT_ASSERT(d.size()==a.size());
		CPPUNIT_ASSERT(d[1][2][(5*9)/2]==-1.0f);
		CPPUNIT_ASSERT(d[0][0][1*(9)+1]==-2.0f);

		CPPUNIT_ASSERT(f.getDimensionsCount()==3);
		CPPUNIT_ASSERT(f.size()==a.size());
		CPPUNIT_ASSERT(f[1*5+2][2][4]==-1.0f);
		CPPUNIT_ASSERT(f[0][1][1]==-2.0f);

		CPPUNIT_ASSERT(g.getDimensionsCount()==3);
		CPPUNIT_ASSERT(g.size()==a.size());
		CPPUNIT_ASSERT(g[1][2*5+2][4]==-1.0f);
		CPPUNIT_ASSERT(g[0][0*5+1][1]==-2.0f);
	}

	void test5DDliceAndCopy()
	{
		size_t dims[6] = {3,3,3,3,3,3};
		GPUData a(6,dims);

		for ( size_t i = 0; i < a.extent(1); ++i)
			{
				a.slice(1,i).fill(i);
			}
		GPUData aslice( a.slice(1,1) );
		GPUData b(5,dims);
		b.copyFrom( aslice );
		for ( size_t i = 0; i < 3; ++i )
		for ( size_t j = 0; j < 3; ++j )
		for ( size_t k = 0; k < 3; ++k )
		for ( size_t l = 0; l < 3; ++l )
		for ( size_t m = 0; m < 3; ++m )
			CPPUNIT_ASSERT( b[i][j][k][l][m] == 1 );
	}

	void testExtendDimensionality() {
		GPUData a(4,4,2);
		a.fill(3.35f);
		a[1][2][1] = -1.0f;
		a[2][1][2] = -2.0f;
		GPUData b = a.extendDimensionLeft();

		CPPUNIT_ASSERT(b.getDimensionsCount()==4);
		CPPUNIT_ASSERT(b.extent(0)==1);
		CPPUNIT_ASSERT(b.size()==a.size());

		CPPUNIT_ASSERT(a[1][2][1] == b[0][1][2][1]);
		CPPUNIT_ASSERT(a[2][1][2] == b[0][2][1][2]);
		CPPUNIT_ASSERT(a[0][0][0] == b[0][0][0][1]);
	}

	void testCopyGPUData2GPUData()
	{
		size_t size = 10*10;
		GPUData a(1,&size);

		size_t dims[] = {10,10};
		GPUData b(2,dims);

		a.fill(0.1f);
		a[10]=3.3f;
		a[99]=9.9f;
		a[0] =2.2f;

		b.reduceDimensionality().copyFrom(a);

		CPPUNIT_ASSERT(b[0][0]==2.2f);
		CPPUNIT_ASSERT(b[0][1]==0.1f);
		CPPUNIT_ASSERT(b[1][0]==3.3f);
		CPPUNIT_ASSERT(b[1][1]==0.1f);
		CPPUNIT_ASSERT(b[9][9]==9.9f);
		CPPUNIT_ASSERT(b[9][8]==0.1f);
		CPPUNIT_ASSERT(b[9][7]==0.1f);
	}

	void testCopyToConst()
	{
		size_t size = 17;
		GPUData a(1,&size);
		a.fill(1.234f);
		a[8] = 4.321f;

		a.copyToConstMemory();

		CPPUNIT_ASSERT(a[7]==1.234f && a[8]==4.321f);
	}

	void testGeneralConvolution1Din4D()
	{
		size_t dim1DSize = 11*11*11*11;
		GPUData a(1,&dim1DSize);
		a.fill(0.0f);

		size_t dims[] = {11, 11, 11, 11};
		int inde[] = { 0 };
		int stri[] = { 11*11*11 , 11*11 , 11 , 1};
		GPUData b = a.view(inde,4,dims,stri);
		b[5][5][5][5] = 4.0f;
		b[0][0][0][0] = 4.0f;

		GPUData c(4, dims);

		CPPUNIT_ASSERT(b.isContiguous());

		CPPUNIT_ASSERT(b[5][5][5][4] == 0.0f);
		CPPUNIT_ASSERT(b[5][5][5][5] == 4.0f);
		CPPUNIT_ASSERT(b[5][5][5][6] == 0.0f);

		size_t kSize = 3;
		GPUData k(kSize);
		k[0]=0.2f;
		k[1]=0.5f;
		k[2]=0.3f;

		b.convolute1D(k,0,c,0);
		c.convolute1D(k,1,b,0);
		b.convolute1D(k,2,c,0);
		c.convolute1D(k,3,b,0);

//		printf("%f, %f, %f\n",(float)(b[5][5][5][4]),(float)(b[5][5][5][5]),(float)(b[5][5][5][6]));
//		printf("%f, %f, %f\n",(float)(c[5][5][5][4]),(float)(c[5][5][5][5]),(float)(c[5][5][5][6]));

		CPPUNIT_ASSERT(b[5][5][5][5] == 4.0f *0.5f*0.5f*0.5f*0.5f);
		CPPUNIT_ASSERT(b[5][5][5][6] == 4.0f *0.3f*0.5f*0.5f*0.5f);
		CPPUNIT_ASSERT(b[6][5][5][5] == 4.0f *0.5f*0.5f*0.5f*0.3f);

		CPPUNIT_ASSERT(b[0][0][0][0] == 4.0f *0.5f*0.5f*0.5f*0.5f);
		CPPUNIT_ASSERT(b[0][0][0][1] == 4.0f *0.3f*0.5f*0.5f*0.5f);
		CPPUNIT_ASSERT(b[1][0][0][0] == 4.0f *0.5f*0.5f*0.5f*0.3f);

		GPUData a2( dim1DSize );
		a.fill(0.0f);
		a[15] = 1.0f;
		a.convolute1D(k,0,a2);

		CPPUNIT_ASSERT(a2[13]==1.0f*0.0f);
		CPPUNIT_ASSERT(a2[14]==1.0f*0.2f);
		CPPUNIT_ASSERT(a2[15]==1.0f*0.5f);
		CPPUNIT_ASSERT(a2[16]==1.0f*0.3f);
		CPPUNIT_ASSERT(a2[17]==1.0f*0.0f);
	}


	inline void iterateDGLExplicitEuler(float& x,
						const float& A,
						const float& B,
						const float& C,
						const float& D,
						const float& E,
						const float& F,
						const float& dt)
	{
		x = x+dt*(x*(A+B+C)+(D+E+F));
	}

	inline void iterateDGL2ExplicitEuler(float& x,
						const float& a,
						const float& b,
						const float& B,
						const float& c,
						const float& C,
						const float& dt)
	{
		x = x+dt*(-a*x+(1-x)*(b*B+c*C));
	}

	inline void iterateDGLImplicitEuler(float& x,
						const float& A,
						const float& B,
						const float& C,
						const float& D,
						const float& E,
						const float& F,
						const float& dt)
	{
		x = (x+dt*(D+E+F))/(1-dt*(A+B+C));
	}

	inline void iterateDGL2ImplicitEuler(float& x,
						const float& a,
						const float& b,
						const float& B,
						const float& c,
						const float& C,
						const float& dt)
	{
		x = (x+dt*(b*B+c*C))/(1+dt*(a+b*B+c*C));
	}

	void testDGLExplicitEuler()
	{
		const int w = 12;
		const int h = 24;

		const int w_idx = 2;
		const int h_idx = 4;

		GPUData x(h,w);

		GPUData  A(h,w);
		GPUData  B(h);
		float	 C;

		GPUData  D(h,w);
		GPUData  E(h);
		float	 F;

		x.fill(0.0f);
		fillLinSpace(A,0,1);
		fillLinSpace(B,0,1);
		C = 0.25f;
		D.copyFrom(A); D.div(2.0f);
		E.copyFrom(B); D.div(2.0f);
		F = C/2.0f;

		const size_t N = 100;
		const float dt = 0.001f;

		float x_local = x[h_idx][w_idx];
		float a_local = A[h_idx][w_idx];
		float b_local = B[h_idx];
		float c_lcoal = C;
		float d_local = D[h_idx][w_idx];
		float e_local = E[h_idx];
		float f_lcoal = F;
		for (size_t i=0; i<N; ++i) {
			x.linFirstOrderODEEulerStep(1,A,1,B,C,1,D,1,E,F,dt);
			iterateDGLExplicitEuler(x_local,a_local,b_local,c_lcoal,d_local,e_local,f_lcoal,dt);
//			printf("%6.3f %6.3f\n",float(x[h_idx][w_idx]),x_local);
		}
//		printf("%6.3f %6.3f\n",float(x[h_idx][w_idx]),x_local);
		CPPUNIT_ASSERT(fabs(x[h_idx][w_idx]-x_local)<0.0001f);
	}

	void testDGLExplicitEulerColMaj()
	{
		const int w = 12;
		const int h = 24;

		const int w_idx = 2;
		const int h_idx = 4;

		GPUData  x(w,h); x = x.shiftDim(1);

		GPUData  A(w,h); A = A.shiftDim(1);
		GPUData  B(h);
		float	 C;

		GPUData  D(w,h); D = D.shiftDim(1);
		GPUData  E(h);
		float	 F;

		x.fill(0.0f);
		fillLinSpaceColMajor(A,0,1,w,h);
		fillLinSpace(B,0,1);
		C = 0.25f;
		D.copyFrom(A); D.div(2.0f);
		E.copyFrom(B); D.div(2.0f);
		F = C/2.0f;

		const size_t N = 100;
		const float dt = 0.001f;

		float x_local = x[h_idx][w_idx];
		float a_local = A[h_idx][w_idx];
		float b_local = B[h_idx];
		float c_lcoal = C;
		float d_local = D[h_idx][w_idx];
		float e_local = E[h_idx];
		float f_lcoal = F;
		for (size_t i=0; i<N; ++i) {
			x.linFirstOrderODEEulerStep(1,A,1,B,C,1,D,1,E,F,dt);
			iterateDGLExplicitEuler(x_local,a_local,b_local,c_lcoal,d_local,e_local,f_lcoal,dt);
//			printf("%6.3f %6.3f\n",float(x[h_idx][w_idx]),x_local);
		}
//		printf("%6.3f %6.3f\n",float(x[h_idx][w_idx]),x_local);
		CPPUNIT_ASSERT(fabs(x[h_idx][w_idx]-x_local)<0.0001f);
	}

	void testDGLExplicitEuler2()
	{
		const int w = 12;
		const int h = 24;

		const int w_idx = 2;
		const int h_idx = 4;

		GPUData x(h,w);

		float    a;
		float    b;
		GPUData  B(h,w);
		float	 c;
		GPUData  C(h);

		x.fill(0.0f);
		a = 1;
		b = 0.5f;
		fillLinSpace(B,0,1);
		c = 0.25f;
		fillLinSpace(C,0,1);

		const int N = 100;
		const float dt = 0.001f;

		float x_local = x[h_idx][w_idx];
		float B_local = B[h_idx][w_idx];
		float C_local = C[h_idx];

		for (int i=0; i<N; ++i)
			{
				x.linFirstOrderODEEulerStep(a,b,B,c,C,dt);
				iterateDGL2ExplicitEuler(x_local,a,b,B_local,c,C_local,dt);
				//			printf("%6.3f %6.3f\n",float(x[h_idx][w_idx]),x_local);
				if (i%5==0) CPPUNIT_ASSERT(fabs(x[h_idx][w_idx]-x_local)<0.0001f);
			}
		//		printf("%6.3f %6.3f\n",float(x[h_idx][w_idx]),x_local);
		CPPUNIT_ASSERT(fabs(x[h_idx][w_idx]-x_local)<0.0001f);
	}

	void testDGLImplicitEuler()
	{
		const int w = 12;
		const int h = 24;

		const int w_idx = 2;
		const int h_idx = 4;

		GPUData x(h,w);

		GPUData  A(h,w);
		GPUData  B(h);
		float	 C;

		GPUData  D(h,w);
		GPUData  E(h);
		float	 F;

		x.fill(0.0f);
		fillLinSpace(A,0,1);
		fillLinSpace(B,0,1);
		C = 0.25f;
		D.copyFrom(A); D.div(2.0f);
		E.copyFrom(B); D.div(2.0f);
		F = C/2.0f;

		const size_t N = 100;
		const float dt = 0.001f;

		float x_local = x[h_idx][w_idx];
		float a_local = A[h_idx][w_idx];
		float b_local = B[h_idx];
		float c_lcoal = C;
		float d_local = D[h_idx][w_idx];
		float e_local = E[h_idx];
		float f_lcoal = F;
		for (size_t i=0; i<N; ++i) {
			x.linFirstOrderODEImplicitEulerStep(1,A,1,B,C,1,D,1,E,F,dt);
			iterateDGLImplicitEuler(x_local,a_local,b_local,c_lcoal,d_local,e_local,f_lcoal,dt);
//			printf("%6.3f %6.3f\n",float(x[h_idx][w_idx]),x_local);
		}
		CPPUNIT_ASSERT(fabs(x[h_idx][w_idx]-x_local)<0.0001f);
	}

	void testDGLImplicitEuler2()
	{
		const int w = 12;
		const int h = 24;

		const int w_idx = 2;
		const int h_idx = 4;

		GPUData x(h,w);

		float    a;
		float    b;
		GPUData  B(h,w);
		float	 c;
		GPUData  C(h);

		x.fill(0.0f);
		a = 1;
		b = 0.5f;
		fillLinSpace(B,0,1);
		c = 0.25f;
		fillLinSpace(C,0,1);

		const int N = 100;
		const float dt = 0.001f;

		float x_local = x[h_idx][w_idx];
		float B_local = B[h_idx][w_idx];
		float C_local = C[h_idx];

		for (int i=0; i<N; ++i) {
			x.linFirstOrderODEImplicitEulerStep(a,b,B,c,C,dt);
			iterateDGL2ImplicitEuler(x_local,a,b,B_local,c,C_local,dt);
//			printf("%6.3f %6.3f\n",float(x[h_idx][w_idx]),x_local);
			if (i%5==0) CPPUNIT_ASSERT(fabs(x[h_idx][w_idx]-x_local)<0.0001f);
		}
//		printf("%6.3f %6.3f\n",float(x[h_idx][w_idx]),x_local);
		CPPUNIT_ASSERT(fabs(x[h_idx][w_idx]-x_local)<0.0001f);
	}
	void testDGLSteadyState()
	{
		const int w = 12;
		const int h = 24;

		const int w_idx = 2;
		const int h_idx = 4;

		GPUData x(h,w);

		GPUData  A(h,w);
		GPUData  B(h);
		float	 C;

		GPUData  D(h,w);
		GPUData  E(h);
		float	 F;

		x.fill(0.0f);
		fillLinSpace(A,0,1);
		fillLinSpace(B,0,1);
		C = 0.25f;
		D.copyFrom(A); D.div(2.0f);
		E.copyFrom(B); D.div(2.0f);
		F = C/2.0f;

		const size_t N = 100;
		const float dt = 0.001f;

		for (size_t i=0; i<N; ++i) {
			x.linFirstOrderODEImplicitEulerStep(1,A,1,B,C,1,D,1,E,F,dt*10000000000.0f);
		}
		float x_local = x[h_idx][w_idx];
		x.fill(0.0f);
		x.linFirstOrderODESteayStateSolution(1,A,1,B,C,1,D,1,E,F);
		CPPUNIT_ASSERT(fabs(x[h_idx][w_idx]-x_local)<0.0001f);
	}

	void testDGLSteadyState2()
	{
		const int w = 12;
		const int h = 24;

		const int w_idx = 2;
		const int h_idx = 4;

		GPUData x(h,w);

		float    a;
		float    b;
		GPUData  B(h,w);
		float	 c;
		GPUData  C(h);

		x.fill(0.0f);
		a = 1;
		b = 0.5f;
		fillLinSpace(B,0,1);
		c = 0.25f;
		fillLinSpace(C,0,1);

		const int N = 1;

		float x_local = x[h_idx][w_idx];
		float B_local = B[h_idx][w_idx];
		float C_local = C[h_idx];

		for (int i=0; i<N; ++i) {
			x.linFirstOrderODESteayStateSolution(a,b,B,c,C);
			x_local = (b*B_local+c*C_local)/(a+b*B_local+c*C_local);
		}
//		printf("%6.3f %6.3f\n",float(x[h_idx][w_idx]),x_local);
		CPPUNIT_ASSERT(fabs(x[h_idx][w_idx]-x_local)<0.0001f);
	}

	void testMatVecMul()
	{
		GPUData matrix(2,3);
		GPUData vector(3);

		vector[0] = 1.0f;
		vector[1] = 2.0f;
		vector[2] = 3.0f;

		matrix[0].fill(1.0f);
		matrix[1][0] = -1.0f;
		matrix[1][1] = +1.0f;
		matrix[1][2] = -1.0f;

		GPUData res(2);

		res.matVecMul( matrix,vector );

//		printf("%f %f\n",float(res[0]),float(res[1]));

		CPPUNIT_ASSERT( res[0] == +6.0f );
		CPPUNIT_ASSERT( res[1] == -2.0f );
	}

	void testRemoveSingletonDimensions()
	{
		GPUData test(2,1,4,1,2);
		GPUData test2(test.removeSingletonDimensions());
		CPPUNIT_ASSERT(test2.getDimensionsCount()==3);
		CPPUNIT_ASSERT(test2.extent(0)==2);
		CPPUNIT_ASSERT(test2.extent(1)==4);
		CPPUNIT_ASSERT(test2.extent(2)==2);
	}

	void testCopy1dSpecial()
	{
		GPUData a(40000,2);
		GPUData b(40000);
		a.fill(1.0f);
		b.fill(0.0f);
		b.copyFrom(a.slice(1,0));
		b.copyFrom(a.slice(1,1));
		CPPUNIT_ASSERT(b[0]==1.0f);
		CPPUNIT_ASSERT(b[4]==1.0f);
		CPPUNIT_ASSERT(b[99]==1.0f);

		b.mul(2.0f);
		a.slice(1,0).removeSingletonDimensions().copyFrom(b);

		CPPUNIT_ASSERT(a[2][0]==2.0f);
		CPPUNIT_ASSERT(a[2][1]==1.0f);
	}



	void testTexDataObject() {

#if 0
		GPUDataIsEqualTo tester;

		GPUTextureData1D test1(5);
		test1.fill(3.0f);
		GPUData dat = test1.read();
		CPPUNIT_ASSERT(dat[0]==3.0f);
		CPPUNIT_ASSERT(dat[4]==3.0f);

		GPUTextureData1D test2(2);
		GPUTextureData1D test3;
		test3 = test2;
		GPUData dat2 = test2.read();
		GPUData dat3 = test3.read();
		CPPUNIT_ASSERT( tester(dat2,dat3) );

		test2.fill(5.0f);
		dat2 = test2.read();
		CPPUNIT_ASSERT(dat2[0]==5.0f);
		CPPUNIT_ASSERT(dat2[1]==5.0f);

		GPUTextureData2D test2d(3,3);
		test2d.bindTexture();
		test2d.fill(3.0f);
		dat = test2d.read();
		CPPUNIT_ASSERT(dat[0][0]==3.0f);
		CPPUNIT_ASSERT(dat[2][2]==3.0f);


		GPUTextureData2D test2d2;
		test2d2 = test2d;
		dat = test2d2.read();
		CPPUNIT_ASSERT(dat[0][0]==3.0f);
		CPPUNIT_ASSERT(dat[2][2]==3.0f);

		GPUData toWrite(5,2);
		for(int j=0; j<5; j++){
			for(int i=0; i<2;i++){
				toWrite[j][i] = (i+j)*1.0f;
			}
		}
//		for(int j=0; j<5; j++){
//			for(int i=0; i<2;i++){
//				std::cout << float(toWrite[j][i]) << " ";
//			}
//			std::cout << std::endl;
//		}
		GPUTextureData2D write;
		write.writeGPUData(toWrite);
		dat = write.read();
//		for(int j=0; j<5; j++){
//			for(int i=0; i<2;i++){
//				std::cout << float(dat[j][i]) << " ";
//			}
//			std::cout << std::endl;
//		}
		CPPUNIT_ASSERT(dat[0][0]==0.0f);
		CPPUNIT_ASSERT(dat[4][1]==5.0f);
		CPPUNIT_ASSERT(dat[4][0]==4.0f);

#endif
	}

	void testMedian3x3 () 
	{
		size_t width = 512;
		size_t height = 512;
		GPUData a(width, height);
		GPUData b(width, height);
		float *img = pgm::read_pgm_file("tests/data/lena_noise.pgm");
		float *cpu_res = (float*)malloc(width*height*sizeof(float));
		float *gpu_res = (float*)malloc(width*height*sizeof(float));
		a.copyFromHostMemory(img);

		// calculate cpu result
		median3x3_cpu(cpu_res, img, width, height);

		// get gpu result
		b.median3x3(a);
		b.copyToHostMemory(gpu_res);

		for (size_t i = 0; i < width*height; i++) 
			CPPUNIT_ASSERT(cpu_res[i] == gpu_res[i]);
	
		free(gpu_res);
		free(cpu_res);
		free(img);
	}

	void testArray()
	{
		size_t w = 64;
		size_t h = 32;
		size_t d1 = 4;
		size_t d2 = 2;
		GPUData A( w, h, d1, d2 );
		GPUData B;
		float *A_cpu = (float*)malloc(w * h * d1 * d2 * sizeof(float));
		float *B_cpu = (float*)malloc(w * h * d1 * d2 * sizeof(float));
		
		A.fill(3.14f);
		A.copyToHostMemory( A_cpu );
		
		B.reference( A );
		B.make_unique();
		
		B.move_to_array();
		B.move_to_data();
		
		B.copyToHostMemory( B_cpu );
		
		CPPUNIT_ASSERT(A.extent(0) == B.extent(0) );
		CPPUNIT_ASSERT(A.extent(1) == B.extent(1) );
		CPPUNIT_ASSERT(A.extent(2) == B.extent(2) );
		CPPUNIT_ASSERT(A.extent(3) == B.extent(3) );
		for (size_t i = 0; i < w * h * d1 * d2; i++) 
			CPPUNIT_ASSERT( A_cpu[i] == B_cpu[i] );
		
	}
};

CPPUNIT_TEST_SUITE_REGISTRATION(GPUCudaTest);

int main() {
	CppUnit::TextUi::TestRunner runner;
	CppUnit::TestFactoryRegistry& registry = CppUnit::TestFactoryRegistry::getRegistry();
	runner.addTest(registry.makeTest());
	return ! runner.run("", false);
}

// Local Variables:
// mode: c++
// coding: utf-8
// c-basic-offset: 4
// tab-width: 4
// End:
