/*

GPUIMAGEPROCESSING

  THIS MODULE PROVIDES A SIMPLE API FOR IMAGE OPERATIONS ON
  A GRAPHICAL PROCESSING UNIT (GPU) BY MAKING USE OF CUDA 
  TECHNOLOGY.

COPYRIGHT (C)

  ULM UNIVERSITY 
  INSTITUTE OF NEURAL INFORMATION PROCESSING (NEUROINFORMATIK)
  89069 ULM
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  INSTITUTE OF NEURAL INFORMATION PROCESSING:
  http://www.uni-ulm.de/en/in/institute-of-neural-information-processing.html

  ULM UNIVERSITY 
  HTTP://WWW.UNI-ULM.DE/

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
#ifndef __MOTION_ESTIMATOR_HPP__1D1424AD_40BA_48CD_B36F_B1251D125407
#define __MOTION_ESTIMATOR_HPP__1D1424AD_40BA_48CD_B36F_B1251D125407

#include "compute_hypos.hcu"
#include "HypoImage.h"
#include "CPUData.hpp"
#include "image_information.hpp"
#include "census.hpp"


class MotionEstimator
{
public:
	MotionEstimator (const int w, const int h, const int ds_factor, const int n_external_feeds);
	
	~MotionEstimator ();

	void process_V1 (ImageInformation &current, ImageInformation &past, ...);

	void process_MT ();

	void setup_external_feeds (HypoFeed *f, ...);

	void init_timestep ();

	float* get_V1_data_ptr();
	float* get_MT_data_ptr();

	void dump_V1 (char *fname);
	void dump_MT (char *fname);
	
	HypoImage& getV1HypoImage() { return _V1; }
	HypoImage& getMTHypoImage() { return _MT; }
	
	void saliencyMask(bool b) { _useSaliencyThreshold = b; }
	bool saliencyMask() { return _useSaliencyThreshold; }

public:
	census::CensusValue *current_list;
	census::CensusValue *past_list;

private:
	const int _w;
	const int _h;
	const int _ds_factor;
	const int _nfeeds;
	HypoFeed *_feeds;
	HypoImage _V1;
	HypoImage _MT;
	CPUData<float> _hostHypoImage;
	float *_censusOcc;
	GPUImage _censusOccGPU;
	
	bool _useSaliencyThreshold;
	GPUImage _saliencyMask;
	GPUImage _slow,_fast;
};


#endif /* __MOTION_ESTIMATOR_HPP__1D1424AD_40BA_48CD_B36F_B1251D125407 */

