/*

GPUIMAGEPROCESSING

  THIS MODULE PROVIDES A SIMPLE API FOR IMAGE OPERATIONS ON
  A GRAPHICAL PROCESSING UNIT (GPU) BY MAKING USE OF CUDA 
  TECHNOLOGY.

COPYRIGHT (C)

  ULM UNIVERSITY 
  INSTITUTE OF NEURAL INFORMATION PROCESSING (NEUROINFORMATIK)
  89069 ULM
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  INSTITUTE OF NEURAL INFORMATION PROCESSING:
  http://www.uni-ulm.de/en/in/institute-of-neural-information-processing.html

  ULM UNIVERSITY 
  HTTP://WWW.UNI-ULM.DE/

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
#include <stdio.h>
#include <cutil.h>
#include <stdlib.h>

//__global__ void setKernel(float *d_inkernel, int inkernelx, int inkernely){
//
//	int idx = blockIdx.x*blockDim.x + threadIdx.x;
//	int idy = blockIdx.y*blockDim.y + threadIdx.y;
//
//	s_kernel[idx+(idy*inkernelx)] = d_inkernel[idx+(idy*inkernelx)];
//
//	__syncthreads();
//}


__global__ void con2Dshared(float *d_in, int inx, int iny, float *d_inkernel, int inkernelx, int inkernely, float *d_erg, bool neumann){


	extern __shared__ float s_kernel[];

	for(int y=0; y<(inkernely+blockDim.y-1)/blockDim.y; y++){
		s_kernel[threadIdx.x+(inkernelx*(threadIdx.y+y*blockDim.y))] = d_inkernel[threadIdx.x+(inkernelx*(threadIdx.y+y*blockDim.y))];
	}
//	s_kernel[threadIdx.x+(inkernelx*threadIdx.y)] = d_inkernel[threadIdx.x+(inkernelx*threadIdx.y)];

	// Load one element per thread from device memory and store it
	int idx = blockIdx.x*blockDim.x + threadIdx.x;
	int idy = blockIdx.y*blockDim.y + threadIdx.y;

	if(idx>=inx || idy>=iny) return;

	int L=inkernelx/2,K=inkernely/2;

	float sum = 0;

	for (int i=-L; i<=L; i++) {
		int t;
		t = idx+i;
		if(neumann){
			if(t<0) { t=0; }
			if(t>=inx) { t=inx-1; }
		}
		if(t>=0 && t<inx){
			for (int j=-K; j<=K; j++) {
				int u;
				u = idy+j;
				if(neumann){
					if(u<0) { u=0; }
					if(u>=iny) { u=iny-1; }
				}
				if(u>=0 && u<iny){
					//s_data[threadIdx.x+(blockDim.x*threadIdx.y)] += (d_in[t+inx*(u)] * s_kernel[-i+L+inkernelx*(-j+K)]);
					sum += (d_in[t+inx*(u)] * s_kernel[-i+L+inkernelx*(-j+K)]);
				}
			}
		}
	}

	//s_data[threadIdx.x+(blockDim.x*threadIdx.y)] = sum;

	// Block until all threads in the block have written their data to shared mem
	__syncthreads();

	//d_erg[idx+(idy*inx)] = s_data[threadIdx.x+(blockDim.x*threadIdx.y)];
	d_erg[idx+(idy*inx)] = sum;
}


__global__ void con2Dstrided( float *d_in, int inx, int indx, int iny, int indy,
							  float *d_inkernel, int inkernelx, int inkernely,
							  float *d_erg, int outdx, int outdy,
							  bool neumann )
{
  const int idx = blockIdx.x*blockDim.x + threadIdx.x;
  const int idy = blockIdx.y*blockDim.y + threadIdx.y;
  const int L=inkernelx/2,K=inkernely/2;

  if ( idx >= inx || idy >= iny ) return;

  /*
	const int ilo = L > idx ? -idx : -L;
	const int ihi = L > (inx - idx) ? (inx - idx) : L;
	const int jlo = K > idy ? -idy : -L;
	const int jhi = K > (iny - idy) ? (iny - idy) : L;
  */

  float * in_py = d_in;
  float * k_py   = d_inkernel;
  float sum = 0;
  for ( int i = -K; i <= K; i++, in_py += indy, k_py += inkernelx )
	{
	  // set row pointers
	  float * in_px  =  in_py;
	  float * k_px   = k_py;
	  // check border
	  int u = idy + i;
	  if ( u < 0 )
		{
		  if ( neumann )
			in_px -= u * indy;
		  else
			continue;
		}
	  else if ( u >= iny )
		{
		  if ( neumann )
			in_px -= ( u - iny + 1 ) * indy;
		  else
			continue;
		}
	  for (int j = -L; j <= L; j++, in_px += indx, ++k_px )
		{
		  // set pixel pointer
		  float * in_p = in_px;
		  // check border
		  int t = idx + j;
		  if ( t < 0 )
			{
			  if ( neumann )
				in_p -= t * indx;
			  else
				continue;
			}
		  else if ( t >= inx )
			{
			  if ( neumann )
				in_p -= ( t - inx + 1 ) * indx;
			  else
				continue;
			}
		  // do accumulate
		  sum += (*in_p) * (*k_px);
		}
	}
  d_erg[ idx * outdx + idy * outdy ] = sum;

	// Block until all threads in the block have written their data to shared mem
	__syncthreads();

}


__global__ void con2D(float *d_in, int inx, int iny, float *d_inkernel, int inkernelx, int inkernely, float *d_erg, bool neumann)
{
// Load one element per thread from device memory and store it
	int idx = blockIdx.x*blockDim.x + threadIdx.x;
	int idy = blockIdx.y*blockDim.y + threadIdx.y;

	if ( idx >= inx || idy >= iny ) return;

	int L=inkernelx/2,K=inkernely/2;

	float sum = 0.0f;


	for (int j=-K; j<=K; j++) {
		int u;
		u = idy+j;
		if(neumann){
			if(u<0) { u=0; }
			if(u>=iny) { u=iny-1; }
		}
		if(u>=0 && u<iny)
		for (int i=-L; i<=L; i++) {
			int t;
			t = idx+i;
			if(neumann){
				if(t<0) { t=0; }
				if(t>=inx) { t=inx-1; }
			}
			if(t>=0 && t<inx){
				sum = sum + (d_in[t+inx*(u)] * d_inkernel[-i+L+inkernelx*(-j+K)]);
			}
		}
	}

	d_erg[idx+(idy)*inx]=sum;

	// Block until all threads in the block have written their data to shared mem
	__syncthreads();
}


extern "C" void conv2D(float *d_in, int in_x, int in_y, float *d_kernel, int kernel_x, int kernel_y, float *d_erg, bool neumann) {

	const int blockDimensions_x = kernel_x;
	const int blockDimensions_y = min(512/kernel_x,kernel_y);
	const int gridDimension_x = (in_x+blockDimensions_x-1)/blockDimensions_x;
	const int gridDimension_y = (in_y+blockDimensions_y-1)/blockDimensions_y;
	int memSize = blockDimensions_x * blockDimensions_y * sizeof(float);

	dim3 dimBlock(blockDimensions_x,blockDimensions_y); // threadIdx
	dim3 dimGrid(gridDimension_x,gridDimension_y);		// blockIdx

	if (blockDimensions_x*blockDimensions_y>512) 
	  {
		con2D<<< dimGrid, dimBlock >>>(d_in, in_x, in_y, d_kernel, kernel_x, kernel_y, d_erg, neumann);
	  }
	else 
	  {
		con2Dshared<<< dimGrid, dimBlock, memSize >>>(d_in, in_x, in_y, d_kernel, kernel_x, kernel_y, d_erg, neumann);
	  }


//	unsigned int timer2 = 0;
//	CUT_SAFE_CALL( cutCreateTimer( &timer2 ) );
//	CUT_SAFE_CALL( cutStartTimer( timer2 ) );
//
//	con2D<<< dimGrid, dimBlock >>>(d_in, in_x, in_y, d_kernel, kernel_x, kernel_y, d_erg, neumann);
//
//	cudaThreadSynchronize();
//	CUT_SAFE_CALL( cutStopTimer( timer2 ) );
//	printf( "Processing time: %f (ms)\n", cutGetTimerValue( timer2 ) );
//	CUT_SAFE_CALL( cutDeleteTimer( timer2 ) );


	//checkCUDAError("kernel execution");
}

extern "C" void conv2Dstrided(float *d_in, int in_x, int in_y, int in_dx, int in_dy,
							  float *d_kernel, int kernel_x, int kernel_y,
							  float *d_erg, int out_dx, int out_dy,
							  bool neumann )
{
#ifdef __DEVICE_EMULATION__
	const int blockDimensions_x = 3;
	const int blockDimensions_y = 2;
#else
	const int blockDimensions_x = 16;
	const int blockDimensions_y = 8;
#endif

	const int gridDimension_x = (in_x+blockDimensions_x-1)/blockDimensions_x;
    const int gridDimension_y = (in_y+blockDimensions_y-1)/blockDimensions_y;

	dim3 dimBlock(blockDimensions_x,blockDimensions_y); // threadIdx
	dim3 dimGrid(gridDimension_x,gridDimension_y);		// blockIdx

	con2Dstrided<<< dimGrid, dimBlock >>>(d_in, in_x, in_y, in_dx, in_dy, d_kernel, kernel_x, kernel_y, d_erg, out_dx, out_dy, neumann);

	//checkCUDAError("strided kernel execution");
}
