/*

GPUIMAGEPROCESSING

  THIS MODULE PROVIDES A SIMPLE API FOR IMAGE OPERATIONS ON
  A GRAPHICAL PROCESSING UNIT (GPU) BY MAKING USE OF CUDA 
  TECHNOLOGY.

COPYRIGHT (C)

  ULM UNIVERSITY 
  INSTITUTE OF NEURAL INFORMATION PROCESSING (NEUROINFORMATIK)
  89069 ULM
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  INSTITUTE OF NEURAL INFORMATION PROCESSING:
  http://www.uni-ulm.de/en/in/institute-of-neural-information-processing.html

  ULM UNIVERSITY 
  HTTP://WWW.UNI-ULM.DE/

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
#include <cuda.h>
#include "cutil.h"

/*
 * The block hase to be 2d (x) and (at most) 16*8 elements
 */
__global__ void shift2DRightDownd(float* from, float* to, int NX, int NY, int offsX, int offsY) {
	// Block index
	const int bIndexX = blockIdx.x;
	const int bIndexY = blockIdx.y;

	// no of threads
	const int noOfThreadsX = blockDim.x;
	const int noOfThreadsY = blockDim.y;

	// Thread index
	const int tIndexX = threadIdx.x;
	const int tIndexY = threadIdx.y;
	const int tIndex  = (threadIdx.y * blockDim.x)+threadIdx.x;

	__shared__ float temp[16*8];

	// index Offset
	const int x  = (bIndexX * noOfThreadsX) + tIndexX; // destination
	const int y  = (bIndexY * noOfThreadsY) + tIndexY; // destination
	const int d  = y*NX+x;
	const int xO = x - offsX;
	const int yO = y - offsY;
	const int o  = yO*NX+xO;

	if (x<NX && y<NY) {

		if (xO<0 || yO<0 || xO>=NX || yO>=NY)
			temp[tIndex] = 0.0f;
		else
			temp[tIndex] = from[o];

	}

	__syncthreads();

	if (x<NX && y<NY) {
		to[d] = temp[tIndex];
	}
}


extern "C" void doShift2D(float* dest, float* src, const int nx, const int ny, const int dx, const int dy) {
	dim3 dimBlock(16,8);
    dim3 dimGrid(iDivUp(nx,dimBlock.x),iDivUp(ny,dimBlock.y));

	shift2DRightDownd<<<dimGrid, dimBlock>>>(src, dest, nx, ny, dx, dy);
}
