/*

GPUIMAGEPROCESSING

  THIS MODULE PROVIDES A SIMPLE API FOR IMAGE OPERATIONS ON
  A GRAPHICAL PROCESSING UNIT (GPU) BY MAKING USE OF CUDA 
  TECHNOLOGY.

COPYRIGHT (C)

  ULM UNIVERSITY 
  INSTITUTE OF NEURAL INFORMATION PROCESSING (NEUROINFORMATIK)
  89069 ULM
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  INSTITUTE OF NEURAL INFORMATION PROCESSING:
  http://www.uni-ulm.de/en/in/institute-of-neural-information-processing.html

  ULM UNIVERSITY 
  HTTP://WWW.UNI-ULM.DE/

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
#include "cutil.h"
#include "cu_math.h"



__global__ void d_smooth_with_conf( float* values, float* confidence, 
                                    int w, int h, int d, 
                                    float* in, float* in_c, 
                                    float Ztau )
{
  const int x   = blockIdx.x * blockDim.x + threadIdx.x; // globalIndex
  const int y   = blockIdx.y * blockDim.y + threadIdx.y; // globalIndex
  
  if ( x >= w || y >= h ) return;
  
  const int pos = y + x * h;
  const int pos_v = (y + x * h) * d;
  
  float conf = in_c[ pos ];
  confidence[ pos ] += conf;
  in_c      [ pos ]  = confidence[ pos ] / Ztau;

  for ( int k = 0; k < d; ++k )
  {
    values[ pos_v + k ] += in    [ pos_v + k ] * conf;
    in    [ pos_v + k ]  = values[ pos_v + k ] / ( fmaxf( 0.00000000001f, in_c[ pos ] ) * Ztau );
  }
  
  
}


__global__ void d_smooth( float* values,
                          int w, int h, int d, 
                          float* in,
                          float Ztau )
{
  const int x   = blockIdx.x * blockDim.x + threadIdx.x; // globalIndex
  const int y   = blockIdx.y * blockDim.y + threadIdx.y; // globalIndex
  
  if ( x >= w || y >= h ) return;
  
  const int pos_v = (y + x * h) * d;
  
  for ( int k = 0; k < d; ++k )
  {
    values[ pos_v + k ] += in    [ pos_v + k ];
    in    [ pos_v + k ]  = values[ pos_v + k ] / Ztau;
  }
  
}


extern "C" void smooth_with_conf( float* values, float* confidence, 
                                    int w, int h, int d, 
                                    float* in, float* in_c, 
                                    float Ztau )
{
    dim3 dimBlock2d = GPUData_dimBlock2d;
	  dim3 dimGrid2d(iDivUp(w,dimBlock2d.x),iDivUp(h,dimBlock2d.y));
    d_smooth_with_conf<<<dimGrid2d, dimBlock2d>>>( values, confidence, w, h, d, in, in_c, Ztau );
}


extern "C" void smooth( float* values, 
                        int w, int h, int d, 
                        float* in, 
                        float Ztau  )
{
      dim3 dimBlock2d = GPUData_dimBlock2d;
	  dim3 dimGrid2d(iDivUp(w,dimBlock2d.x),iDivUp(h,dimBlock2d.y));
    d_smooth<<<dimGrid2d, dimBlock2d>>>( values, w, h, d, in, Ztau );
  
}
