/*

GPUIMAGEPROCESSING

  THIS MODULE PROVIDES A SIMPLE API FOR IMAGE OPERATIONS ON
  A GRAPHICAL PROCESSING UNIT (GPU) BY MAKING USE OF CUDA 
  TECHNOLOGY.

COPYRIGHT (C)

  ULM UNIVERSITY 
  INSTITUTE OF NEURAL INFORMATION PROCESSING (NEUROINFORMATIK)
  89069 ULM
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  INSTITUTE OF NEURAL INFORMATION PROCESSING:
  http://www.uni-ulm.de/en/in/institute-of-neural-information-processing.html

  ULM UNIVERSITY 
  HTTP://WWW.UNI-ULM.DE/

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
#include <cuda.h>
#include "cutil.h"

__global__ void d_subsBy2(float* origin, float* dest, const int NxO, const int NyO, const int pitchO, const int pitchD) {

	// no of threads per block
	const int noOfThreadsX = blockDim.x;
	const int noOfThreadsY = blockDim.y;

	// block Idx
	const int bIndexX = blockIdx.x;
	const int bIndexY = blockIdx.y;

	// global indexO
	const int indexX  = bIndexX * noOfThreadsX + threadIdx.x;
	const int indexY  = bIndexY * noOfThreadsY + threadIdx.y;
	const int index   = indexX+indexY*pitchO;

	// global indexD
	const int indexXD  = (indexX)/2;
	const int indexYD  = (indexY)/2;
	const int indexD   = indexXD+indexYD*pitchD;

	// local index

	const int indexL = threadIdx.x+threadIdx.y*noOfThreadsX;

	__shared__ float temp[16*8];

	if (indexX<NxO && indexY<NyO)
		temp[indexL] = origin[index];
	else
		temp[indexL] = 0.0f;

	__syncthreads();

	if (threadIdx.x%2==0) {
		temp[indexL] = temp[indexL]+temp[indexL+1];
	}

	__syncthreads();

	if (threadIdx.y%2==0 && threadIdx.x%2==0 && indexX<NxO && indexY<NyO) {
		dest[indexD] = temp[indexL]+temp[indexL+noOfThreadsX];
	}
}

extern "C" void doSubsBy2(float* origin, float* dest, const int NxO, const int NyO, const int pitchO, const int pitchD)
{
  //dim3 dimBlock2d = GPUData_dimBlock2d;
	//dim3 dimGrid2d(iDivUp(NxO,dimBlock2d.x),iDivUp(NyO,dimBlock2d.y));
	dim3 dimBlock(16,8);
  dim3 dimGrid(iDivUp(NxO,dimBlock.x),iDivUp(NyO,dimBlock.y));

    d_subsBy2<<<dimGrid, dimBlock>>>(origin,dest, NxO, NyO, pitchO, pitchD);
}
