/*

GPUIMAGEPROCESSING

  THIS MODULE PROVIDES A SIMPLE API FOR IMAGE OPERATIONS ON
  A GRAPHICAL PROCESSING UNIT (GPU) BY MAKING USE OF CUDA 
  TECHNOLOGY.

COPYRIGHT (C)

  ULM UNIVERSITY 
  INSTITUTE OF NEURAL INFORMATION PROCESSING (NEUROINFORMATIK)
  89069 ULM
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  INSTITUTE OF NEURAL INFORMATION PROCESSING:
  http://www.uni-ulm.de/en/in/institute-of-neural-information-processing.html

  ULM UNIVERSITY 
  HTTP://WWW.UNI-ULM.DE/

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
#ifndef CENSUSCLASS_H_
#define  CENSUSCLASS_H_

#include <HypoImage.h>
#include <vector>
#include "compute_hypos.hcu"
#include "census.hpp"

class CensusClass : public census::CensusValue
{

public:

	CensusClass(void);
	CensusClass(unsigned int, int, int, int, int);
	~CensusClass(void);

	void computeHyposUnambiguous(CensusClass*, unsigned int *currCensusList, float*, HypoImage &, int, int, int, int);

	void computeHyposUnambiguous(CensusClass*, unsigned int *currCensusList, float*, CPUData<float> & out, int, int, int, int);

	void computeHyposAmbiguous( float* currCensusList, 
								HypoImage HypoArrayV1, 
								HypoImage HypoArrayMT, 
								GPUImage censusOccDev, 
								GPUImage currCensus, 
								GPUImage pastCensus, 
								int fbf, int H_MAX_2, int VEL_MAX, float p0 = 1.0f );

	void computeHyposAmbiguous( float* currCensusList, 
								HypoImage HypoArrayV1, 
								const std::vector<HypoFeed> & feeds ,
								GPUImage censusOccDev, 
								GPUImage currCensus, 
								GPUImage pastCensus, 
								int H_MAX_2, int VEL_MAX, float p0 = 1.0f );


	void createSortedCensusList(unsigned int *, int, int);

};

class SequentialCensusMatcher
{

 public:
  
  SequentialCensusMatcher( );
  
  void break_sequence();
  void resize( size_t width, size_t height );
  void set_params( size_t vmax, size_t hmax1, size_t hmax2, float fbf );
  void take_census_image( GPUImage & );

  void match_initial( float velo_power = 0.0f );
  void use_feedback( HypoImage & feedback, float p0 = 1.0f );
  void use_feedback( const std::vector<HypoFeed> & feedbacks, float p0 );
  
  const CPUData<float> & initial_matches();
  const CPUData<float> & initial_ambiguities();
  const HypoImage      & final_matches();

  size_t width()  { return _width ; }
  size_t height() { return _height; }

  bool ready();

 private:

  typedef CensusClass CensusEntry;
  typedef std::vector<CensusEntry> CensusEntries;

  size_t _width, _height, _vmax, _hmax1, _hmax2;
  float _fbf;

  bool _have_current, _have_last, _gpu_hypos_valid;

  CensusEntries _entries1, _entries2;
  CensusEntries & _current_entries,  & _last_entries;

  CPUData<float> _matches;
  HypoImage _gpuhypos;
  GPUImage _census_image1, _census_image2, _gpuambiguities;
  std::vector<unsigned int> _census_cpu_buffer;

  GPUImage & _current_census_image, & _last_census_image;
  CPUData<float> _ambiguities;

};


#endif
