/*

GPUIMAGEPROCESSING

  THIS MODULE PROVIDES A SIMPLE API FOR IMAGE OPERATIONS ON
  A GRAPHICAL PROCESSING UNIT (GPU) BY MAKING USE OF CUDA 
  TECHNOLOGY.

COPYRIGHT (C)

  ULM UNIVERSITY 
  INSTITUTE OF NEURAL INFORMATION PROCESSING (NEUROINFORMATIK)
  89069 ULM
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  INSTITUTE OF NEURAL INFORMATION PROCESSING:
  http://www.uni-ulm.de/en/in/institute-of-neural-information-processing.html

  ULM UNIVERSITY 
  HTTP://WWW.UNI-ULM.DE/

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
#ifndef CLASSICUULMNEURALMODEL_H_
#define CLASSICUULMNEURALMODEL_H_

#include "GPUVelActivities.h"
#include "kernels.hpp"

class ClassicUUlmNeuralModel
{

public:
	ClassicUUlmNeuralModel(GPUVelActivities* inputPtrA,
						   float cA = 100.0f,
						   int   donwSampelFactorA=5,
						   float sigma4DownsamplingA=7.0f,
						   float sigma4VelBlurringA=0.75f,
						   float sigmaSpatialInhibitionV1=0.0f,
						   float sigmaSpatialInhibitionMT=0.0f);
	virtual ~ClassicUUlmNeuralModel();

protected:
	int 	downSampelFactorE;
	float 	sigma4Downsampling;
	float	sigma4VelBlurring;
	float	sigmaSpatialInhibitionV1;
	float 	sigmaSpatialInhibitionMT;
	float	cE;

	GPUData spaceKernelE;
	GPUData velKernelE;
	GPUData spaceInhibKernelV1;
	GPUData spaceInhibKernelMT;

	bool useSpaceInhibKernelV1;
	bool useSpaceInhibKernelMT;

	GPUVelActivities* inputPtrE;
	/*
	 * real temp space
	 */
	GPUVelActivities tempInput1E;
	/*
	 * shares memory with the input!
	 */
	GPUVelActivities  tempInput2E;
	/**
	 * down-sample-view to tempInput1E
	 */
	GPUVelActivities  tempInput1DownSampelViewE;
	/*
	 * pointer to feedback date
	 */
	GPUVelActivities* feedbackPtrE;
	/*
	 * shares memory with tempInput2E and thus with the input
	 */
	GPUVelActivities  tempOutputE;

	GPUData			  ones;
	GPUData			  onesSmall;
	GPUData 		  tempInhibE;
	GPUImage		  tempInhib2DViewE;
	GPUData 		  tempInhibSmallE;
	GPUImage		  tempInhibSmall2DViewE;

	// Arrays for the spatial inhibition
	GPUData tempSpatialInhibV1;
	GPUData tempSpatialInhibMT;

	/*
	 * real temp space
	 */
	GPUVelActivities  outputE;

public:
	/**
	 * iterate V1
	 *
	 * This function uses the input from inputPtrE and feedbackPtrE
	 * (if set). During this calculation, the content of inputPtr
	 * is overwritten.
	 * It is valid to use tempInput1E (returned by getOutputWithInputSize()
	 * as feedback input to the system if you don't use it again after
	 * calling this function.
	 */
	void iterateV1();

	/**
	 * iterate MT
	 *
	 * This function destroys the content of inputPtr
	 * It is not allowed to call getOutpuDataWithInputSize() between the calls
	 * of iterateV1() and this function.
	 */
	void iterateMT();

	/**
	 * data, returned by this function is only valid after calling
	 * iterateV1() and before calling iterateMT(). Calling
	 * getOutputWithInputSize() destroys V1 results.
	 */
	GPUVelActivities& getV1Output();
	/*
	 * returns the result of MT
	 */
	GPUVelActivities& getMTOutput();
	/**
	 * the memory, returned by this function is identical to tempImage1E
	 */
	GPUVelActivities& getMTOutputWithInputSize(bool incudePredictionShift = false);

	void setFeedbackInput(GPUVelActivities& feedbackA);

private:
	void init();
};

#endif /*CLASSICUULMNEURALMODEL_H_*/



// Local Variables:
// mode: c++
// coding: utf-8
// c-basic-offset: 4
// tab-width: 4
// End:
