/*

GPUIMAGEPROCESSING

  THIS MODULE PROVIDES A SIMPLE API FOR IMAGE OPERATIONS ON
  A GRAPHICAL PROCESSING UNIT (GPU) BY MAKING USE OF CUDA 
  TECHNOLOGY.

COPYRIGHT (C)

  ULM UNIVERSITY 
  INSTITUTE OF NEURAL INFORMATION PROCESSING (NEUROINFORMATIK)
  89069 ULM
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  INSTITUTE OF NEURAL INFORMATION PROCESSING:
  http://www.uni-ulm.de/en/in/institute-of-neural-information-processing.html

  ULM UNIVERSITY 
  HTTP://WWW.UNI-ULM.DE/

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
#ifndef ELABREICHARDT_H_
#define ELABREICHARDT_H_

#include "GPUImage.h"
#include "GPUVelActivities.h"
#include "GPUVelField.h"
#include "SepGaborWaveletFilter.h"
#include <vector>

class ElabReichardtDetector {
	
private:
	
	GPUData gauss;

	int maxVelX;
	int maxVelY;
	
	bool evenFilters;
	bool oddFilters;

	std::vector< SepGaborWaveletFilter* > allFilters;

	GPUImage tempImage;
	GPUImage tempImage2;
	GPUImage tempImageN;
	
	GPUImage tempOplus;
	GPUImage tempOminus;
	
	std::vector<GPUImage*> tempImages;
	
public:
	
	ElabReichardtDetector(int resX, int resY, int maxVel, int orientations);
	~ElabReichardtDetector();

	std::vector<GPUImage*>* createTempImgagesForPreFiltering();

	void preFilter(GPUImage& image, std::vector<GPUImage*>& filterdImages);
	
	void correlate(std::vector<GPUImage*>& images1, std::vector<GPUImage*>& images2, int shiftX, int shiftY, GPUImage& correlationResult);

	/**
	 * calculates the correlation for all velocities and stores them all together into correlationResult. 
	 */
	void correlateAll(std::vector<GPUImage*>& images1, std::vector<GPUImage*>& images2, GPUVelActivities& correlationResult);
	
	
	void correlateAll(std::vector<GPUImage*>& images1, std::vector<GPUImage*>& images2, GPUVelField& velField);
	
	GPUImage getXVelMatrix() const;
	GPUImage getYVelMatrix() const;
};


#endif

