/*

GPUIMAGEPROCESSING

  THIS MODULE PROVIDES A SIMPLE API FOR IMAGE OPERATIONS ON
  A GRAPHICAL PROCESSING UNIT (GPU) BY MAKING USE OF CUDA 
  TECHNOLOGY.

COPYRIGHT (C)

  ULM UNIVERSITY 
  INSTITUTE OF NEURAL INFORMATION PROCESSING (NEUROINFORMATIK)
  89069 ULM
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  INSTITUTE OF NEURAL INFORMATION PROCESSING:
  http://www.uni-ulm.de/en/in/institute-of-neural-information-processing.html

  ULM UNIVERSITY 
  HTTP://WWW.UNI-ULM.DE/

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
#include "GPUComplexImage.hpp"

GPUComplexImage::GPUComplexImage(int w, int h)
	: GPUFeatureImage(w,h,2)
{
	;
}

GPUComplexImage::GPUComplexImage(const GPUHermRedFourierImage& part)
	: GPUFeatureImage(part.getOriginalWidth(),part.getOriginalHeigth(),2)
{

	int w_small = part.extent(1);
	int w = part.getOriginalWidth();
	int h = part.extent(0);

	int offs_for_copy_left[3]   = {0,0,0};
	size_t size_of_copy[3]      = {h,w_small,2};

	view(offs_for_copy_left,size_of_copy).copyFrom(part);

	//TODO: make this more efficient
	GPUHermRedFourierImage part_complex_conjugated(part.copy());
	part_complex_conjugated.getImagPartView().mul(-1.0f);

	{ // for zero frequency
		size_t size_for_reverse_copy[3]   = {1,(w+1)/2-1,2};
		int offs_for_reverse_copy_destination[3] = {0,w_small,0};
		int offs_for_reverse_copy_origin[3]  = {0,(w+1)/2-1,0};
		int stride_for_reverse_copy[3]  = {part.getStrides()[0],-part.getStrides()[1],part.getStrides()[2]};

		view(offs_for_reverse_copy_destination,size_for_reverse_copy).copyFrom(
				part_complex_conjugated.view(offs_for_reverse_copy_origin,3,size_for_reverse_copy,stride_for_reverse_copy));
	}
	{ // for the rest
	   size_t size_for_reverse_copy[3]   = {h-1,(w+1)/2-1,2};
		int offs_for_reverse_copy_destination[3] = {1,w_small,0};
		int offs_for_reverse_copy_origin[3]  = {h-1,(w+1)/2-1,0};
		int stride_for_reverse_copy[3]  = {-part.getStrides()[0],-part.getStrides()[1],part.getStrides()[2]};

		view(offs_for_reverse_copy_destination,size_for_reverse_copy).copyFrom(
				part_complex_conjugated.view(offs_for_reverse_copy_origin,3,size_for_reverse_copy,stride_for_reverse_copy));
	}

//	int w_small = part.extent(1);
//	int w = part.getOriginalWidth();
//	int h = part.extent(0);
//
//	int offs_for_copy_left[3]   = {0,0,0};
//	int size_of_copy[3]         = {h,w_small,2};
//
//	view(offs_for_copy_left,size_of_copy).copyFrom(part);
//
//	int size_for_reverse_copy[3]   = {h,(w+1)/2-1,2};
//	int offs_for_reverse_copy_destination[3] = {0,w_small,0};
//	int offs_for_reverse_copy_origin[3]  = {0,(w+1)/2-1,0};
//	int stride_for_reverse_copy[3]  = {part.getStrides()[0],-part.getStrides()[1],part.getStrides()[2]};
//
//	//TODO: make this more efficient
//	GPUHermRedFourierImage part_complex_conjugated(part.copy());
//	part_complex_conjugated.getImagPartView().mul(-1.0f);
//
//	view(offs_for_reverse_copy_destination,size_for_reverse_copy).copyFrom(
//			part_complex_conjugated.view(offs_for_reverse_copy_origin,3,size_for_reverse_copy,stride_for_reverse_copy));
//
//	int w_small = part.extent(1);
//	int w = part.getOriginalWidth();
//	int h = part.extent(0);
//
//	int offs_for_copy_left[3]   = {0,0,0};
//	int size_of_copy[3]         = {h,w_small,2};
//
//	view(offs_for_copy_left,size_of_copy).copyFrom(part);
//
//	int size_for_reverse_copy[3]   = {h,(w-1)/2,2};
//	int offs_for_reverse_copy_destination[3] = {0,w_small,0};
//	int offs_for_reverse_copy_origin[3]  = {0,(w-1)/2-1,0};
//	int stride_for_reverse_copy[3]  = {part.getStrides()[0],-part.getStrides()[1],part.getStrides()[2]};
//
//	//TODO: make this more efficient
//	GPUHermRedFourierImage part_complex_conjugated(part.copy());
//	part_complex_conjugated.getImagPartView().mul(-1.0f);
//
//	view(offs_for_reverse_copy_destination,size_for_reverse_copy).copyFrom(
//			part_complex_conjugated.view(offs_for_reverse_copy_origin,3,size_for_reverse_copy,stride_for_reverse_copy));

}

GPUComplexImage::GPUComplexImage(const GPUData& d)
	: GPUFeatureImage(d)
{
	if ( d.getDimensionsCount()!=3 ) {
		throw GPUDataException("GPUComplexImage has to be 3-dimensional");
	}
}

GPUComplexImage& GPUComplexImage::operator=(const GPUImage& im) {
	getRealPartView().copyFrom(im);
	return *this;
}

GPUComplexImage GPUComplexImage::ifft2() const {
	if (!isContiguous())
		throw GPUDataNYIException("ifft of non contiguous data not yet implemented");

	const int w = getWidth();
	const int h = getHeight();

	GPUComplexImage resul(w,h);

	cufftResult   cufft_return;
	cufftHandle*  plan;
	plan_key 	  p_key(CUFFT_C2C,h,w);

	if (getPlanMap().count(p_key)>0) {
		plan = getPlanMap()[p_key];
	} else {
		plan = p_key.create_plan();
		getPlanMap()[p_key] = plan;
	}

	cufftComplex *idata = (cufftComplex*) data();
	cufftComplex *odata = (cufftComplex*) resul.getDataPointer();

	/* Use the CUFFT plan to transform the signal out of place. */
	cufft_return = cufftExecC2C(*plan, idata, odata, CUFFT_INVERSE);
	if (cufft_return!=CUFFT_SUCCESS)
		throw GPUDataException("error executin cufft in GPUComplexImage::ifft()");

	resul.div(float(w*h));

	return resul;
}

GPUComplexImage GPUComplexImage::fft2() const {
	if (!isContiguous())
		throw GPUDataNYIException("fft of non contiguous data not yet implemented");

	const int w = getWidth();
	const int h = getHeight();

	GPUComplexImage resul(w,h);

	cufftResult   cufft_return;
	cufftHandle*  plan;
	plan_key 	  p_key(CUFFT_C2C,h,w);

	if (getPlanMap().count(p_key)>0) {
		plan = getPlanMap()[p_key];
	} else {
		plan = p_key.create_plan();
		getPlanMap()[p_key] = plan;
	}

	cufftComplex *idata = (cufftComplex*) data();
	cufftComplex *odata = (cufftComplex*) resul.getDataPointer();

	/* Use the CUFFT plan to transform the signal out of place. */
	cufft_return = cufftExecC2C(*plan, idata, odata, CUFFT_FORWARD);
	if (cufft_return!=CUFFT_SUCCESS)
		throw GPUDataException("error executin cufft in GPUComplexImage::fft()");

	return resul;
}

GPUImage GPUComplexImage::getRealPartView() const
{
	return GPUImage( slice(2,0) );
}
GPUImage GPUComplexImage::getImagPartView() const
{
	return GPUImage( slice(2,1) );
}
GPUImage GPUComplexImage::getAmplitude() const {
	GPUImage rtv(extent(1),extent(0));
	rtv.pythagoras(getRealPartView(),getImagPartView());
	return rtv;
}
GPUImage GPUComplexImage::getPhase() const {
	//TODO: implement
	throw GPUDataNYIException("GPUComplexImage::getPhase()");
}

extern "C" void do_complexMul(const int N, float* res, const float* a, const float* b);
void GPUComplexImage::mulWithComplexFilter(const GPUComplexImage& filter)
{
	const size_t s = size();
	if (isContiguous() && filter.isContiguous() && s==filter.size()) {
	  do_complexMul(s/2,data(),data(),filter.getDataPointer());
	} else {
		throw GPUDataNYIException("GPUComplexImage::mulWithComplexFilter()");
	}
}

void GPUComplexImage::mulWithRealFilter(const GPUImage& filter) {
	getRealPartView().mul(filter);
	getImagPartView().mul(filter);
}
