/*

GPUIMAGEPROCESSING

  THIS MODULE PROVIDES A SIMPLE API FOR IMAGE OPERATIONS ON
  A GRAPHICAL PROCESSING UNIT (GPU) BY MAKING USE OF CUDA 
  TECHNOLOGY.

COPYRIGHT (C)

  ULM UNIVERSITY 
  INSTITUTE OF NEURAL INFORMATION PROCESSING (NEUROINFORMATIK)
  89069 ULM
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  INSTITUTE OF NEURAL INFORMATION PROCESSING:
  http://www.uni-ulm.de/en/in/institute-of-neural-information-processing.html

  ULM UNIVERSITY 
  HTTP://WWW.UNI-ULM.DE/

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
#ifndef GPUCOMPLEXIMAGE
#define GPUCOMPLEXIMAGE

#include <GPUData.h>
#include "GPUFeatureImage.hpp"
#include "GPUImage.h"
#include "GPUHermRedFourierImage.hpp"

/**
 * This class represents images with complex valued points. Thus the
 * actual data is 3-dimensional (height,width,real_or_imag) with a size
 * of 2 in the deepest dimension.
 */
class GPUComplexImage : public GPUFeatureImage {

public:

	GPUComplexImage(int w, int h);
	/**
	 * This constructor extends the herm reduced spectrum
	 * to the full size.
	 * For This, the data is copied!
	 */
	GPUComplexImage(const GPUHermRedFourierImage& d);
	GPUComplexImage(const GPUData& d);

	GPUComplexImage& operator=(const GPUImage& im);

	inline int getXRes() const { return extent(1); }
	inline int getYRes() const { return extent(0); }

	inline int getWidth() const { return extent(1); }
	inline int getHeight() const { return extent(0); }

	/**
	 * computes a 2-dimensional inverse, not normalized fft
	 */
	GPUComplexImage ifft2() const;

	/**
	 * computes a 2-dimensional, not normalized fft
	 */
	GPUComplexImage fft2() const;

	void mulWithComplexFilter(const GPUComplexImage& filter);
	void mulWithRealFilter(const GPUImage& filter);

	GPUImage getRealPartView() const;
	GPUImage getImagPartView() const;
	GPUImage getAmplitude() const;
	GPUImage getPhase() const;
};

#endif
