/*Copyright or  or Copr. 
PSPC-lab - Biophysical and Electronic Engineering Department - University of Genoa
contributors: Manuela Chessa, Silvio P. Sabatini, Fabio Solari (February, 2011)
manuela.chessa@unige.it, silvio.sabatini@unige.it, fabio.solari@unige.it

This software is a computer program whose purpose is to compute the horizontal 
and vertical disparity maps from a stereo image pair.

This software is governed by the CeCILL-C  license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL-C
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.*/

#include "GPUDisparity.h"
#include "cuda.h"


void GPUDisparity::pyramid()
{
	left_pyr[n_scale-1]=left;
	right_pyr[n_scale-1]=right;


	for(int i=n_scale-2; i>=0; i--)
	{
	
		left_pyr[i+1]->extrapolate(*left_pyr[i],true);
              	right_pyr[i+1]->extrapolate(*right_pyr[i],true);
	}
	for(int i=0; i<n_scale; i++)
	{

		HostImage im(left_pyr[i]->getWidth(),left_pyr[i]->getHeight(),HostImage::FLOAT,HostImage::LUM);
		unsigned int mem[left_pyr[i]->getWidth()*left_pyr[i]->getHeight()];
		left_pyr[i]->copyToHostMemory((float*)&mem);
		left_pyr[i]->copyToHostMemory((float*)im.getPointerToImage());
		char nomefile[256];
		sprintf(nomefile,"%d.pgm",i);
		//im.writeImage(nomefile);
	}
}

void GPUDisparity::work()
{

	float* DXexp;
	float* DYexp;
	float *DX;
	float *DY;

	for(int scale=0; scale<n_scale; scale++)
	{
		data_size=left_pyr[scale]->getWidth()*left_pyr[scale]->getHeight()*sizeof(float);
		data_w=left_pyr[scale]->getWidth();
		data_h=left_pyr[scale]->getHeight();

		d_DataA[0]=left_pyr[scale]->getDataPointer();
		d_DataA[1]=right_pyr[scale]->getDataPointer();


		if(scale!=0)
		{
			cudaFree(DX);
			cudaFree(DY);
		}


		cudaMalloc( (void **)&DX, data_size);
		cudaMalloc( (void **)&DY, data_size);


		if(scale==0)
			Disparity(d_DataA, DX, DY,scale,n_scale,data_size,data_w,data_h);
		else
		    Disparity(d_DataA, DX, DY,scale,n_scale,data_size,data_w,data_h,DXexp,DYexp);


		if(scale!=0)
		{
			cudaFree( DXexp );
			cudaFree( DYexp );
		}


		cudaMalloc( (void **)&DXexp, left_pyr[scale]->getWidth()*left_pyr[scale]->getHeight()*sizeof(float)*4);
		cudaMalloc( (void **)&DYexp, left_pyr[scale]->getWidth()*left_pyr[scale]->getHeight()*sizeof(float)*4);


		if(scale!=n_scale-1)
		{

			Expand(DX, DXexp, left_pyr[scale]->getWidth(), left_pyr[scale]->getHeight(), left_pyr[scale]->getWidth()*left_pyr[scale]->getHeight()*sizeof(float));
			Expand(DY, DYexp, left_pyr[scale]->getWidth(), left_pyr[scale]->getHeight(), left_pyr[scale]->getWidth()*left_pyr[scale]->getHeight()*sizeof(float));
		}

		if(scale==n_scale-1)
		{

			h_DX = (float *) malloc(left_pyr[scale]->getWidth()*left_pyr[scale]->getHeight()*sizeof(float));
			h_DY = (float *) malloc(left_pyr[scale]->getWidth()*left_pyr[scale]->getHeight()*sizeof(float));

			cudaMemcpy(h_DX, DX, left_pyr[scale]->getWidth()*left_pyr[scale]->getHeight()*sizeof(float), cudaMemcpyDeviceToHost);
			cudaMemcpy(h_DY, DY, left_pyr[scale]->getWidth()*left_pyr[scale]->getHeight()*sizeof(float), cudaMemcpyDeviceToHost);

		}

	}


}


void GPUDisparity::writeToFile(char* dispxout, char* dispyout)
{

	float mx=100;
	float Mx=-100;
	float my=100;
	float My=-100;
	for(int i=0; i< left_pyr[n_scale-1]->getWidth()*left_pyr[n_scale-1]->getHeight(); i++)
	{
		if(h_DX[i]<mx && (!isnan(h_DX[i])))
			mx=h_DX[i];
		if(h_DX[i]>Mx && (!isnan(h_DX[i])))
			Mx=h_DX[i];
		if(h_DY[i]<my && (!isnan(h_DY[i])))
			my=h_DY[i];
		if(h_DY[i]>My && (!isnan(h_DY[i])))
			My=h_DY[i];
	}
	unsigned char* h_DX_ub = (unsigned char*)malloc(left_pyr[n_scale-1]->getWidth()*left_pyr[n_scale-1]->getHeight()*sizeof(unsigned char));
	unsigned char* h_DY_ub = (unsigned char*)malloc(left_pyr[n_scale-1]->getWidth()*left_pyr[n_scale-1]->getHeight()*sizeof(unsigned char));
	for(int i=0; i< left_pyr[n_scale-1]->getWidth()*left_pyr[n_scale-1]->getHeight(); i++)
	{
		if(!isnan(h_DX[i]))
			h_DX_ub[i]=(unsigned char)(255*(h_DX[i]-mx)/(Mx-mx));
		if(!isnan(h_DY[i]))
			h_DY_ub[i]=(unsigned char)(255*(h_DY[i]-my)/(My-my));
	}
	FILE * fp=fopen(dispxout,"wb");
	fwrite(h_DX_ub,sizeof(unsigned char),left_pyr[n_scale-1]->getWidth()*left_pyr[n_scale-1]->getHeight(),fp);
	fclose(fp);
	//cutSavePGMub(dispxout,h_DY_ub,left_pyr[n_scale-1]->getWidth(),left_pyr[n_scale-1]->getHeight());
	//cutSavePGMub(dispyout,h_DX_ub,left_pyr[n_scale-1]->getWidth(),left_pyr[n_scale-1]->getHeight());
	free(h_DX_ub);
	free(h_DY_ub);
}

void GPUDisparity::dispose()
{
  cudaFree(d_DataA[0]);
  cudaFree(d_DataA[1]);
  free(h_DX);
  free(h_DY);
}






