/*Copyright or  or Copr. 
PSPC-lab - Biophysical and Electronic Engineering Department - University of Genoa
contributors: Manuela Chessa, Silvio P. Sabatini, Fabio Solari (February, 2011)
manuela.chessa@unige.it, silvio.sabatini@unige.it, fabio.solari@unige.it

This software is a computer program whose purpose is to compute the horizontal 
and vertical disparity maps from a stereo image pair.

This software is governed by the CeCILL-C  license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL-C
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.*/

#ifndef INC_H
#define INC_H

#include "cutil.h"
#include <cuda_runtime_api.h>
#include <cuda_runtime.h>
#include <cuda.h>
//#include <GPUData.h>
////////////////////////////////////////////////////////////////////////////////
// Common host and device functions
////////////////////////////////////////////////////////////////////////////////

extern "C" void Disparity(float **I, float *DX, float *DY, int scale,int n_scale, int data_size, int data_w, int data_h, float *DXwarp=0, float* DYwarp=0);
extern "C" void shift_in_phase(float *even,float* odd, float** even_r, float** odd_r, int data_w, int data_h,int data_size,float* temp11, float* temp22);
extern "C" void compute_energy(float* even, float* odd, float** even_r, float** odd_r, float** E, int n_ph, int data_size, int data_w, int data_h,float* temp11, float* temp22);
extern "C" void CudaWarping(float* input, float* output, float *d_X, float *d_Y, int data_w, int data_h, int data_size);
extern "C" void Expand(float* input, float* output, int data_w, int data_h, int data_size);


__global__ void convolutionRowGPU(float *d_Result,float *d_Data,int dataW,int dataH);
__global__ void convolutionColumnGPU(float *d_Result,float *d_Data,int dataW,int dataH,int smemStride,int gmemStride);
__global__ void matrixSumGPU(float *d_Result,float *d_A,float *d_B,int dataW);
__global__ void matrixSumGPU_temp(float *d_Result,float *d_A,float *d_B,int dataW);
__global__ void matrixSubGPU_temp(float *d_Result,float *d_A,float *d_B, int dataW);
__global__ void matrixSubGPU(float *d_Result,float *d_A,float *d_B, int dataW);
__global__ void Sqr(float *d_Result,float *d_A,float *d_B,int dataW);
__global__ void Decode(
	float *d_Result_x,
	float *d_Result_y,
	float **d_A,  //orientation fixed (n_ph x data_size)
    int dataW,
	int ori
);
__global__ void FullDisparity(
	float *d_Result_x, //horizontal disparity
	float *d_Result_y, //vertical disparity
	float **dx,		   //horizontal disparity along each orientation
	float **dy,		//vertical disparity along each orientation
    int dataW
);
__global__ void inverseWarping_kernel(
										float *glob_x,
										float *glob_y, 
										float *glob_result,
										int dataW,
										int dataH );
__global__ void expandOnly_kernel(
				float *glob_result,
				int dataW,
				int dataH );


//Global macro, controlling innermost convolution loop unrolling
#define UNROLL_INNER


////////////////////////////////////////////////////////////////////////////////
// Kernel configuration
////////////////////////////////////////////////////////////////////////////////
#define KERNEL_RADIUS 5
#define      KERNEL_W (2 * KERNEL_RADIUS + 1)


// Assuming ROW_TILE_W, KERNEL_RADIUS_ALIGNED and dataW 
// are multiples of coalescing granularity size,
// all global memory operations are coalesced in convolutionRowGPU()
#define            ROW_TILE_W 128
#define KERNEL_RADIUS_ALIGNED 16

// Assuming COLUMN_TILE_W and dataW are multiples
// of coalescing granularity size, all global memory operations 
// are coalesced in convolutionColumnGPU()
#define COLUMN_TILE_W 16
#define COLUMN_TILE_H 48

//Image width should be aligned to maximum coalesced read/write size
//for best global memory performance in both row and column filter.

const int   KERNEL_SIZE = KERNEL_W * sizeof(float);

#define SUM_TILE_W	16
#define SUM_TILE_H	4

#define TEMP_W	32
#define TEMP_H	16

#define SUM_TILE_W_D	16
#define SUM_TILE_H_D	4

#define WARP_TILE_W	16
#define WARP_TILE_H	16	

const int n_ph=9;	//phase shifts (number of cells for each orientation)
const int n_ori=8;	//number of orientation

//24-bit multiplication is faster on G80,
//but we must be sure to multiply integers
//only within [-8M, 8M - 1] range
#define IMUL(a, b) __mul24(a, b)	//CUDA Intrinsic


#define QUIET_NAN	0x7FC00000
#define NEW_NAN	__int_as_float(QUIET_NAN)

// declare texture for 2D float texture

#endif
