/*

GPUIMAGEPROCESSING

  THIS MODULE PROVIDES A SIMPLE API FOR IMAGE OPERATIONS ON
  A GRAPHICAL PROCESSING UNIT (GPU) BY MAKING USE OF CUDA 
  TECHNOLOGY.

COPYRIGHT (C)

  ULM UNIVERSITY 
  INSTITUTE OF NEURAL INFORMATION PROCESSING (NEUROINFORMATIK)
  89069 ULM
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  INSTITUTE OF NEURAL INFORMATION PROCESSING:
  http://www.uni-ulm.de/en/in/institute-of-neural-information-processing.html

  ULM UNIVERSITY 
  HTTP://WWW.UNI-ULM.DE/

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
#include "GPUVelActivities.h"
#include <iostream>
#include <cmath>

GPUVelActivities::GPUVelActivities()
	: realNDO ()
	, owner(true)
{
	;
}

GPUVelActivities::GPUVelActivities(GPUData da)
	: realNDO (da)
	, owner(false)
{
	;
}

GPUVelActivities::GPUVelActivities(int y, int x, int v, int u)
	: realNDO ()
	, owner(true)
{
	realNDO = GPUData(y,x,v,u);
}

GPUVelActivities::GPUVelActivities(GPUData& data, int y, int x, int v, int u)
	: realNDO (data)
	, owner(false)
{
	size_t dims[4];
	dims[0] = y;
	dims[1] = x;
	dims[2] = v;
	dims[3] = u;
	int offs[4]; offs[0]=0; offs[1]=0; offs[2]=0; offs[3]=0;
	int stri[4];
	stri[0] = x*v*u;
	stri[1] = v*u;
	stri[2] = u;
	stri[3] = 1;
	realNDO = data.view(offs,4,dims,stri);
}

void GPUVelActivities::reInit(GPUData& data)
{
	owner = false;
	realNDO = data;
}


GPUVelActivities::~GPUVelActivities()
{
	;
}


bool GPUVelActivities::isImageContiguous() {
    GPUData temp = getOneVelocityAsImage(0,0);    
    return temp.isContiguous() && temp.isRowMajor(); 
}
bool GPUVelActivities::isVelSpaceContiguous() {
    GPUData temp = getOneVelocitySpacesAsImage(0,0);    
    return temp.isContiguous() && temp.isRowMajor(); 
}

GPUData& GPUVelActivities::getData() {
	return realNDO;
}

const GPUData& GPUVelActivities::getData() const
{
	return realNDO;
}

GPUVelField GPUVelActivities::getFlowField() {
	GPUVelField rtv(getXRes(),getYRes());
	getFlowField(rtv);
	return rtv;
}

//void GPUVelActivities::getFlowField(GPUVelField& rtv) {
//	GPUImage    xPart = rtv.getXPart(); xPart.fill(0.0f);
//	GPUImage    yPart = rtv.getYPart(); yPart.fill(0.0f);
//
//	GPUImage    temp(getXRes(),getYRes());
//	GPUImage    temp2(getXRes(),getYRes()); temp2.fill(0.00000001f);
//
//	const int maxVelY = (getVRes()-1)/2;
//	const int maxVelX = (getURes()-1)/2;
//
//	for (int dy=-maxVelY; dy<=maxVelY; ++dy) {
//		for (int dx=-maxVelX; dx<=maxVelX; ++dx)  {
//			temp.copyFrom(getOneVelocityAsImage(dx+maxVelX,dy+maxVelY));
//			xPart.add(temp,float(dx));
//			yPart.add(temp,float(dy));
//			//std::cout << tempImage2.min() << " " << tempImage2.max() << std::endl;
//			temp2.add(temp);
//		}
//	}
//	xPart.div(temp2);
//	yPart.div(temp2);
//
////	temp2.heavisideStepFunction(0.97f);
////
////	//temp2.show();
////
////	xPart.mul(temp2);
////	yPart.mul(temp2);
//}

GPUImage GPUVelActivities::getFeatureSum() {
//	GPUImage rtv(getXRes(),getYRes());
//	GPUData activitySum(rtv.reduceDimensionality());
//
//	if ( onesSizeVelSpace.size() != getURes() * getVRes() )
//		getXValues(); // to init onesSizeVelSpace
//
//	activitySum.matVecMul(get2DView(), onesSizeVelSpace);

	GPUImage    rtv(getXRes(),getYRes()); rtv.fill(0.00000001f);

	const int Y = getVRes();
	const int X = getURes();

	for (int y=0; y<Y; ++y) {
		for (int x=0; x<X; ++x)  {
			rtv.add(getOneVelocityAsImage(x,y));
		}
	}
	return rtv;
}

void GPUVelActivities::getFlowField(GPUVelField& rtv) {

//	GPUData xPart = rtv.getXPart().reduceDimensionality();
//	GPUData yPart = rtv.getYPart().reduceDimensionality();
//	GPUData activitySum(getXRes()*getYRes());
//
//	xPart.matVecMul(get2DView(), getXValues());
//	yPart.matVecMul(get2DView(), getYValues());
//	activitySum.matVecMul(get2DView(), onesSizeVelSpace);
//	activitySum.add(0.000001);
//
//	xPart.div(activitySum);
//	yPart.div(activitySum);

	GPUImage    xPart = rtv.getXPart(); xPart.fill(0.0f);
	GPUImage    yPart = rtv.getYPart(); yPart.fill(0.0f);

	GPUImage    temp(getXRes(),getYRes());
	GPUImage    temp2(getXRes(),getYRes()); temp2.fill(0.00000001f);
//	GPUImage    temp3(getXRes(),getYRes()); temp3.fill(0.0f);

	const int maxVelY = (getVRes()-1)/2;
	const int maxVelX = (getURes()-1)/2;

	if (isImageContiguous()) {
		for (int dy=-maxVelY; dy<=maxVelY; ++dy) {
			for (int dx=-maxVelX; dx<=maxVelX; ++dx)  {
				temp.copyFrom(getOneVelocityAsImage(dx+maxVelX,dy+maxVelY));
				xPart.add(temp,float(dx));
				yPart.add(temp,float(dy));
				temp2.add(temp);
	//			temp3.max(temp);
			}
		}
	} else {

		for (int dy=-maxVelY; dy<=maxVelY; ++dy) {
			for (int dx=-maxVelX; dx<=maxVelX; ++dx)  {
				temp = getOneVelocityAsImage(dx+maxVelX,dy+maxVelY);
				xPart.add(temp,float(dx));
				yPart.add(temp,float(dy));
				temp2.add(temp);
	//			temp3.max(temp);
			}
		}
	}
	temp2.max( 0.0001f );
	xPart.div(temp2);
	yPart.div(temp2);
	float vmax = ( getURes() - 1 ) / 2.0f;
	xPart.clamp( -vmax, vmax );
	yPart.clamp( -vmax, vmax );
}

void GPUVelActivities::getFlowField(GPUVelField& rtv, float relative_hsum_threshold, float unknown_value) 
{

  getFlowField( rtv );

}


//void GPUVelActivities::getFlowFieldSlice(GPUVelField& dest, int xPos, int yPos, int xSize, int ySize) {
//
//	int offs[] = {yPos,xPos,0,0};
//	size_t size[] = {ySize,xSize,getVRes(),getURes()};
////	int stri[] = {getXRes()*getVRes()*getURes() , getVRes()*getURes() , getURes() , 1};
//
//	GPUVelActivities view = GPUVelActivities(realNDO.view(offs,size).copy());
////	GPUVelActivities view = GPUVelActivities(realNDO.view(offs,2,size,stri));
//
//	view.getFlowField(dest);
//}

void GPUVelActivities::shutOfBadPositions(const float maxThreashold) {

	GPUImage    temp(getXRes(),getYRes());
	GPUImage    temp3(getXRes(),getYRes()); temp3.fill(0.0f);

	const int maxVelY = (getVRes()-1)/2;
	const int maxVelX = (getURes()-1)/2;

	for (int dy=-maxVelY; dy<=maxVelY; ++dy) {
		for (int dx=-maxVelX; dx<=maxVelX; ++dx)  {
			temp.copyFrom(getOneVelocityAsImage(dx+maxVelX,dy+maxVelY));
			temp3.max(temp);
		}
	}

//	temp3.setIfBetween(-0.0001f,0.00075f,0.0f);
//	temp3.setIfBetween(-0.0001f,0.00075f,0.0f);

	temp3.heavisideStepFunction(maxThreashold);

	for (int dy=-maxVelY; dy<=maxVelY; ++dy) {
		for (int dx=-maxVelX; dx<=maxVelX; ++dx)  {
			temp.copyFrom(getOneVelocityAsImage(dx+maxVelX,dy+maxVelY));
			temp.mul(temp3);
			getOneVelocityAsImage(dx+maxVelX,dy+maxVelY).copyFrom(temp);
		}
	}
}

void GPUVelActivities::norm_to_sum(const float epsilon) {

	GPUImage    temp(getFeatureSum());
				temp.add(epsilon);
	GPUImage    temp2(temp.getWidth(),temp.getHeight());

	const int maxVelY = (getVRes()-1)/2;
	const int maxVelX = (getURes()-1)/2;

	if (isImageContiguous()) {
		for (int dy=-maxVelY; dy<=maxVelY; ++dy) {
			for (int dx=-maxVelX; dx<=maxVelX; ++dx)  {
				temp2.copyFrom(getOneVelocityAsImage(dx+maxVelX,dy+maxVelY));
				temp2.div(temp);
				getOneVelocityAsImage(dx+maxVelX,dy+maxVelY).copyFrom(temp2);
			}
		}
	} else {
		for (int dy=-maxVelY; dy<=maxVelY; ++dy) {
			for (int dx=-maxVelX; dx<=maxVelX; ++dx)  {
				getOneVelocityAsImage(dx+maxVelX,dy+maxVelY).div(temp);
			}
		}
	}
}

void GPUVelActivities::convoluteInXAndY(GPUData& kernel, GPUVelActivities& temp) {
	realNDO.convolute1D(kernel,0,temp.realNDO);
	temp.realNDO.convolute1D(kernel,1,realNDO);
}

void GPUVelActivities::convoluteInUAndV(GPUData& kernel, GPUVelActivities& temp) {
	realNDO.convolute1D(kernel,3,temp.realNDO);
	temp.realNDO.convolute1D(kernel,2,realNDO);
}

void GPUVelActivities::convoluteInU(GPUData& kernel, GPUVelActivities& dest, bool cyclic) {
	realNDO.convolute1D(kernel,3,dest.realNDO,1);
}

void GPUVelActivities::convoluteInV(GPUData& kernel, GPUVelActivities& dest) {
	realNDO.convolute1D(kernel,2,dest.realNDO);
}

void GPUVelActivities::mulToVelSpace(GPUImage& values) {
	if (realNDO.isContiguous() && realNDO.isRowMajor()) {
		realNDO.mulRepeated(values);
	} else {
		GPUImage    temp2(getXRes(),getYRes());

		const int maxY = getXRes();
		const int maxX = getURes();

		for (int y=0; y<maxY; ++y) {
			for (int x=0; x<maxX; ++x)  {
				temp2.copyFrom(getOneVelocitySpacesAsImage(x,y));
				temp2.mul(values);
				getOneVelocitySpacesAsImage(x,y).copyFrom(temp2);
			}
		}
	}
}

void GPUVelActivities::addToVelSpace(GPUImage& values, const float f) {
	if (realNDO.isContiguous() && realNDO.isRowMajor()) {
		realNDO.add_repeated(values,f);
	} else {
		GPUImage    temp2(getXRes(),getYRes());

		const int maxY = getXRes();
		const int maxX = getURes();

		for (int y=0; y<maxY; ++y) {
			for (int x=0; x<maxX; ++x)  {
				temp2.copyFrom(getOneVelocitySpacesAsImage(x,y));
				temp2.add(values);
				getOneVelocitySpacesAsImage(x,y).copyFrom(temp2);
			}
		}
	}
}


GPUImage GPUVelActivities::getVelocitySpacesInOneImage(bool normalize, int xPos, int yPos, int xSize, int ySize, int xStep, int yStep)
{
	if (xPos==-1)  xPos  = 0;
	if (yPos==-1)  yPos  = 0;
	if (xStep==-1) xStep = 1;
	if (yStep==-1) yStep = 1;
	if (xSize==-1) xSize = getXRes();
	if (ySize==-1) ySize = getYRes();

	GPUImage rtv(((xSize/xStep)*(getURes()+1)),(ySize/xStep)*(getVRes()+1));

	copyAllVelocitySpacesIntoOneImage(xPos,xStep,xSize,yPos,yStep,ySize,rtv,normalize);

	return rtv;
}

void GPUVelActivities::getVelocitySpacesInOneImage(GPUImage& dest, bool normalize, int xPos, int yPos, int xSize, int ySize, int xStep, int yStep)
{
	if (xPos==-1)  xPos  = 0;
	if (yPos==-1)  yPos  = 0;
	if (xStep==-1) xStep = 1;
	if (yStep==-1) yStep = 1;
	if (xSize==-1) xSize = getXRes();
	if (ySize==-1) ySize = getYRes();

	if ( (dest.getWidth() != ((xSize)*(getURes()+1))/xStep) || (dest.getHeight() != ((ySize)*(getVRes()+1))/yStep) ) {
		dest = GPUImage(((xSize)*(getURes()+1))/xStep,(ySize)*(getVRes()+1)/yStep);
	}

	copyAllVelocitySpacesIntoOneImage(xPos,xStep,xSize,yPos,yStep,ySize,dest,normalize);
}

void GPUVelActivities::copyAllVelocitySpacesIntoOneImage(GPUImage& im) {

	copyAllVelocitySpacesIntoOneImage(0,1,realNDO.getDimensions()[1],0,1,realNDO.getDimensions()[0],im,false);
}

void GPUVelActivities::copyAllVelocitySpacesIntoOneImage(int xp, int xs, int yp, int ys, GPUImage& im, bool normalize) {
	copyAllVelocitySpacesIntoOneImage(xp,1,xs,yp,1,ys,im,normalize);
}

void GPUVelActivities::copyAllVelocitySpacesIntoOneImage(int xp, int xstep, int xs, int yp, int ystep, int ys, GPUImage& im, bool normalize) {
	if (!im.isContiguous()) {
		throw GPUDataException("error in The4Dims::copyAllVelocitySpacesInOneImage() : the input image hase to use simple storage");
	}

	const int border = 1;
	const size_t sizeU  = realNDO.extent(3);
	const size_t sizeV  = realNDO.extent(2);

	size_t sizesImage[2]; sizesImage[0] = realNDO.getDimensions()[2]; sizesImage[1] = realNDO.getDimensions()[3];
	int offsetsImage[2];

	if (normalize) {
		im.fill(1/3.0f);
		GPUImage temp(sizeU,sizeV);
		for (int y=0; y<ys; y+=ystep) {
		for (int x=0; x<xs; x+=xstep) {
			temp.copyFrom(getOneVelocitySpacesAsImage(x+xp,y+yp));
			temp.mul(float(1.0/(temp.max()+0.00001)));

			offsetsImage[0] = (sizeV+border)*y/ystep;
			offsetsImage[1] = (sizeU+border)*x/xstep;
			GPUImage subIm = im.view(offsetsImage,sizesImage);

			subIm.copyFrom(temp);
		}
		}
	} else {
		float max = realNDO.max();
		im.fill(max/3.0f);

//		GPUImage temp(sizeU,sizeV);
		for (int y=0; y<ys; y+=ystep) {
		for (int x=0; x<xs; x+=xstep) {
			offsetsImage[0] = (sizeV+border)*y/ystep;
			offsetsImage[1] = (sizeU+border)*x/xstep;
			GPUImage subIm = im.view(offsetsImage,sizesImage);

//			temp.copyFrom(getOneVelocitySpacesAsImage(x+xp,y+yp));
			subIm.copyFrom(getOneVelocitySpacesAsImage(x+xp,y+yp));

//			std::cout << im.absMin() << " " << im.absMax() << std::endl;
//			std::cout << im.absMin() << " " << im.absMax() << std::endl;
		}
		}
	}
}

GPUImage GPUVelActivities::getOneVelocitySpacesAsImage(size_t x, size_t y)
{

	GPUImage im(realNDO[y][x]);

	return im;
}

GPUImage GPUVelActivities::getOneVelocityAsImage(size_t u, size_t v) {

	GPUData data(realNDO);
//
//	const int velDims = realNDO.getDimensions()[2]*realNDO.getDimensions()[3];
//	const size_t sizeX   = realNDO.getDimensions()[1];
//	const size_t sizeY   = realNDO.getDimensions()[0];
//
//	size_t sizesData[2]; sizesData[0] = sizeY; sizesData[1] = sizeX;
//	int stridesData[2]; stridesData[0] = velDims*sizeX; stridesData[1] = velDims;
//	int offsetsData[4]; offsetsData[0] = 0; offsetsData[1] = 0; offsetsData[2] = v; offsetsData[3]=u;
//	
//	GPUImage im(realNDO.view(offsetsData,2,sizesData,stridesData));
	
	data = data.shiftDim(2);

	GPUImage im(data[v][u]);

	return im;
}

void GPUVelActivities::computePrediction() {
    GPUData temp = getOneVelocityAsImage(0,0);    
    bool reverseOrdering = isImageContiguous();
    
	GPUImage    temp1;
				temp1 = reverseOrdering?temp:GPUImage(getXRes(),getYRes());
	GPUImage    temp2(getXRes(),getYRes());

	const int maxVelY = (getVRes()-1)/2;
	const int maxVelX = (getURes()-1)/2;
	if (reverseOrdering) {
		for (int dy=-maxVelY; dy<=maxVelY; ++dy) {
			for (int dx=-maxVelX; dx<=maxVelX; ++dx)  {
				GPUImage view(getOneVelocityAsImage(dx+maxVelX,dy+maxVelY));
	
				temp2.copyFromShifted(view,dx,dy);
	
				view.copyFrom(temp2);
			}
		}
	} else {
		for (int dy=-maxVelY; dy<=maxVelY; ++dy) {
			for (int dx=-maxVelX; dx<=maxVelX; ++dx)  {
				GPUImage view(getOneVelocityAsImage(dx+maxVelX,dy+maxVelY));
	
				temp1.copyFrom(view);
	
				temp2.copyFromShifted(temp1,dx,dy);
	
				view.copyFrom(temp2);
			}
		}
	}
}

void GPUVelActivities::computePredictionAndExtrapolate(GPUVelActivities& dest) {

    if ( realNDO.empty() ) return;
    
    GPUData temp = getOneVelocityAsImage(0,0);    
    bool reverseOrdering = isImageContiguous();

    GPUImage    temp1;
				temp1 = reverseOrdering?temp:GPUImage(getXRes(),getYRes());
	GPUImage    temp2(dest.getXRes(),dest.getYRes());


	const int maxVelY = (getVRes()-1)/2;
	const int maxVelX = (getURes()-1)/2;

	const float subsFa = float(dest.getXRes())/getXRes();
//	const float alphaX = float(dest.getXRes()-1)/(getXRes()-1.0f);
//	const float alphaY = float(dest.getYRes()-1)/(getYRes()-1.0f);
	const float alphaX = float(dest.getXRes())/(getXRes());
	const float alphaY = float(dest.getYRes())/(getYRes());
	const float p1 = 1.0f/alphaX - 1.0f;
	const float p4 = 1.0f/alphaY - 1.0f;
		  float p5;
		  float p6;

	if (reverseOrdering) {
		for (int dy=-maxVelY; dy<=maxVelY; ++dy) {
			for (int dx=-maxVelX; dx<=maxVelX; ++dx)  {
				p5 = -dx/subsFa;
				p6 = -dy/subsFa;
				temp1 = getOneVelocityAsImage(dx+maxVelX,dy+maxVelY);
				temp2.affineInverseWarpeFrom(temp1,p1,0.0f,0.0f,p4,p5,p6,0.0f,0.0f);
				//temp2.copyFromShifted(temp1,dx,dy);
				temp.copyFrom(temp2);
			}
		}
	} else {
		for (int dy=-maxVelY; dy<=maxVelY; ++dy) {
			for (int dx=-maxVelX; dx<=maxVelX; ++dx)  {
				p5 = -dx/subsFa;
				p6 = -dy/subsFa;
				temp1.copyFrom(getOneVelocityAsImage(dx+maxVelX,dy+maxVelY));
				temp2.affineInverseWarpeFrom(temp1,p1,0.0f,0.0f,p4,p5,p6,0.0f,0.0f);
				//temp2.copyFromShifted(temp1,dx,dy);
				dest.getOneVelocityAsImage(dx+maxVelX,dy+maxVelY).copyFrom(temp2);
			}
		}
	}
}

void GPUVelActivities::computeExtrapolation(GPUVelActivities& dest) {
    GPUData temp = getOneVelocityAsImage(0,0);    
    bool reverseOrdering = isImageContiguous();
    
	GPUImage    temp1;
				temp1 = reverseOrdering?temp:GPUImage(getXRes(),getYRes());
	GPUImage    temp2(dest.getXRes(),dest.getYRes());

	const int maxVelY = (getVRes()-1)/2;
	const int maxVelX = (getURes()-1)/2;

	if (reverseOrdering) {
		for (int dy=-maxVelY; dy<=maxVelY; ++dy) {
			for (int dx=-maxVelX; dx<=maxVelX; ++dx)  {
				temp1 = getOneVelocityAsImage(dx+maxVelX,dy+maxVelY);
				temp1.extrapolate(temp2);
				dest.getOneVelocityAsImage(dx+maxVelX,dy+maxVelY).copyFrom(temp2);
			}
		}
	} else {
		for (int dy=-maxVelY; dy<=maxVelY; ++dy) {
			for (int dx=-maxVelX; dx<=maxVelX; ++dx)  {
				temp1.copyFrom(getOneVelocityAsImage(dx+maxVelX,dy+maxVelY));
				temp1.extrapolate(temp2);
				dest.getOneVelocityAsImage(dx+maxVelX,dy+maxVelY).copyFrom(temp2);
			}
		}
	}
}

void GPUVelActivities::killBorder(int borderSizeX, int borderSizeY) {
	const int uSize = getVRes();
	const int vSize = getURes();
	const int xSize = getXRes();
	const int ySize = getYRes();

	int offseUp[] = {0,0,0,0};
	size_t sizesUp[] = {borderSizeY, xSize, vSize, uSize};

	int offseDown[] = {ySize-borderSizeY,0,0,0,0};
	size_t sizesDown[] = {borderSizeY, xSize, vSize, uSize};

	int offseLeft[] = {0,0,0,0};
	size_t sizesLeft[] = {ySize, borderSizeX, vSize, uSize};

	int offseRight[] = {0,xSize-borderSizeX,0,0,0};
	size_t sizesRight[] = {ySize, borderSizeX, vSize, uSize};
	{
		GPUData viewUp(realNDO.view(offseUp, sizesUp));
		GPUData viewDown(realNDO.view(offseDown, sizesDown));
		GPUData viewLeft(realNDO.view(offseLeft, sizesLeft));
		GPUData viewRight(realNDO.view(offseRight, sizesRight));

		viewUp.fill(0.0f);
		viewDown.fill(0.0f);
		viewLeft.fill(0.0f);
		viewRight.fill(0.0f);
	}
}

//void The4Dims::getVelocityExpectation(int x, int y, float& u, float& v, const Image& valX, const Image& valY, float threshold)
//{
//	if (!valX.usesSimpleStorage() || !valY.usesSimpleStorage()) {
//		fprintf(stderr,"error in The4Dims::getVelocityExpectation(...) : complex storage not yet supported");
//		exit(-1);
//	}
//	const int offset =(x+y*sizeX)*sizeU*sizeV;
//	const int xSize = ((Image&)valX).realNDO.getSize();
//	GPUData xData = ((Image&)valX).getData();
//	const int ySize = ((Image&)valY).realNDO.getSize();
//	GPUData yData =	((Image&)valY).getData();
//	float sum = realNDO->sum(offset,xSize);
////	float max = realNDO->maxPart(0,realNDO->getSize());
////	if (sum<0.1*3) {
//	if (sum<threshold) {
//		sum = 1000;
//		u = 0;
//		v = 0;
//	} else {
//		u = realNDO->mulAndSum(0,1,*(xData),offset,1,xSize)/sum;
//		v = realNDO->mulAndSum(0,1,*(yData),offset,1,ySize)/sum;
//	}
//}

GPUImage GPUVelActivities::get2DView() const {
	size_t protect_1 = 1;
	size_t protect_2 = 2;
	return GPUImage(realNDO.reduceDimensionality(protect_1).reduceDimensionality(protect_2));
}

GPUImage& GPUVelActivities::getXValues() {
	if (xValues.isEmpty() || xValues.size()!=getURes()*getVRes()) {
		onesSizeVelSpace = GPUData(getURes()*getVRes());
		onesSizeVelSpace.fill(1.0f);
		xValues = GPUImage(getURes(),getVRes());
		xValues.copyFrom(getYValues().shiftDim(1));
	}
	return xValues;
}

GPUImage& GPUVelActivities::getYValues() {
	if (yValues.isEmpty() || yValues.size()!=getURes()*getVRes()) {
		yValues = GPUImage(getURes(),getVRes());
		const int ys = getVRes();
		const int yc = ys/2;
		for (int y=0; y<ys; ++y)
			yValues[y].fill(float(y-yc));
	}
	return yValues;
}
