/*

GPUIMAGEPROCESSING

  THIS MODULE PROVIDES A SIMPLE API FOR IMAGE OPERATIONS ON
  A GRAPHICAL PROCESSING UNIT (GPU) BY MAKING USE OF CUDA 
  TECHNOLOGY.

COPYRIGHT (C)

  ULM UNIVERSITY 
  INSTITUTE OF NEURAL INFORMATION PROCESSING (NEUROINFORMATIK)
  89069 ULM
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  INSTITUTE OF NEURAL INFORMATION PROCESSING:
  http://www.uni-ulm.de/en/in/institute-of-neural-information-processing.html

  ULM UNIVERSITY 
  HTTP://WWW.UNI-ULM.DE/

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
#ifndef GPUVELFIELD_H_
#define GPUVELFIELD_H_

#include "GPUImage.h"

class GPUVelField
{

private:

	GPUData realNDO;

	GPUImage xPart;
	GPUImage yPart;

	bool useCartesianInput;

public:

	inline int getWidth() const { return realNDO.extent(2); }
	inline int getHeight() const { return realNDO.extent(1); }

	GPUVelField();
	GPUVelField( const GPUData &, bool cartesian = true );
	GPUVelField(const GPUVelField& d);
	GPUVelField(int x, int y);
	GPUVelField(size_t x, size_t y );
	virtual ~GPUVelField();
	GPUVelField operator=(const GPUVelField& d);

	GPUImage getXPart();

	GPUImage getYPart();

	const GPUImage getXPart() const;

	const GPUImage getYPart() const;

	GPUData & getData() { return realNDO; }

	const GPUData & getData() const { return realNDO; }

	GPUImage getSpeed() const;

	void getSpeed(GPUImage& dest) const;

	GPUImage getDirection() const;

	void getDirection(GPUImage& dest) const;

	GPUData getAsFeatureImage() const;
	GPUData getAsFeatureImageView();

//	void show(int subs = 1);
//
//	void show(HostImage& hi, int subs = 1);

	void savepcm(const char* name);

	void loadpcm(const char* name);

	float getMaxSpeed();
	
	GPUVelField copy();

private:
//	void draw(const int rad, const float lengthFactor, const int subs, HostImage& im);
};

#endif /*GPUVELFIELD_H_*/
