/*

GPUIMAGEPROCESSING

  THIS MODULE PROVIDES A SIMPLE API FOR IMAGE OPERATIONS ON
  A GRAPHICAL PROCESSING UNIT (GPU) BY MAKING USE OF CUDA 
  TECHNOLOGY.

COPYRIGHT (C)

  ULM UNIVERSITY 
  INSTITUTE OF NEURAL INFORMATION PROCESSING (NEUROINFORMATIK)
  89069 ULM
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  INSTITUTE OF NEURAL INFORMATION PROCESSING:
  http://www.uni-ulm.de/en/in/institute-of-neural-information-processing.html

  ULM UNIVERSITY 
  HTTP://WWW.UNI-ULM.DE/

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
/*
 * GradientFeatureExtractor.hpp
 *
 *  Created on: Jan 14, 2009
 *      Author: jbouecke
 */
/**
 * This class computes the first derivation of an image.
 * 
 * By default the number of derivations are 2 and thus the
 * gradient (diffx,diffy) is caluclated.
 * 
 * If desired, more than 2 directional derivatives are interpolated
 * from the gradient (setNoOfDirections(n)).
 */
#ifndef GRADIENTFEATUREEXTRACTOR_HPP_
#define GRADIENTFEATUREEXTRACTOR_HPP_

#include "FeatureExtractor.h"

class GradientFeatureExtractor : public FeatureExtractor {
public:
	enum type_t {
		VECTOR,
		RAD
	} type;

private:

	GPUData  temp;
	GPUImage dx,dy;

	int  n;

public:
	GradientFeatureExtractor(type_t type_= VECTOR) : type(type_),n(2) { ; }

	void extrapolatDirections(int n);

	virtual ~GradientFeatureExtractor();

	virtual int getNoOfFeatures() const { return 2; };

	virtual type_t getFeatureType() const { return type; }

	/**
	 * creates for each feature a 2-d slice (image) and
	 * returns these as a 3-d GPUData object (n,height,weight).
	 */
	virtual GPUData getFeatureImages( GPUImage& input );

	/**
	 * the same like getFeatureImagesAsGPUData(const GPUImage& input) but as
	 * in place function.
	 */
	virtual void getFeatureImages( GPUImage& input, GPUData& result );

	/**
	 * reconstructs an image out of its features
	 */
	virtual GPUImage getReconstructedImage(GPUData& input);

	/**
	 * same like getReconstructedImage(GPUData& input) but as in
	 * place function
	 */
	virtual void getReconstructedImage(GPUData& input, GPUImage& result);
};

#endif /* GRADIENTFEATUREEXTRACTOR_HPP_ */
