/*

GPUIMAGEPROCESSING

  THIS MODULE PROVIDES A SIMPLE API FOR IMAGE OPERATIONS ON
  A GRAPHICAL PROCESSING UNIT (GPU) BY MAKING USE OF CUDA 
  TECHNOLOGY.

COPYRIGHT (C)

  ULM UNIVERSITY 
  INSTITUTE OF NEURAL INFORMATION PROCESSING (NEUROINFORMATIK)
  89069 ULM
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  INSTITUTE OF NEURAL INFORMATION PROCESSING:
  http://www.uni-ulm.de/en/in/institute-of-neural-information-processing.html

  ULM UNIVERSITY 
  HTTP://WWW.UNI-ULM.DE/

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
#include "HDF5RawDataFileAdapter.hpp"

HDF5RawDataFileAdapter::HDF5RawDataFileAdapter(const char* fileName, bool overwrite)
	: HDF5FloatFile(fileName,overwrite)
{
	;
}

HDF5RawDataFileAdapter::~HDF5RawDataFileAdapter()
{
	//this->~HDF5MotionFile();
}

void HDF5RawDataFileAdapter::readData(GPUData& dest, const char* ds_name)
{
	if (ds_name!=0)
		loadDataSet(ds_name);

	int  ndims = getDataDimension();
	size_t* sizes = getDataSizes();

	if (ndims!=dest.getDimensionsCount()) 
	  {
		throw std::runtime_error(("error within HDF5RawDataFileAdapter::readData\n number of dimensions dosn't match\n"));
	  }
	int sizeL = 1;
	for (int i=0; i<ndims; ++i)
		sizeL *= sizes[i];

	if (sizeL!=dest.size()) {
		throw std::runtime_error(("error within HDF5RawDataFileAdapter::readData\n number of elements dosn't match\n"));
	}

	float* hData = new float[sizeL];

	readDataSet(hData);

	dest.copyFromHostMemory(hData);

	delete[] hData;
}

void HDF5RawDataFileAdapter::readData(size_t* offset, GPUData& dest, const char* ds_name)
{
	if (ds_name!=0)
		loadDataSet(ds_name);

	size_t  ndims = getDataDimension();
	size_t* sizes = getDataSizes();

	if (ndims!=dest.getDimensionsCount()) {
		throw std::runtime_error(("error within HDF5RawDataFileAdapter::readData\n number of dimensions dosn't match\n"));
	}

	const size_t* sizes_stride = dest.getDimensions();

	float* hData = new float[dest.size()];

	readDataSet(hData,offset,sizes_stride);

	dest.copyFromHostMemory(hData);

	delete[] hData;
}

void HDF5RawDataFileAdapter::readData(int* offset, GPUData& dest, const char* ds_name)
{
	int ndims = getDataDimension();
	size_t* newOffsets = new size_t[ndims];
	for (int i=0; i<ndims; ++i) {
		newOffsets[i] = (size_t) offset[i];
	}
	readData(newOffsets,dest,ds_name);
}

void HDF5RawDataFileAdapter::writeData(const GPUData& data, const char* ds_name)
{
	if (ds_name!=0) 
	  {
		createDataSet(ds_name,data.getDimensionsCount(),data.getDimensions());
	  }

	int  ndims = getDataDimension();
	size_t* sizes = getDataSizes();
	size_t* offs  = new size_t[ndims];
	for (int i=0; i<ndims; ++i) offs[i]=0;

	if (ndims!=data.getDimensionsCount()) {
		throw std::runtime_error(("error within HDF5RawDataFileAdapter::writeData\n number of dimensions dosn't match\n"));
	}

	float* hData = new float[data.size()];

	data.copyToHostMemory(hData);

	HDF5FloatFile::writeDataSet(hData,offs,sizes);

	delete[] offs;
	delete[] hData;
}

/*void HDF5RawDataFileAdapter::writeData(const GPUData& data, const size_t* offs)
{
	int ndims = getDataDimension();
	size_t* newOffsets = new size_t[ndims];
	for (int i=0; i<ndims; ++i) {
		newOffsets[i] = (size_t) offs[i];
	}
	writeData(data,newOffsets);
}*/

void HDF5RawDataFileAdapter::writeData(const GPUData& data, const size_t* offs)
{

	int  ndims = getDataDimension();
	size_t* sizes = getDataSizes();

	if (ndims!=data.getDimensionsCount()) {
		throw std::runtime_error(("error within HDF5RawDataFileAdapter::writeData\n number of dimensions dosn't match\n"));
	}

	float* hData = new float[data.size()];

	data.copyToHostMemory(hData);

	HDF5FloatFile::writeDataSet(hData,offs,data.getDimensions());

	delete[] hData;
}
