/*

GPUIMAGEPROCESSING

  THIS MODULE PROVIDES A SIMPLE API FOR IMAGE OPERATIONS ON
  A GRAPHICAL PROCESSING UNIT (GPU) BY MAKING USE OF CUDA 
  TECHNOLOGY.

COPYRIGHT (C)

  ULM UNIVERSITY 
  INSTITUTE OF NEURAL INFORMATION PROCESSING (NEUROINFORMATIK)
  89069 ULM
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  INSTITUTE OF NEURAL INFORMATION PROCESSING:
  http://www.uni-ulm.de/en/in/institute-of-neural-information-processing.html

  ULM UNIVERSITY 
  HTTP://WWW.UNI-ULM.DE/

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
#include <HDF5RawDataFileAdapter.hpp>
#include <SimpleHostImageSequence.h>
#include <GPUVelActivities.h>
#include <DataWindow.hpp>
#include <iostream>
#include <cmath>

int show_flow_pX(HDF5RawDataFileAdapter& file, const char* p_name = "p2", const float threshold = -0.1f) {

	int ndims  	  = file.getDataDimension(p_name);
	size_t *in_sizes = file.getDataSizes();

	for (int i=0; i<ndims; ++i) {
		printf("%4d ",in_sizes[i]);
	}
	printf("\n");

	if (ndims!=4) {
		std::cerr << "error! input data has to be 4-dimensional" << std::endl;
		delete[] in_sizes;
		return -1;
	}

	const int w = in_sizes[3];
	const int h = in_sizes[2];
	const int uv= in_sizes[1];
	const int n = in_sizes[0];

	delete[] in_sizes;

	in_sizes = file.getDataSizes("stimulus");
	const int ts = n/(in_sizes[0]-1);
	std::cout << "number of images : " << in_sizes[0] << std::endl;

	delete[] in_sizes;

	const int u = int(sqrt(double(uv)));
	const int v = u;

	if (u*v!=uv || u%2==0)  {
		std::cerr << "error! input velocity space has to be square (#u=#v)" << std::endl;
		return -1;
	}

	std::cout << "max speed is " << (u-1)/2 << std::endl;

	unsigned int d = 2;
	GPUData data(1,uv,h,w); data.fill(0.5f);
	GPUData data_3d_view(data.reduceDimensionality(d));
//	GPUData			 velActDataINRIAOrder(v,u,h,w);
	GPUVelActivities velActData(h,w,v,u); velActData.getData().fill(0.0f);
	d = 1;
	GPUData			 velActData_3d_view(velActData.getData().reduceDimensionality(d));
	GPUVelField		 velField = velActData.getFlowField();

	GPUData imageData(1,h,w);
	GPUImage image(w,h); image.fill(0.5f); image[0][0] = 0.0f;


	int offs[4] = {0,0,0,0};
	file.readData(offs,data,p_name);
	file.readData(offs,imageData,"stimulus");
	image.copyFrom(imageData.slice(0,0));

	DataWindow  window(image.getWidth(),image.getHeight());
	DataWindow  window_2(image.getWidth(),image.getHeight());
	window.setArrowStyle(DataWindow::LINES);
	window.update(image);
	window.repaint();
	window.process_all_pending_gtk_events();
	while( window.is_visible() )
	for (int i=0; i<n && window.is_visible(); i+=1) {

		std::cout << "show frame " << i/double(ts) << std::endl;
		if (i%ts==0) {
			offs[0] = i/ts;
			file.readData(offs,imageData,"stimulus");
			image.copyFrom(imageData.slice(0,0));
			offs[0] = i;
			file.readData(offs,data,p_name);
		} else {
			offs[0] = i;
			file.readData(offs,data);
		}

		printf("x \\in [%f,%f]\n",data.min(),data.max());

//		int i=0;
//		for (int iv=0; iv<v; iv++) {
//			for (int iu=0; iu<u; iu++) {
//				velActData.getOneVelocityAsImage(iu,iv).copyFrom(data[0][i++]);
//			}
//		}
		velActData_3d_view.shiftDim(1).copyFrom(data_3d_view);


		{
			int sx = w/2-1 > 10 ? 10 : w/2-1;
			int sy = h/2-1 > 10 ? 10 : h/2-1;
			window_2.update(velActData.getVelocitySpacesInOneImage(false,w/2,h/2,sx,sy));
		}
		velActData.shutOfBadPositions(threshold);
		velActData.getFlowField(velField);
		window.update(image,velField);
		DataWindow::process_all_pending_gtk_events();
		DataWindow::busy_wait(200);
	}
	return 0;
}

int show_stimulus(HDF5RawDataFileAdapter& file) {

	int ndims  	  = file.getDataDimension("stimulus");
	size_t *in_sizes = file.getDataSizes();

	for (int i=0; i<ndims; ++i) {
		printf("%4d ",in_sizes[i]);
	}
	printf("\n");

	if (ndims!=3) {
		std::cerr << "error! input data has to be 3-dimensional" << std::endl;
		delete[] in_sizes;
		return -1;
	}

	const int w = in_sizes[2];
	const int h = in_sizes[1];
	const int n = in_sizes[0];

	delete[] in_sizes;

	int offs[3] = {2,0,0};

	GPUData data(1,h,w);

	file.readData(offs,data);

	GPUImage image(w,h);

	DataWindow window(w*3,h*3,"stimulus");

	while( window.is_visible() )
	for (int i=0; i<n && window.is_visible(); i+=1) {
		offs[0] = i;

		std::cout << "show frame " << offs[0] << std::endl;

		file.readData(offs,data);
		image.copyFrom(data.slice(0,0));

		window.update(image);
		DataWindow::busy_wait(500);
		DataWindow::process_all_pending_gtk_events();
	}

	return 0;
}

void create_stimulus(HDF5RawDataFileAdapter& file, HostImageSequence& input) {
	std::vector<HostImage> input_vector;
	while (input.hasNextImage()) {
		input_vector.push_back(input.getNextImage());
	}

	const int n = input_vector.size();
	const int w = input_vector.front().getWidth();
	const int h = input_vector.front().getHeight();

	std::cout << "Loaded " << n << " input images of " << std::endl;
	std::cout << "size " << w << "x" << h << "." << std::endl;

	GPUData data(n,h,w);
	for (int i=0; i<n; ++i) {
		data[i].copyFrom(GPUImage(input_vector[i]));
	}
	std::cout << "all copied" << std::endl;
	file.writeData(data,"stimulus");
	std::cout << "all written" << std::endl;
}

int main( int argc, char** argv) {

	Window::global_gtk_gl_init(argc,argv);

	char* fileName;

	if (argc<3) {
		std::cout << "use : " << argv[0];
		std::cout << " [create|show|result] HDF5FILE VAR1 VAR2 VAR3"<<std::endl;
		std::cout << "\n  If using command 'create' with, VAR1 = in-file pattern,";
		std::cout << "\n                                  VAR2 = start index,";
		std::cout << "\n                                  VAR3 = end index,";
		std::cout << "\n  the given image sequence is written into the given";
		std::cout << "\n  HDF5 file into a dataset called 'stimulus'.";
		std::cout << "\n  ";
		std::cout << "\n  The command 'show' shows the input sequence 'stimulus'";
		std::cout << "\n  out of the given HDF5 file.";
		std::cout << "\n  ";
		std::cout << "\n  'result' creates an optic flow out of the velocity hypothesis";
		std::cout << "\n  from model area MT and visualize it.";
		std::cout << "\n  ";
		std::cout << "\n  For any flow display, a threshold parameter could be";
		std::cout << "\n  applied via option -t THRESHOLD";
		std::cout << std::endl;
		return 1;
	}

	fileName = argv[2];

	float display_threshold = 0.0f;
	bool  show_vels_too = false;
	float xo=0.0f;
	float yo=0.0f;
	float xs=1.0f;
	float ys=1.0f;
	for (int i=1; i<argc-1; ++i)
		if (0==strcmp(argv[i],"-t"))
			sscanf(argv[i+1],"%f",&display_threshold);
	for (int i=2; i<argc-1; ++i)
		if (0==strcmp(argv[i],"-c"))
			sscanf(argv[i+1],"%f,%f,%f,%f",&xo,&yo,&xs,&ys);
	{
		float m = 1;
		while (fmax(fmax(xo,yo),fmax(xs,ys))>m)
			m *= 10;
		xo/=m;
		yo/=m;
		xs/=m;
		ys/=m;
	}

	if (argv[1][0] == 's') {
		HDF5RawDataFileAdapter file(fileName);
		show_stimulus(file);
	} else if (argv[1][0] == 'c') {
		if (argc<6) {
			std::cout << "wrong number of parameters for 'create'" << std::endl;
			return 1;
		}
		FILE* f = fopen(fileName,"r");
		bool f_exists = f!=0;
		if (f_exists) fclose(f);
//		HDF5RawDataFileAdapter file(fileName,true);
		HDF5RawDataFileAdapter file(fileName,!f_exists);
		std::string file_pattern = argv[3];
		int a,b;
		sscanf(argv[4],"%d",&a);
		sscanf(argv[5],"%d",&b);
		SimpleHostImageSequence sequence(file_pattern.c_str(),a,b,xo,yo,xs,ys);
		create_stimulus(file,sequence);
	} else if (argv[1][0] == 'r') {
		HDF5RawDataFileAdapter file(fileName);
		show_flow_pX(file,"p2",display_threshold);
	} else if (argv[1][0] == 'p') {
		HDF5RawDataFileAdapter file(fileName);
		show_flow_pX(file,argv[1],display_threshold);
	}
	return 0;
}
