/*

GPUIMAGEPROCESSING

  THIS MODULE PROVIDES A SIMPLE API FOR IMAGE OPERATIONS ON
  A GRAPHICAL PROCESSING UNIT (GPU) BY MAKING USE OF CUDA 
  TECHNOLOGY.

COPYRIGHT (C)

  ULM UNIVERSITY 
  INSTITUTE OF NEURAL INFORMATION PROCESSING (NEUROINFORMATIK)
  89069 ULM
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  INSTITUTE OF NEURAL INFORMATION PROCESSING:
  http://www.uni-ulm.de/en/in/institute-of-neural-information-processing.html

  ULM UNIVERSITY 
  HTTP://WWW.UNI-ULM.DE/

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
#ifndef LUCASKANADE_H_
#define LUCASKANADE_H_

#include <GPUImage.h>
#include <GPUVelField.h>
#include <FloatingAverage.h>

#include <string>

/**
 * This class implementes a simple, one scale,
 * Lucas-Kanade approach to calculate a flow filed out
 * of two different images
 */
class LucasKanade
{
private:
	GPUData kernel_d;
	GPUData kernel_gauss_small;
	GPUData kernel_gauss;
	GPUData kernel_gauss_large;

	GPUImage pastIm;

	GPUImage temp1;
	GPUImage temp2;
	GPUImage temp3;
	GPUImage temp4;
	GPUImage temp5;

	GPUImage imdx;
	GPUImage imdy;
	GPUImage imdt;

	GPUImage imGxx;
	GPUImage imGxy;
	GPUImage imGyy;

	GPUImage imGxt;
	GPUImage imGyt;

	GPUVelField res;
	GPUImage    resX;
	GPUImage    resY;

	int pastCounter;

	FloatingAverage floatingAverage;
	FloatingAverage floatingAverage2;
	FloatingAverage floatingAverage3;
	
	bool useSaliencyAsSpeedMask;
	bool useSimpleSaliency;
	
public:
	LucasKanade(int w, int h);
	virtual ~LucasKanade();

	void computeStep(const GPUImage& past, const GPUImage& now);
	GPUVelField& next(const GPUImage& now);

	GPUVelField& getFlowField();
	GPUImage& getConfidence() { return temp5; }
	
	void useSaliencyMaskOnSpeed(bool b) { useSaliencyAsSpeedMask = b; }
	void switchToSimpleSaliency(bool b) { useSimpleSaliency = b; }
	
	//static std::vector<GPUVelField> go(std::string filePattern, int startIdx, int stopIdx);
	static void go(std::string filePattern, int startIdx, int stopIdx);
	
private:
	void computeDerivatives(
				const GPUImage& im,
				const GPUImage& imPast);

	void computeMulAndIntegrate(
				GPUImage& res,
				const GPUImage& a,
				const GPUImage& b,
				int noOfScales=1);
	
	void computeMulAndIntegrate(
				GPUImage& res,
				const GPUImage& a,
				int noOfScales=1);

//	void computeMulAndIntegrateAdditive(
//				GPUImage& res,
//				const GPUImage& a,
//				const GPUImage& b,
//				int noOfScales=2);

	void computeMul(
			GPUImage& res,
			const GPUImage& a,
			const GPUImage& b);
};

#endif /*LUCASKANADE_H_*/
