/*

GPUIMAGEPROCESSING

  THIS MODULE PROVIDES A SIMPLE API FOR IMAGE OPERATIONS ON
  A GRAPHICAL PROCESSING UNIT (GPU) BY MAKING USE OF CUDA 
  TECHNOLOGY.

COPYRIGHT (C)

  ULM UNIVERSITY 
  INSTITUTE OF NEURAL INFORMATION PROCESSING (NEUROINFORMATIK)
  89069 ULM
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  INSTITUTE OF NEURAL INFORMATION PROCESSING:
  http://www.uni-ulm.de/en/in/institute-of-neural-information-processing.html

  ULM UNIVERSITY 
  HTTP://WWW.UNI-ULM.DE/

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
#include "GPUData.h"

#include <boost/scoped_ptr.hpp>

class Kernel;

class ModelArea
{

public:

  typedef enum {NONE, GAUSS, DIPOLE, CURVE, MODULATED_DIFFUSION} KernelType;

  ModelArea();
  ~ModelArea();

  //assumes first two dimensions are image dimensions and all others are feature dimensions!!
  void iterate( GPUData & input, GPUData & output, GPUData feedback = GPUData(), bool pred_vel_shift_fb = false );

  void set_sigma( int dim, float sigma );
  void set_typeBorder( int dim, int typeBorder );
  void set_sigma_for_spatial_inhibition(float sigma);
  void set_kernel_type( KernelType ktype );
  void set_feedback_gain( float g );
  void set_shunting_params( float dc, float df, float ds, float sf = 0.0f);
  void set_do_square( bool b );
  void set_dt(float = 0.05f);
  void set_ori_neigh( unsigned n );

  void set_modulator( const GPUData & modulator );

  void set_use_recurrent_inhibition(bool b=true);
  void set_use_rectified_simple_input(bool b=true);
  void set_INRIA_dynamic( float exitat, float inhib, float leak );

private:

  void init( GPUData & input );

  unsigned _n_dip_ori_neigh;
  float _feedback_gain;
  float _divConst;
  float _divFactor;
  float _divSelf;
  float _subFactor;
  bool _do_square;
  bool _use_forward_inhibition;
  bool _rect_inria_style_input;

  float _dt;

  float _sigma_for_spatial_inhibition;

  GPUData _tmp_inhibit;
  GPUData _tmp_inhibit2;
  GPUData _ones;
  GPUData _fb_buffer;
  GPUData _activity;

  bool _kernel_settings_changed;
  KernelType _kernel_type;
  float _sigma[5];
  int _typeBorder[5];
  boost::scoped_ptr<Kernel> _kernel;
  boost::scoped_ptr<Kernel> _kernel2;
};
