/*

GPUIMAGEPROCESSING

  THIS MODULE PROVIDES A SIMPLE API FOR IMAGE OPERATIONS ON
  A GRAPHICAL PROCESSING UNIT (GPU) BY MAKING USE OF CUDA 
  TECHNOLOGY.

COPYRIGHT (C)

  ULM UNIVERSITY 
  INSTITUTE OF NEURAL INFORMATION PROCESSING (NEUROINFORMATIK)
  89069 ULM
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  INSTITUTE OF NEURAL INFORMATION PROCESSING:
  http://www.uni-ulm.de/en/in/institute-of-neural-information-processing.html

  ULM UNIVERSITY 
  HTTP://WWW.UNI-ULM.DE/

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
/*
 * ModulatedIntegration.hpp
 *
 *  Created on: Feb 11, 2009
 *      Author: jbouecke
 */

#ifndef MODULATEDINTEGRATION_HPP_
#define MODULATEDINTEGRATION_HPP_

#include <GPUData.h>
#include "GPUImage.h"
#include <vector>

/**
 * This class uses the set modulator to modulate the spatial integration for
 * different orientations.
 *
 * The modulator has to be 3-dimensionsional (ori,height,width) with orientation
 * modulation factors for the sampled set of orientations in [0,pi].
 *
 * The input is assumed to be 4-dimensional (height, weight, v, u) where
 * the integration process is done in each [u,v] separately.
 */
class ModulatedIntegration {
private:

public:
	float _sigma_small;
	float _sigma_large;
	float _sigma_rest;

	GPUData _input;
	GPUData _modulator;

	GPUImage _temp1;
	GPUImage _temp2;
	GPUImage _temp3;
	GPUImage _temp4;
	GPUImage _temp5;
	std::vector<GPUImage>  _filters;
	GaussKernel2 cirularGauss;
	GaussKernel2 cirularGauss_large;

	double _angle_offset;

  void init( size_t w, size_t h, size_t o = 0 );

public:

	ModulatedIntegration();
	ModulatedIntegration(float sigma_large, float sigma_small, float sigma_rest=0.0f);

	virtual ~ModulatedIntegration();

	void setInput(GPUData& data);
	void setModulator(GPUData& data);

	/**
	 * The work method works on the given modulatory input (via setModulator())
	 * and the given input (set via setInput()).
	 * Both inputs are overwritten during work. The result is written back into
	 * the given input.
	 */
	void work();
};

#endif /* MODULATEDINTEGRATION_HPP_ */
