/*

GPUIMAGEPROCESSING

  THIS MODULE PROVIDES A SIMPLE API FOR IMAGE OPERATIONS ON
  A GRAPHICAL PROCESSING UNIT (GPU) BY MAKING USE OF CUDA 
  TECHNOLOGY.

COPYRIGHT (C)

  ULM UNIVERSITY 
  INSTITUTE OF NEURAL INFORMATION PROCESSING (NEUROINFORMATIK)
  89069 ULM
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  INSTITUTE OF NEURAL INFORMATION PROCESSING:
  http://www.uni-ulm.de/en/in/institute-of-neural-information-processing.html

  ULM UNIVERSITY 
  HTTP://WWW.UNI-ULM.DE/

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
#pragma once

#include "FeatureExtractor.h"

class RHOGFeatureExtractor {

public:

  RHOGFeatureExtractor( int cell_size, int block_size, int block_overlap, int n_orientations, bool interpolate = false
						, float alpha = 1.0f, bool felzenszwalb = false, bool contrast_sensitive = false
						, float xscale = 1, float yscale = 1 )
	: _cell_size( cell_size )
	, _block_size( block_size )
	, _block_overlap( block_overlap )
	, _n_orientations( n_orientations )
	, _interpolate( interpolate )
	, _felzenszwalb(felzenszwalb)
	, _contrast_sensitive(contrast_sensitive)
	, _alpha(alpha)
	, _xscale(xscale)
	, _yscale(yscale)
  {
  }
  
  /**
   * Returns the number of features at each position.
   */
  virtual int getNoOfFeatures() const;

  /**
   * creates for each feature a floating one 2-d slice (image) and
   * returns these as a 3-d GPUData object
   * <code> getNoOfFeatures() </code> length.
   */
  virtual GPUData getFeatureImages( GPUImage& input );
  
  /**
   * the same like getFeatureImagesAsGPUData(const GPUImage& input) but as
   * in place function.
   */
  virtual void getFeatureImages( GPUImage& input, GPUData& result );
  
  /**
   * reconstructs an image out of its features
   */
  virtual GPUImage getReconstructedImage(GPUData& input);
  
  /**
   * same like getReconstructedImage(std::vector<GPUImage>& input) but as in place
   * function
   */
  virtual void getReconstructedImage(GPUData& input, GPUImage& result);
  
  virtual ~RHOGFeatureExtractor();
  
private:
  
  int _cell_size, _block_size, _block_overlap, _n_orientations;
  bool _interpolate;
  bool _felzenszwalb;
  bool _contrast_sensitive;
  float _alpha;
  float _xscale;
  float _yscale;
};
