/*

GPUIMAGEPROCESSING

  THIS MODULE PROVIDES A SIMPLE API FOR IMAGE OPERATIONS ON
  A GRAPHICAL PROCESSING UNIT (GPU) BY MAKING USE OF CUDA 
  TECHNOLOGY.

COPYRIGHT (C)

  ULM UNIVERSITY 
  INSTITUTE OF NEURAL INFORMATION PROCESSING (NEUROINFORMATIK)
  89069 ULM
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  INSTITUTE OF NEURAL INFORMATION PROCESSING:
  http://www.uni-ulm.de/en/in/institute-of-neural-information-processing.html

  ULM UNIVERSITY 
  HTTP://WWW.UNI-ULM.DE/

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
#ifndef __MOTION_UTILS_H
#define __MOTION_UTILS_H

#include <H5Cpp.h>

using namespace std;
#ifndef H5_NO_NAMESPACE
using namespace H5;
#endif

#define START_CHRONO(name) \
  clock_t chrono_##name##_start = clock(), chrono_##name##_end;

#define STOP_CHRONO(name) \
  chrono_##name##_end = clock(); \
  cout << "Elapsed time for chrono " #name ": " \
       << (((double) (chrono_##name##_end - chrono_##name##_start)) / CLOCKS_PER_SEC) << endl;

#define VSPACE_VX(vspace, v) ((v) % (vspace) - ((vspace) - 1) / 2)
#define VSPACE_VY(vspace, v) ((v) / (vspace) - ((vspace) - 1) / 2)


static inline void
load_or_store_frame(float* out, DataSet& dset, int frame, bool load)
{
	DataSpace fspace = dset.getSpace();
	int rank = fspace.getSimpleExtentNdims();
	hsize_t* fcount = new hsize_t[rank];
	fspace.getSimpleExtentDims(fcount);
	fcount[0] = 1;

	hsize_t* fstart = new hsize_t[rank];
	for (int i = 1; i < rank; i++)
		fstart[i] = 0;
	fstart[0] = frame;
	fspace.selectHyperslab(H5S_SELECT_SET, fcount, fstart);

	DataSpace mspace(rank, fcount);

	if (load)
		dset.read(out, PredType::NATIVE_FLOAT, mspace, fspace);
	else
		dset.write(out, PredType::NATIVE_FLOAT, mspace, fspace);

	delete[] fcount;
	delete[] fstart;
}

static inline void
load_frame(float* out, DataSet& dset, int frame)
{
	load_or_store_frame(out, dset, frame, true);
}

static inline void
store_frame(float* data, DataSet& dset, int frame)
{
	load_or_store_frame(data, dset, frame, false);
}

#endif


// Local Variables:
// mode: c++
// coding: utf-8
// c-basic-offset: 4
// tab-width: 4
// End:
