/*

GPUIMAGEPROCESSING

  THIS MODULE PROVIDES A SIMPLE API FOR IMAGE OPERATIONS ON
  A GRAPHICAL PROCESSING UNIT (GPU) BY MAKING USE OF CUDA 
  TECHNOLOGY.

COPYRIGHT (C)

  ULM UNIVERSITY 
  INSTITUTE OF NEURAL INFORMATION PROCESSING (NEUROINFORMATIK)
  89069 ULM
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  INSTITUTE OF NEURAL INFORMATION PROCESSING:
  http://www.uni-ulm.de/en/in/institute-of-neural-information-processing.html

  ULM UNIVERSITY 
  HTTP://WWW.UNI-ULM.DE/

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
#ifndef __OPTS_HPP__F22E90C9_7B05_4947_8F73_95F7C91FAF53
#define __OPTS_HPP__F22E90C9_7B05_4947_8F73_95F7C91FAF53

#include <boost/program_options.hpp>


inline void
setup_opts (
		boost::program_options::options_description &desc,
		boost::program_options::positional_options_description &p)
{
	desc.add_options()
		("help,h", "produce help message")
		("config-file", boost::program_options::value<std::string>())
		("cuda-device", boost::program_options::value<int>()->default_value(0)
				, "The cuda device number to use for computation");

	boost::program_options::options_description input_options("Input");
	input_options.add_options()
		("input.files", boost::program_options::value< std::string >(),
							"Set the printf-pattern for reading the .png input files."
							"This option could also be given as the first positional option.")
		("input.start", boost::program_options::value<int>(),
							"Index of the first file to process.\n"
							"This option could be given as the second positional option")
		("input.stop", boost::program_options::value<int>(),
							"Index of the last file to process.\n"
							"This option could be given as the third positional option")
		("input.crop-input,c", boost::program_options::value< std::string >(),
							"Use only a sub region of the given input sequence.\n"
							"Example: x,y,w,h       	(all given in [0,1])");

	boost::program_options::options_description output_options("Output");
	output_options.add_options()
		("output.file,o", boost::program_options::value< std::string >()->default_value("out.h5"),
							"The hdf5 file where the results should be stored.")
		("output.data,d",boost::program_options::value< std::vector< std::string > >()->composing(),
							"The data to write: [all|flow|v1|v1_normed|mt|mt_normed|init-features|init-normed-features|init-detection]\n"
							"The input stimulus is written in addition to each of the other output data.")
		("output.debug,v","Enables output of max-/min-activity values each iteration")
		("output.show-results","Enables instantan output of some results using CUDA-GL interface")
		("output.threshold-for-visualization,t", boost::program_options::value<float>()->default_value(-1.0f),
							"Threshold to shut of positions with low maximum activity before visualization"
							"If the value is set to -1, no threshold is applied.")
		("output.delay", boost::program_options::value<int>(),"delays the output at each frame (in ms)");

	boost::program_options::options_description feature_options("Feature");
	feature_options.add_options()
		("feature.type", boost::program_options::value< std::string >()->default_value("gabor")
				,"With this parameter the intial feature type with the associated "
						"correlator could be choosen (gabor,log)")
		("feature.norm0", boost::program_options::value<std::string>()->default_value("true")
								, "use a pre normalization")
		("feature.norm0.naka-rushton", boost::program_options::value<float>()->default_value(-1))
		("feature.norm0.epsilon", boost::program_options::value<float>()->default_value(0.01f))
		("feature.norm0.sigma", boost::program_options::value<float>()->default_value(4))
		("feature.norm", boost::program_options::value<std::string>()->default_value("true")
				, "use the normalization")
		("feature.norm.naka-rushton", boost::program_options::value<float>()->default_value(0.05))
		("feature.norm.epsilon", boost::program_options::value<float>()->default_value(-1))
		("feature.norm.sigma", boost::program_options::value<float>()->default_value(-1.0f))
		("feature.norm2", boost::program_options::value<std::string>()->default_value("true")
				, "use a second normalization to max sum == 1")
		("feature.sigma-per-lambda", boost::program_options::value<float>()->default_value(0.4f))
		("feature.lambda0", boost::program_options::value<float>()->default_value(5.0f))
		("feature.use_old_prior", boost::program_options::value<int>()->default_value(1)
				, "With this value, the impact of a prior to low velocities could be set")
		("feature.use_prior", boost::program_options::value<float>()->default_value(0.25f)
				,"With this value, the impact of a prior to low velocities could be set")
		("feature.prior_sigma", boost::program_options::value<float>()->default_value(0.75f)
				,"With this value, the impact of a prior to low velocities could be set");;

	boost::program_options::options_description motion_options("Motion");
	motion_options.add_options()
		("motion.max-vel,m", boost::program_options::value<int>()->default_value(7),
							"The maximal velocity which should be represented in the velocity space.")
		("motion.feedback,f", boost::program_options::value<float>()->default_value(10.0f),
							"Feedback factor")
		("motion.mt-downsampling,r",boost::program_options::value< int >()->default_value(5),
							"Downsampling factor from v1 to mt (integer value).")
		("motion.sigma-for-mt-integration,s", boost::program_options::value< float >()->default_value(5.0f),
							"Sigma for spatial bluring before subsample from V1 to MT")
		("motion.sigma-for-vel-blur", boost::program_options::value< float >()->default_value(0.0f),
							"Sigma for velocity bluring in V1 to MT")
		("motion.V1-sigma-for-spatial-inhibition", boost::program_options::value< float >()->default_value(4.0f),
							"Sigma for spatial bluring before performing the inhibition")
		("motion.V1-square", boost::program_options::value< bool >()->default_value(true))
		("motion.V1-div-const", boost::program_options::value< float >()->default_value(0.015f))
		("motion.V1-div-factor", boost::program_options::value< float >()->default_value(20))
		("motion.V1-div-factor-scaled", boost::program_options::value< float >(),
				"uses the sum instead of the mean activity for the divisive inhibition")
		("motion.V1-div-self", boost::program_options::value< float >()->default_value(1))
		("motion.V1-sub-factor", boost::program_options::value< float >()->default_value(0.25f))
		("motion.V1-sub-factor-scaled", boost::program_options::value< float >()
				,"uses the sum instead of the mean activity for the subtractive inhibition")
		("motion.MT-sigma-for-spatial-inhibition", boost::program_options::value< float >()->default_value(-1),
							"Sigma for spatial blurring before performing the inhibition"
							"If this is set to a negative value, the value is set to the one"
							"of Model Area V1 divide by the sub sampling factor")
		("motion.MT-square", boost::program_options::value< bool >()->default_value(true))
		("motion.MT-div-const", boost::program_options::value< float >()->default_value(0.015f))
		("motion.MT-div-factor", boost::program_options::value< float >()->default_value(20))
		("motion.MT-div-factor-scaled", boost::program_options::value< float >()
				,"uses the sum instead of the mean activity for the divisive inhibition")
		("motion.MT-div-self", boost::program_options::value< float >()->default_value(1))
		("motion.MT-sub-factor", boost::program_options::value< float >()->default_value(0.25f))
		("motion.MT-sub-factor-scaled", boost::program_options::value< float >()
				,"uses the sum instead of the mean activity for the divisive inhibition")
		("motion.dt", boost::program_options::value<float>()->default_value(-1.0),
							"This parameter determines the time step within each iteration"
							"If it is negative, a fix point iterations is applied.")
		("motion.recurrent-inhibition", "to use recurrent inhibition in stead of forward inhibition")
		("motion.rectified-simple-input", "to rectify the over all input (only possible for special parameter sets")
		("motion.local-steps,s",boost::program_options::value< int >()->default_value(0),"no of steps between two images")
		("motion.use-form",boost::program_options::value< std::string >(),"enables form specific integration")
	;

	p.add("input.files", 1).add("input.start", 1).add("input.stop", 1).add("output.file",-1);
	desc.add(input_options).add(output_options).add(feature_options).add(motion_options);
}

#endif /* __OPTS_HPP__F22E90C9_7B05_4947_8F73_95F7C91FAF53 */

