/*

GPUIMAGEPROCESSING

  THIS MODULE PROVIDES A SIMPLE API FOR IMAGE OPERATIONS ON
  A GRAPHICAL PROCESSING UNIT (GPU) BY MAKING USE OF CUDA 
  TECHNOLOGY.

COPYRIGHT (C)

  ULM UNIVERSITY 
  INSTITUTE OF NEURAL INFORMATION PROCESSING (NEUROINFORMATIK)
  89069 ULM
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  INSTITUTE OF NEURAL INFORMATION PROCESSING:
  http://www.uni-ulm.de/en/in/institute-of-neural-information-processing.html

  ULM UNIVERSITY 
  HTTP://WWW.UNI-ULM.DE/

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
#include "temp.hpp"
#include "kernels.hpp"
#include <cstdio>
#include <png.h>

// DISCLAIMER:
// this file is for temporarily used functions only! They should not be used in
// the long term. Their sole purpose is to be a stupid replacement for broken
// functions or placeholders

namespace placeholder {

void simple_downsample (GPUData &in, GPUData &out, int factor)
{
	const size_t *sizes = in.getSizes();
	
	GPUData copy (sizes[0], sizes[1]);
	GPUData temp (sizes[0], sizes[1]);
	copy.copyFrom(in);

	// apply will destroy it's input image, therefore use a copy of it
	GaussKernel2 gauss(1.5f);
	gauss.apply(copy, temp);
	
	const size_t *sizes2 = temp.getSizes();
	const int offset[] = {0,0};
	const size_t ndim = 2;
	const size_t dims[] = {sizes2[0]/factor, sizes2[1]/factor};
	const int strides[] = {sizes2[1] * 2, 2};

	out.copyFrom(temp.view(offset, ndim, dims, strides));
}

float* read_gray_png(const char* fileName, int& w, int& h) {
	// --------------------- //
	// -- check file type -- //
	// --------------------- //

	struct file_handler { FILE * fp; ~file_handler(){fclose(fp);} };    

	FILE *fp = fopen(fileName, "rb");
	if (!fp) 
	  throw( std::runtime_error( str( boost::format( "could not open: %s" ) % fileName ) ) );
	file_handler file_guard = {fp};

	const int number = 8;
	Byte header[number];
	fread(header, 1, number, fp);
	bool is_png = !png_sig_cmp(header, 0, number);

	if (!is_png)
	  throw( std::runtime_error( str( boost::format( "not a png file: %s" ) % fileName ) ) );

	// ---------------- //
	// -- read image -- //
	// ---------------- //

	//deleter for the png read stuff
	struct png_read_handler
	{ 
	  png_structpp s; png_infopp i1; png_infopp i2;
	  ~png_read_handler() { png_destroy_read_struct(s,i1,i2);}
	};
	png_read_handler png_read_guard = {0,0,0};
   
	png_structp png_ptr = png_create_read_struct
		(PNG_LIBPNG_VER_STRING,
		(png_voidp)NULL, NULL, NULL);
	if (!png_ptr)
	  throw( std::runtime_error( str( boost::format( "could not read png from: %s" ) % fileName ) ) );
	png_read_guard.s = &png_ptr;

	png_infop info_ptr = png_create_info_struct(png_ptr);
	if (!info_ptr)
	  throw( std::runtime_error( str( boost::format( "could not get info for png: %s" ) % fileName ) ) );
	png_read_guard.i1 = &info_ptr;

	png_init_io(png_ptr, fp);
	png_set_sig_bytes(png_ptr, number);
	
	png_read_info(png_ptr, info_ptr);  /* read all PNG info up to image data */
	
	const int width            = png_get_image_width(png_ptr,info_ptr);
	const int height           = png_get_image_height(png_ptr,info_ptr);
	int bit_depth        = png_get_bit_depth(png_ptr,info_ptr);
	int color_type       = png_get_color_type(png_ptr,info_ptr);

	png_set_expand(png_ptr);
	png_set_strip_16(png_ptr);
	png_set_gray_to_rgb(png_ptr);
	
	png_uint_32 i, rowbytes;
	png_bytep row_pointers[height];
  
	png_read_update_info(png_ptr, info_ptr);
  
	rowbytes = png_get_rowbytes(png_ptr, info_ptr);
	int channels = (int)png_get_channels(png_ptr, info_ptr);
  
	unsigned char* image_data = (unsigned char *)malloc(rowbytes*height);
  
	for (i = 0;  i < (png_uint_32)height;  ++i)
		row_pointers[i] = image_data + i*rowbytes;
	
	float* floatData = new float[width*height];
	
	png_read_image(png_ptr, row_pointers);
	
	int n=0;
	Byte r,g,b;    
	for (int y=0; y<height; ++y) {
		for (int x=0; x<width; ++x) {
			r = row_pointers[y][x*3+0];
			g = row_pointers[y][x*3+1];
			b = row_pointers[y][x*3+2];
	//		floatData[n++] = float((r+g+b)/1.0);
			floatData[n++] = (0.3f*r+0.59f*g+0.11f*b)/1.0f;
		}
	}
	png_destroy_read_struct(&png_ptr, &info_ptr, NULL);
	return floatData;
}

};
