/*

GPUIMAGEPROCESSING

  THIS MODULE PROVIDES A SIMPLE API FOR IMAGE OPERATIONS ON
  A GRAPHICAL PROCESSING UNIT (GPU) BY MAKING USE OF CUDA 
  TECHNOLOGY.

COPYRIGHT (C)

  ULM UNIVERSITY 
  INSTITUTE OF NEURAL INFORMATION PROCESSING (NEUROINFORMATIK)
  89069 ULM
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  INSTITUTE OF NEURAL INFORMATION PROCESSING:
  http://www.uni-ulm.de/en/in/institute-of-neural-information-processing.html

  ULM UNIVERSITY 
  HTTP://WWW.UNI-ULM.DE/

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
#include "util.hpp"
#include <cuda_runtime.h>

void printf_deviceInfo (int deviceId)
{
	//cudaSetDevice(vm["cuda-device"].as<int>());
	cudaGetDevice(&deviceId);
	std::cout << "cuda device number: " << deviceId << std::endl;
	cudaDeviceProp deviceProp;
	cudaGetDeviceProperties(&deviceProp, deviceId);
	printf("\nDevice %d: \"%s\"\n", deviceId, deviceProp.name);
	printf("  Major revision number:                         %d\n",
		   deviceProp.major);
	printf("  Minor revision number:                         %d\n",
		   deviceProp.minor);
	printf("  Total amount of global memory:                 %d bytes\n",
		   int(deviceProp.totalGlobalMem));
#if CUDART_VERSION >= 2000
	printf("  Number of multiprocessors:                     %d\n",
		   deviceProp.multiProcessorCount);
	printf("  Number of cores:                               %d\n",
		   8 * deviceProp.multiProcessorCount);
#endif
	printf("  Total amount of constant memory:               %d bytes\n",
		   int(deviceProp.totalConstMem));
	printf("  Total amount of shared memory per block:       %d bytes\n",
		   int(deviceProp.sharedMemPerBlock));
	printf("  Total number of registers available per block: %d\n",
		   deviceProp.regsPerBlock);
	printf("  Warp size:                                     %d\n",
		   deviceProp.warpSize);
	printf("  Maximum number of threads per block:           %d\n",
		   deviceProp.maxThreadsPerBlock);
	printf("  Maximum sizes of each dimension of a block:    %d x %d x %d\n",
		   deviceProp.maxThreadsDim[0],
		   deviceProp.maxThreadsDim[1],
		   deviceProp.maxThreadsDim[2]);
	printf("  Maximum sizes of each dimension of a grid:     %d x %d x %d\n",
		   int(deviceProp.maxGridSize[0]),
		   int(deviceProp.maxGridSize[1]),
		   int(deviceProp.maxGridSize[2]) );
	printf("  Maximum memory pitch:                          %d bytes\n",
		   int(deviceProp.memPitch));
	printf("  Texture alignment:                             %d bytes\n",
		   int(deviceProp.textureAlignment));
	printf("  Clock rate:                                    %.2f GHz\n",
		   deviceProp.clockRate * 1e-6f);
#if CUDART_VERSION >= 2000
	printf("  Concurrent copy and execution:                 %s\n",
		   deviceProp.deviceOverlap ? "Yes" : "No");
#endif
//	std::cout << "increas by one = " << ++deviceId << std::endl;
//	cudaSetDevice(deviceId);
}

