/*

GPUIMAGEPROCESSING

  THIS MODULE PROVIDES A SIMPLE API FOR IMAGE OPERATIONS ON
  A GRAPHICAL PROCESSING UNIT (GPU) BY MAKING USE OF CUDA 
  TECHNOLOGY.

COPYRIGHT (C)

  ULM UNIVERSITY 
  INSTITUTE OF NEURAL INFORMATION PROCESSING (NEUROINFORMATIK)
  89069 ULM
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  INSTITUTE OF NEURAL INFORMATION PROCESSING:
  http://www.uni-ulm.de/en/in/institute-of-neural-information-processing.html

  ULM UNIVERSITY 
  HTTP://WWW.UNI-ULM.DE/

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
#ifndef __UTIL_HPP__09CC3951_DA41_4FFF_8BEE_01237436CBFF
#define __UTIL_HPP__09CC3951_DA41_4FFF_8BEE_01237436CBFF

#include <string>
#include <cstdio>
#include "GPUVelActivities.h"
#ifdef HAVE_HDF5
	#include "HDF5RawDataFileAdapter.hpp"
#endif

// declarations ---------------------------------------------------------------

void printf_deviceInfo (int deviceId);


// inline implementations -----------------------------------------------------

inline std::string 
imageName (int i) 
{
	char* str = new char[1024];
	sprintf(str,"input_image_%05d",i);
	std::string rtv(str);
	delete[] str;
	return rtv;
}


inline std::string 
featureName (int i) 
{
	char* str = new char[1024];
	sprintf(str,"features_%05d",i);
	std::string rtv(str);
	delete[] str;
	return rtv;
}


inline std::string 
featureNormed0Name (int i) 
{
	char* str = new char[1024];
	sprintf(str,"features_normed_0_%05d",i);
	std::string rtv(str);
	delete[] str;
	return rtv;
}


inline std::string 
featureNormedName (int i) 
{
	char* str = new char[1024];
	sprintf(str,"features_normed_%05d",i);
	std::string rtv(str);
	delete[] str;
	return rtv;
}


inline std::string 
featureNormed2Name (int i) 
{
	char* str = new char[1024];
	sprintf(str,"features_normed_2_%05d",i);
	std::string rtv(str);
	delete[] str;
	return rtv;
}


inline std::string 
initDetName (int i) 
{
	char* str = new char[1024];
	sprintf(str,"init_det_%05d",i);
	std::string rtv(str);
	delete[] str;
	return rtv;
}


inline std::string 
flowFieldName (int i) 
{
	char* str = new char[1024];
	sprintf(str,"V1_flow_%05d",i);
	std::string rtv(str);
	delete[] str;
	return rtv;
}


inline std::string 
flowField_mtName (int i) 
{
	char* str = new char[1024];
	sprintf(str,"MT_flow_%05d",i);
	std::string rtv(str);
	delete[] str;
	return rtv;
}


inline std::string 
flowField_mt_large_Name (int i) 
{
	char* str = new char[1024];
	sprintf(str,"MT_large_flow_%05d",i);
	std::string rtv(str);
	delete[] str;
	return rtv;
}


inline std::string 
hypoName (int i) 
{
	char* str = new char[1024];
	sprintf(str,"V1_%05d",i);
	std::string rtv(str);
	delete[] str;
	return rtv;
}


inline std::string 
hypo_mtName (int i) 
{
	char* str = new char[1024];
	sprintf(str,"MT_%05d",i);
	std::string rtv(str);
	delete[] str;
	return rtv;
}

#ifdef HAVE_HDF5

inline void 
writeGrayImage (
		HDF5RawDataFileAdapter& file,
		const std::string& ds_name,
		const GPUImage& image) 
{
	file.writeData(image,ds_name.c_str());
}


inline void 
writeFeatures (
		HDF5RawDataFileAdapter& file,
		const std::string& ds_name,
		const GPUData& data) 
{
	file.writeData(data,ds_name.c_str());
}


/**
 * This function assumes that the given data set has already been created
 */
inline void 
writeGrayImage(
		HDF5RawDataFileAdapter& file,
		const std::string& ds_name,
		const GPUImage& image,
		const int index) 
{
	file.loadDataSet(ds_name.c_str());
	size_t offs[3] = {index,0,0};
	file.writeData(image,offs);
}


inline void 
writeVelocityField(
		HDF5RawDataFileAdapter& file,
		const std::string& ds_name,
		const GPUVelField& velf) 
{
	file.writeData(velf.getData(),ds_name.c_str());
}


inline void 
writeVelocityActivities(
		HDF5RawDataFileAdapter& file,
		const std::string& ds_name,
		const GPUVelActivities& vela) 
{
	file.writeData(vela.getData(),ds_name.c_str());
}


/**
 * This function assumes that the given data set has already been created
 */
inline void 
writeVelocityActivities(
		HDF5RawDataFileAdapter& file,
		const std::string& ds_name,
		const GPUVelActivities& vela,
		const int index)
{
	size_t offs[4] = {index,0,0,0};
	size_t d = 1;
	GPUData data(vela.getData().reduceDimensionality(d)); // 3d view (h,w,feature)
	data = data.extendDimensionLeft(); // insert singleton dim 0

	file.loadDataSet(ds_name.c_str());
	file.writeData(data,offs);
}

#endif

#endif /* __UTIL_HPP__09CC3951_DA41_4FFF_8BEE_01237436CBFF */

