/*

GPUIMAGEPROCESSING

  THIS MODULE PROVIDES A SIMPLE API FOR IMAGE OPERATIONS ON
  A GRAPHICAL PROCESSING UNIT (GPU) BY MAKING USE OF CUDA 
  TECHNOLOGY.

COPYRIGHT (C)

  ULM UNIVERSITY 
  INSTITUTE OF NEURAL INFORMATION PROCESSING (NEUROINFORMATIK)
  89069 ULM
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  INSTITUTE OF NEURAL INFORMATION PROCESSING:
  http://www.uni-ulm.de/en/in/institute-of-neural-information-processing.html

  ULM UNIVERSITY 
  HTTP://WWW.UNI-ULM.DE/

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
#include <cppunit/extensions/HelperMacros.h>
#include <cppunit/ui/text/TestRunner.h>
#include <boost/format.hpp>

#include <HostImage.h>
#ifdef HAVE_BAUMER
#include <BaumerImageSequence.h>
#endif

#include <cmath>
#include <vector>
#include <unistd.h> // for sleep() function

#define DO_PROFILE
#ifdef DO_PROFILE
#include <time.h>
#endif

static const float eps = 1e-05;

class BaumerImageSequenceTest : public CppUnit::TestFixture {


   	CPPUNIT_TEST_SUITE(BaumerImageSequenceTest);
	  CPPUNIT_TEST(testAccessCamera);
   	CPPUNIT_TEST_SUITE_END();

public:



	void testAccessCamera()
	{
#ifdef HAVE_BAUMER
    // load ImageSequence
    	//std::cout << "\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD"<<std::endl;
		BaumerImageSequence sequence( HostImage::LUM );
		
		HostImage pic = sequence.getNextImage();
		HostImage pic2 = sequence.getCurrentImage();
		
		
		int size = 10;
		std::vector<HostImage> Images( size );
		clock_t timer1 = clock();
		for(int i = 0; i< size; i++)
		{
			clock_t timer0 = clock();

			Images[i] = sequence.getNextImage();
			//Images[i].clear();
			timer0 = clock() - timer0;
			//printf("Image number %d (time %.02fms).\n", Images[i].getFrameNumber(), (timer0*1000.f)/ CLOCKS_PER_SEC );
			fflush( stdout );

			usleep(0);
		}
		timer1 = clock() - timer1;
		//printf("Time needed for %d images: %.02fs.\n", size, (timer1*1.f)/ CLOCKS_PER_SEC );
			
		
	/*
		for(int i = 0; i< size; i++)
		{	
			pic.writeImage( str(boost::format("tests/baumer/test_Baumer_%05i.png")%i).c_str() );
			Images[i].clear();
		}*/
		int w=0, h=0;
		
		pic.getSize(w,h);
		std::cout << "\n    testBaumerImageSequence: got "<<size+1<<" images of size ["<<w<<" x "<<h<<" ]"<<std::endl;
		pic.writeImage( "tests/test_BaumerImageSequence.png" );
		CPPUNIT_ASSERT( w>0 && h>0 );
		//std::cout << "\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD"<<std::endl;
#endif		
	}

};

CPPUNIT_TEST_SUITE_REGISTRATION(BaumerImageSequenceTest);
