/*

GPUIMAGEPROCESSING

  THIS MODULE PROVIDES A SIMPLE API FOR IMAGE OPERATIONS ON
  A GRAPHICAL PROCESSING UNIT (GPU) BY MAKING USE OF CUDA 
  TECHNOLOGY.

COPYRIGHT (C)

  ULM UNIVERSITY 
  INSTITUTE OF NEURAL INFORMATION PROCESSING (NEUROINFORMATIK)
  89069 ULM
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  INSTITUTE OF NEURAL INFORMATION PROCESSING:
  http://www.uni-ulm.de/en/in/institute-of-neural-information-processing.html

  ULM UNIVERSITY 
  HTTP://WWW.UNI-ULM.DE/

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
#include <cppunit/extensions/HelperMacros.h>
#include <cppunit/ui/text/TestRunner.h>


#include <HostImage.h>
#include <SimpleHostImageSequence.h>
#ifdef HAVE_FFMPEG
#include <VideoImageSequence.h>
#endif

#include <cmath>


	static const std::string path_sequence_pattern("exit2_52_56_%05i.png");
	static const std::string path_video("tests/testvideo1.mpg");
	static const std::string path_video_sequence_pattern("tests/testvideo1-%03i.png");

	static const int begin = 0;
	static const int end = 4;
	static const float eps = 1e-05;

class ImageSequenceTest : public CppUnit::TestFixture {


   	CPPUNIT_TEST_SUITE(ImageSequenceTest);
	  CPPUNIT_TEST(testLoadImageSequence);
#ifdef HAVE_FFMPEG
	  CPPUNIT_TEST(testLoadVideoSequence);
#endif
   	CPPUNIT_TEST_SUITE_END();

public:



	void testLoadImageSequence()
	{
    // load ImageSequence
		SimpleHostImageSequence simplesequence( path_sequence_pattern.c_str(), begin, end );
		// Compare it to reference
		for (int i = begin; i <= end; ++i)
		{
		  char* fileName = new char[1024];
		  sprintf(fileName,path_sequence_pattern.c_str(),i);
		  HostImage pic1( fileName, HostImage::FLOAT, HostImage::RGB );
		  HostImage pic2 = simplesequence.getNextImage();
		  pic2.convert_to_type( HostImage::FLOAT );
		  pic2.convert_to_space( HostImage::RGB );
		  float err = pic1.cpu_data_float().manhattan_distance( pic2.cpu_data_float() );
		  
		  int w, h;
		  pic1.getSize(w,h);
		  err /= w * h;
		  
		  std::cout << "err = " << err << std::endl;

		  CPPUNIT_ASSERT( std::abs(err)<eps );

		}
	}

#ifdef HAVE_FFMPEG
	void testLoadVideoSequence()
	{
		// load Video
		VideoImageSequence videosequence( path_video.c_str() );

		// extract the first n frames and compare to reference
		for (int i = begin; i <= end; ++i)
		{
			char* fileName = new char[1024];
			sprintf(fileName,path_video_sequence_pattern.c_str(),i);
			HostImage pic1( fileName, HostImage::UBYTE, HostImage::RGB );
			HostImage pic2 = videosequence.getNextImage();

			float err=0.f;
			int n=0;
			CPUData<HostImage::ubyte>::iterator iter1 = pic1.cpu_data_ubyte().begin();
			CPUData<HostImage::ubyte>::iterator iter2 = pic2.cpu_data_ubyte().begin();
			for (;iter1!=pic1.cpu_data_ubyte().end() && iter2!=pic2.cpu_data_ubyte().end(); iter1++, iter2++)
			{
				err += std::abs((*iter1)[0] - (*iter2)[0]);
				err += std::abs((*iter1)[1] - (*iter2)[1]);
				err += std::abs((*iter1)[2] - (*iter2)[2]);
				++n;
			}
			int w,h;
			pic1.getSize(w,h);
			err /= w*h;

			CPPUNIT_ASSERT( std::abs(err)<eps );
		}

	}
#endif
};

CPPUNIT_TEST_SUITE_REGISTRATION(ImageSequenceTest);
