/*

GTKMMBINDINGS

  THIS MODULE PROVIDES AN INTERFACE FOR VISUALISATION OF OPTIC 
  FLOW, SALIENCY MAPS OR IMAGES BY MAKING USE OF GTK.

COPYRIGHT (C)

  ULM UNIVERSITY 
  INSTITUTE OF NEURAL INFORMATION PROCESSING (NEUROINFORMATIK)
  89069 ULM
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  INSTITUTE OF NEURAL INFORMATION PROCESSING:
  http://www.uni-ulm.de/en/in/institute-of-neural-information-processing.html

  ULM UNIVERSITY 
  HTTP://WWW.UNI-ULM.DE/

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/

/*
 * DataWindow.h
 *
 *  Created on: Dec 16, 2008
 *      Author: doberhoff
 */

#ifndef DataWindow_H_
#define DataWindow_H_

#include "Window.h"
#include "HostImage.h"
#include <GPUVelField.h>


class DataWindow : public Window
{

public:

  struct marker      /// x y colorindex
  {
    float x;
    float y;
    float vx;
    float vy;
    float c;
    marker( float ix, float iy, float ic, float ivx = 0, float ivy = 0 )
      : x(ix), y(iy), c(ic), vx(ivx), vy(ivy)
      {}
  };
  struct box         /// x y colorindex width height
  {
    float x;
    float y;
    float w;
    float h;
    float c;
    float s;
    box( float ix, float iy, float iw, float ih,float ic,float is )
      : x(ix), y(iy), w(iw), h(ih), c(ic), s(is)
      {}
  };
  
  struct ellipse      /// x y colorindex radius1 radius2 angle
  {
    float x;
    float y;
    float c;
    float r1;  // major radius (i.e., semimajor axis)
    float r2;  // minor radius (i.e., semiminor axis)
    float angle;  // in radians
    ellipse( float _x, float _y, float _c, float _r1, float _r2, float _angle )
    : x(_x), y(_y), c(_c), r1(_r1), r2(_r2), angle(_angle)
    {}
  };


  struct trajectory /// list of markers
  {
      std::vector< marker > points;
      trajectory( )
      {}
      void append( float ix, float iy, float ic )
      {
          points.push_back( marker( ix, iy, ic ) );
      }
  };

  enum ArrowStyle {LINES, TRIANGLES, NONE};

  DataWindow(int w, int h, const char* title = "");

  virtual ~DataWindow();

  void update();
  void update(const GPUImage& ref);
  void update( const GPUImage& ref, const GPUVelField& velFiel );
  void update( const GPUVelField& velFiel );
  void update( const GPUImage& ref, const GPUData& features );
  void update( const GPUData& features );
  void update( const GPUData& features, bool flag );

  void set_range_display( float vmin, float vmax );

  void repaint(bool directly=true);

  void add_marker( float x, float y, float c = 0, float vx = 0, float vy = 0 ) { _markers.push_back( marker(x,y,c,vx,vy) ); }
  void clear_markers() { _markers.clear(); }
  void set_marker_size( float s ) { _marker_size = s; }

  void add_box( float x, float y, float w = 0, float h = 0, float c = 0, float s = 0 ) { _boxes.push_back( box(x,y,w,h,c,s) ); }
  void clear_boxes() { _boxes.clear(); }
  void set_box_size( float s ) { _box_size = s; }

  void add_ellipse( const ellipse &  e ) { _ellipses.push_back( e ); }
  void clear_ellipses() { _ellipses.clear(); }

  void set_random_colormap( bool is_random );
  void add_trajectory( trajectory t ){ _trajectories.push_back( t ); }
  void clear_trajectories() { _trajectories.clear(); }

	/// returns the content of the window as HostImage
  HostImage dump_to_host();
	/// writes the content of the window to the specified file
  void writeToFile(std::string filename);

  int getWidth() const { return w; }
  int getHeight() const { return h; }


  static void show(const GPUImage& grayIm, const GPUVelField& image, bool blocking = true);
  static void show(const GPUImage& grayIm, const GPUVelField& image, const char* title, bool blocking = true);
  static void show(const GPUVelField& image, bool blocking = true);
  static void show(const GPUVelField& image, const char* title, bool blocking = true);
  static void show(const GPUImage& image, bool blocking = true);
  static void show(const GPUImage& image, const char* title, bool blocking = true);
  void resize( int nw, int nh );
  void doubleWindowSize();
  void set_colormap( int c );
  void setLineLengthFactor(float f);
  void setMaxSpeed(float s);
  void setArrowSpacing( int spacing );
  void setArrowStyle( ArrowStyle styleA ) { style = styleA; }

  void setHaveLegend( bool b ) { _have_flow_legend = b;}
  void setBrighten( bool br ) {brighten = br;}
  bool getBrighten() const {return brighten;}
  void setSaturateEntropy( bool b ) {saturate_entropy = b;}
  void setMaxIndex( bool b ) { maxindex = b;}
  void setOpacity( float o ) { _opacity = o; }
  void setLineWidth( float w ) { _line_width = w; }
  void setShowFps( bool show ) { _show_fps = show; }
  void setShowMinMax( bool show ) { _show_min_max = show; }
  void setStatusBarVisible( bool b ) { _show_statusbar = b; }
#ifdef HAVE_GLUT
  void setRenderScoreMode( bool rs, int rss, std::string rsp ) { _render_score = rs; _render_score_size = rss, _render_score_pattern = rsp; }
#endif

protected:

  int w;
  int h;
  int arrow_spacing;
  bool brighten, saturate_entropy, maxindex, is_image;
  bool updateEveryTime, hsv;

  std::string  title;

  ArrowStyle style;

  float maxSpeed;
  float min_feat;
  float max_feat;
  float mean_feat;

  Gtk::VBox        vBox;
  Gtk::GL::DrawingArea da;
  Gtk::Statusbar     statusBar;

  GPUImage      xpart;
  GPUImage      ypart;
  GPUImage      speed;
  GPUImage      direction;
  GPUImage      velConfidence;

  GPUImage  imageLocal;

  GPUData featuresLocal;

  GLuint  vbo;
  GLuint  pbo;    // OpenGL pixel buffer object
  GLuint  texid;  // texture

  GPUVelField ffieldRef;
  GPUImage  imageRef;
  GPUData featuresRef;

  void initGUI();
  void initDa();
  void initStatusBar();

  void alloc_pbo_and_texture();
  void alloc_vbo();
  void dealloc_vbo();

  void gl_begin();
  void gl_end();

  void render_plain_image();
  void render_velocity_image();
  void render_feature_image();
  void tex_pbo();
  void set_gl_color( float );
  void render_arrows();
  void draw_arrows();
  void draw_image();
  void draw_markers();
  void draw_boxes();
  void draw_ellipses();
  void draw_trajectories();
  
  void draw();

  virtual void closing();
  virtual bool window_event(GdkEventWindowState *event);
  virtual bool config_event(GdkEventConfigure *event);
  virtual bool expose_event(GdkEventExpose *event);

  int 	_colormap;
  double                _last_time;
  double                _fps;
  bool                  _have_flow_legend;
  bool                  _show_min_max;
  bool                  _show_fps;
  std::vector< marker > _markers;
  float                 _marker_size;
  std::vector< box >    _boxes;
  float                 _box_size;
  float                 _line_width;
  float                 _opacity;
  std::vector< ellipse > _ellipses;
  std::vector< trajectory > _trajectories;
  bool                  _random_colormap;
  bool                  _show_statusbar;
  
#ifdef HAVE_GLUT
  bool                  _render_score;
  int                   _render_score_size;
  std::string           _render_score_pattern;
#endif
};

#endif /* DataWindow_H_ */
