/*
GTKMMBINDINGS

  THIS MODULE PROVIDES AN INTERFACE TO OPEN GUI ELEMENTS
  BY MAKING USE OF GTK.

DEVELOPED 

  2008-2011 BY
  ULM UNIVERSITY 
  INSTITUTE OF NEURAL INFORMATION PROCESSING (NEUROINFORMATIK)
  89069 ULM
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  INSTITUTE OF NEURAL INFORMATION PROCESSING:
  http://www.uni-ulm.de/en/in/institute-of-neural-information-processing.html

  ULM UNIVERSITY 
  HTTP://WWW.UNI-ULM.DE/

This software is governed by the CeCILL  license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

#include <cppunit/extensions/HelperMacros.h>
#include <cppunit/ui/text/TestRunner.h>
#include <GPUImage.h>
#include <GPUVelField.h>
#include <HostImage.h>
#include <kernels.hpp>
#include "downsample.hpp"
#include "DataWindow.hpp"

class GTKImageTest : public CppUnit::TestFixture
{

	CPPUNIT_TEST_SUITE(GTKImageTest);
	//	CPPUNIT_TEST(testGrayImage);
	CPPUNIT_TEST(testCreation);
	CPPUNIT_TEST_SUITE_END();

public:

//	void testGrayImage()
//	{
//		GrayImageWindow::global_gtk_gl_init();
//		const int w = 512;
//		const int h = 512;
//		GPUImage a( HostImage("../GPUImageProcessing/lena.png") );
//		CPPUNIT_ASSERT(!a.isEmpty());
//		CPPUNIT_ASSERT(a.isContiguous());
//		CPPUNIT_ASSERT(a.getXRes()==w);
//		CPPUNIT_ASSERT(a.getYRes()==h);
//
//		GrayImageWindow gwindow(a);
//		gwindow.wait_until_closed();
//	}

	void testCreation()
	{
		Window::global_gtk_gl_init();
		const int w = 512;
		const int h = 512;
		GPUImage a(HostImage("../GPUImageProcessing/lena.png"));
		CPPUNIT_ASSERT(!a.isEmpty());
		CPPUNIT_ASSERT(a.isContiguous());
		CPPUNIT_ASSERT(a.getXRes()==w);
		CPPUNIT_ASSERT(a.getYRes()==h);

		int ndown = 4;

		GPUVelField velField(w/ndown,h/ndown);
		velField.getData().fill(float(2/fmax(double(w/ndown),double(h/ndown))));

		GPUData line(w/ndown);
		GPUData col(w/ndown);
		fillLinSpace(line,-w/2.0/ndown,w/2.0f/ndown);
		fillLinSpace(col,-h/2.0/ndown,h/2.0f/ndown);
		velField.getXPart().mulRepeated(line);
		velField.getYPart().mulRepeated(col);

		DataWindow window( 200, 300, "test" );
		window.update( a, velField );

		window.wait_until_closed();

		for (int i=0; i<20; i++)
			Window::process_all_pending_gtk_events();

	}



};

CPPUNIT_TEST_SUITE_REGISTRATION(GTKImageTest);

int main() {
	CppUnit::TextUi::TestRunner runner;
	CppUnit::TestFactoryRegistry& registry = CppUnit::TestFactoryRegistry::getRegistry();
	runner.addTest(registry.makeTest());
	return ! runner.run("", false);
}


// Local Variables:
// mode: c++
// coding: utf-8
// c-basic-offset: 4
// tab-width: 4
// End:
