/*
Copyright INRIA 2011

Authors: Jérôme Piovano, Jean Philippe Pons, Mikael Rousson,
         Theodore Papadopoulo <Theodore.Papadopoulo@inria.fr>

This software is a computer program whose purpose is to implement various
segmentation algorithms based on level sets.

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL-C
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.
*/

#include <FastMarching.h>
#include <Globals.h>
#include <Schemes.h>

#include <blitz/array.h>

#include <cmath>
#include <sstream>

/**  @example 2D_FastMarching.C 2D Fast Marching
*/
int main(int argc, char* argv[])
{	
	using namespace blitz;
	using namespace levelset;

	const int nx = 500;
  	const int ny = 500;

	TinyVector<int, 2> shape(nx, ny);

	Array<float, 2>    image(shape);    // image of size nx*ny
	Eikonal<2,float>   march(&image);   // fast marching eikonal 

	// initialization 
	image = 0.0;

	srand(10000);

	// 20 random seeds
	for (int k = 0 ; k < 20 ; k++){

		int x = static_cast<int>(nx * (double)rand() / ((double)(RAND_MAX)+(double)(1)));
		int y = static_cast<int>(ny * (double)rand() / ((double)(RAND_MAX)+(double)(1)));
		Array<float, 2>::T_index ind(x, y);

		march.add_trial_point(ind);
	}

	// 20 circles of forbidden points of radius <= 30
	for (int k = 0 ; k < 20 ; k++){

		int x = static_cast<int>(30 + (nx-60) * (double)rand() / ((double)(RAND_MAX)+(double)(1)));
		int y = static_cast<int>(30 + (ny-60) * (double)rand() / ((double)(RAND_MAX)+(double)(1)));
		int r = static_cast<int>(          30 * (double)rand() / ((double)(RAND_MAX)+(double)(1)));
	
		for (int i = x-r ; i < x+r ; i++)
			for (int j = y-r ; j< y+r ; j++)
				if(sqrt( (double) (i - x)*(i - x) + (j - y)*(j - y)) < r){
					Array<float, 2>::T_index ind(i, j);
					march.add_forbidden_point(ind);
				}
	}

	//timeval _time;
	//Globals::initChrono(_time); 

	// fast marching
	march.run();
  
	//Globals::printChrono("\n	=> FastMarching : ", _time);
	//Globals::initChrono(_time); 

	system ("mkdir -p results");
	//Globals::save(image, "results/2D_Fastmarching.inr");
	//Globals::save(image, "results/2D_Fastmarching.png");

	return 0;
}

