/*
Copyright INRIA 2011

Authors: Jérôme Piovano, Jean Philippe Pons, Mikael Rousson,
         Theodore Papadopoulo <Theodore.Papadopoulo@inria.fr>

This software is a computer program whose purpose is to implement various
segmentation algorithms based on level sets.

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL-C
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.
*/

#include <Segmentation.h>
#include <SegmentationModule/RegularizationModule.h>
#include <SegmentationModule/GaussianRegionModule.h>
#include <blitz/array.h>

/**  @example 2D_Gaussian.C Gaussian Region Segmentation of a 2D Image
*/
int main(int argc, char* argv[])
{
	using namespace blitz;
 	using namespace levelset;
 	using namespace segmentation;

	// image to segment
	Array<float, 2> image;
	Globals::load(image, "images/tiger.inr");
	Globals::normalize<2, float>(image, 0.0f, 250.0f);

	// initialization of the segmentations objects
        FastMarchingInitializer<2> init;                               // Fast Marching Initializer
	LevelSet<2>                phi(image.shape(), (double) 2.0);   // Level Set of band thickness = 2
	Segmentation<2,float>      segm(&phi, &init, &image);          // Segmentation

	// Initalization of the level set
	LevelSet<2>::Index tl(100, 100);
	LevelSet<2>::Index br(381, 221);
	phi.init_rectangle(tl, br, init);

	// initialization of the modules
	RegularizationModule<2>        regul(1.f);                  // parameters : weight
	GaussianRegionModule<2,float>  gaussian(&image, 0.2f, true);   // parameters : image, weight, variance of the kernel

        segm.add_module(regul);
	segm.add_module(gaussian);

	// launch the segmentation
	timeval _time;
	Globals::initChrono(_time);

	std::ostringstream name;
	name.str("");  
	name << "mkdir -p results/gaussian";
	system (name.str().c_str());
	name.str("");  
	name << "results/gaussian/";
	Globals::initChrono(_time);

        segm.segment(2000, name.str().c_str());

	Globals::printChrono("\n	=> Gaussian : ", _time);
	Globals::initChrono(_time); 

        std::cout << "OK" << std::endl;

	return 0;
}
