/*
Copyright INRIA 2011

Authors: Jérôme Piovano, Jean Philippe Pons, Mikael Rousson,
         Theodore Papadopoulo <Theodore.Papadopoulo@inria.fr>

This software is a computer program whose purpose is to implement various
segmentation algorithms based on level sets.

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL-C
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.
*/

#ifndef BALLOONMODULE_H
#define BALLOONMODULE_H

#include <SegmentationModule/SegmentationModule.h>

namespace segmentation {

	/**
	* @author Jerome Piovano
	*
	* @brief SegmentationModule specialization used to drive the LevelSet with a balloon force thanks to the weight 
	*
	* @note can be used for interactivity purpose thanks to an image weight
	*/
	template<unsigned DIM, typename Pixel = float>
	class BalloonModule : public SegmentationModule<DIM, Pixel>
	{
	public:

		typedef           SegmentationModule<DIM, Pixel>  base;
		typedef typename  base::LevelSetType              LevelSetType;
		typedef typename  base::Index                     Index;
		typedef typename  base::SchemesType               SchemesType;

		/** @name Constructors - Destructor */
		//@{

		/// Constructor
		BalloonModule(float _weight=1.f   ///< weight of the module
			      );

		/// Destructor
		virtual ~BalloonModule();

		//@}
		/** @name Member functions */
		//@{

		/// Value of the module at a point
		float value(const LevelSetType & phi, const Index & ind);
		/// Energy of the module at a point
		float energy(const LevelSetType & phi, const Index & ind);
	};


	////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// Constructors - Destructor


	template <unsigned DIM, typename Pixel>
		BalloonModule<DIM, Pixel>::BalloonModule(float _weight)
		:SegmentationModule<DIM, Pixel>(_weight, false)
	{ }

	template <unsigned DIM, typename Pixel>
		BalloonModule<DIM, Pixel>::~BalloonModule() 
	{ }


	////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// Member functions


	template <unsigned DIM, typename Pixel>
		float BalloonModule<DIM, Pixel>::value(const LevelSetType & phi, const Index & ind)
	{
		return  SchemesType::norm_gradient(phi,ind);
	}	

	template <unsigned DIM, typename Pixel>
		float BalloonModule<DIM, Pixel>::energy(const LevelSetType & phi, const Index & ind)
	{
		return  0;
	}
}

#endif // BALLOONMODULE_H
