/*

MATH

  THIS MODULE INCLUDES GENERIC MATH UTILITIES

COPYRIGHT (C)

  FRAUNHOFER FIT-LIFE
  SCHLOSS BIRLINGHOVEN
  53754 ST. AUGUSTIN
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  Fraunhofer-Institut für Angewandte Informationstechnik FIT 
  http://fit.fraunhofer.de

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
#ifndef GAUSSIAN2DIM_HPP
#define GAUSSIAN2DIM_HPP


// normalized Gaussian, 2-dimensional
// Author: Frank 20.08.2010

#include "matrix_ops.hpp"
#include <stdexcept>
#include <boost/format.hpp>

class Gaussian2dim
{
	
public:	

	Gaussian2dim()
	{
		mean0 = mean1 = 0;
		covariance = Matrix( ublas::identity_matrix<double>(2) );
	}

	Gaussian2dim( float m0, float m1, float var0, float cov01, float var1 )
	: mean0( m0 )
	, mean1( m1 )
	{
		if (var0*var1 - cov01*cov01 < 0)
			throw std::runtime_error( str(boost::format("Not a valid covariance matrix! Not positive definite. var0=%f var1=%f cov01=%f det=%f - %f < 0!")%var0%var1%cov01%(var0*var1)%(cov01*cov01) ));
		covariance.resize(2,2);
		covariance(0,0) = var0;
		covariance(0,1) = covariance(1,0) = cov01;
		covariance(1,1) = var1;
	}
	
	
	float eval( float x0, float x1 )
	{
		// calculate precision matrix once when eval is called the first time
		if ( _precision.size1() != 2 )
			calculate_precision();
		GaussianUnnormalized unnormalized ( _precision );
		if ( _det_precision < 0 )
			std::cout << "_det_precision = " << _det_precision << std::endl;
		float output = _det_precision * unnormalized.eval(x0 - mean0, x1 - mean1);
		if ( output < 0)
			std::cout << "output = " << output << std::endl;
		output /= 2*M_PI;
		return output;
	}
	
	
	Matrix Precision()
	{
		if ( _precision.size1() != 2 )
			calculate_precision();
		return _precision;
	}
	
	// deprecated
	Matrix Covariance()
	{
		return covariance;
	}
	
	Gaussian2dim &operator = (const Gaussian2dim &other)
	{
		if ( this != &other )
		{
			mean0 = other.mean0;
			mean1 = other.mean1;
			covariance = other.covariance;
		}
		return *this;
	}
	
	
	float mean0, mean1;
	Matrix covariance;
	
	
private:
	
	void calculate_precision()
	{
		_precision = Matrix(2,2);
		bool ok = matrix_inverse( covariance, _precision );
		if (!ok)
			throw std::runtime_error("Inversion of precision matrix of Gaussian2dim failed. Error.");

		_det_precision = _precision(0,0)*_precision(1,1) - _precision(0,1)*_precision(1,0);
		if ( _det_precision < 0 )
			throw std::runtime_error( str(boost::format("Determinant of precision matrix: %f < 0 !")%_det_precision ) );
	}
	
	
	
	Matrix _precision;
	float _det_precision;
	
};

#endif
