/*

MATH

  THIS MODULE INCLUDES GENERIC MATH UTILITIES

COPYRIGHT (C)

  FRAUNHOFER FIT-LIFE
  SCHLOSS BIRLINGHOVEN
  53754 ST. AUGUSTIN
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  Fraunhofer-Institut für Angewandte Informationstechnik FIT 
  http://fit.fraunhofer.de

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
#pragma once

#include <iostream>
#include <boost/format.hpp>
#include <stdexcept>

template<typename input_iter_t, typename T1, typename T2>
void update_beta_simple_with_polynomial_likelihood( input_iter_t pcoeffs_begin, input_iter_t pcoeffs_end
													,input_iter_t ncoeffs_begin, input_iter_t ncoeffs_end
													, T1 & mu, T2 & N, double delta = 1.0 )
{
  double alpha = mu * N;
  double beta = ( 1 - mu ) * N;

  double Z        = 0;
  double Eq     = 0;
  double Eq2 = 0;

  if ( ( ncoeffs_end != ncoeffs_begin ) && ( pcoeffs_end != pcoeffs_begin ) )
	{
	  double Eqi = 1;
	  input_iter_t ipcoeffs = pcoeffs_begin;
	  for( int i = 0; ipcoeffs != pcoeffs_end; ++ipcoeffs, ++i )
		{
		  input_iter_t incoeffs = ncoeffs_begin;
		  double Eqinqj = Eqi;
		  for( int j = 0; incoeffs != ncoeffs_end; ++incoeffs, ++j )
			{
			  double Einq = ( beta  + j ) / ( alpha + beta + i + j );
			  double Eiq  = ( alpha + i ) / ( alpha + beta + i + j );
			  double Eiq2 = Eiq * ( alpha + i + 1 ) / ( alpha + beta + i + j + 1 );
			  double w = *ipcoeffs * *incoeffs * Eqinqj;
			  Z += w;
			  Eq += w * Eiq;
			  Eq2 += w * Eiq2;
			  Eqinqj *= Einq;
			}
		  double Eiq  = ( alpha + i ) / ( alpha + beta + i );
		  Eqi *= Eiq;
		}
	}
  else if ( pcoeffs_end != pcoeffs_begin )
	{
	  double Eqi = 1;
	  input_iter_t ipcoeffs = pcoeffs_begin;
	  for( int i = 0; ipcoeffs != pcoeffs_end; ++ipcoeffs, ++i )
		{
		  double Eiq  = ( alpha + i ) / ( alpha + beta + i );
		  double Eiq2 = Eiq * ( alpha + i + 1 ) / ( alpha + beta + i + 1 );
		  double w = *ipcoeffs * Eqi;
		  Z += w;
		  Eq += w * Eiq;
		  Eq2 += w * Eiq2;
		  Eqi *= Eiq;
		}
	}
  else if ( ncoeffs_end != ncoeffs_begin )
	{
	  double Enqi = 1;
	  input_iter_t incoeffs = ncoeffs_begin;
	  for( int i = 0; incoeffs != ncoeffs_end; ++incoeffs, ++i )
		{
		  double Einq  = ( beta + i ) / ( alpha + beta + i );
		  double Eiq  = ( alpha ) / ( alpha + beta + i );
		  double Eiq2 = Eiq * ( alpha + 1 ) / ( alpha + beta + i + 1 );
		  double w = *incoeffs * Enqi;
		  Z += w;
		  Eq += w * Eiq;
		  Eq2 += w * Eiq2;
		  Enqi *= Einq;
		}
	}

  //  std::cout << str( boost::format( " Z <q> = %1.2e Z <q^2> = %1.2e Z = %1.2e " ) % Eq % Eq2 % Z ) << std::endl;
  if ( Z <= 0 )
	{
	  std::cout << " Z = " << Z << " < 0 skipping..." << std::endl;
	  return;
	}
  Eq /= Z;
  Eq2 /= Z;
  if ( Eq2 == Eq * Eq )
	{
	  std::cout << " singularity, skipping..." << std::endl;
	  return;
	}
  double Npost = ( Eq - Eq2 ) / ( Eq2 - Eq * Eq );
  if ( Npost < 1e-3 || Eq < 0 || Eq > 1 )
	{
	  std::cout << " mu " << mu << " -> " << Eq << " dmu = " << ( mu - Eq ) 
				<< " N " << N << " -> " << Npost << " dN " << ( Npost - N ) << " Z " << Z;
	  std::cout << " skipping... " << std::endl;
	  return;
	}
  N  += delta * ( Npost - N );
  mu += delta * ( Eq - mu );
}
											
