/*

MATH

  THIS MODULE INCLUDES GENERIC MATH UTILITIES

COPYRIGHT (C)

  FRAUNHOFER FIT-LIFE
  SCHLOSS BIRLINGHOVEN
  53754 ST. AUGUSTIN
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  Fraunhofer-Institut für Angewandte Informationstechnik FIT 
  http://fit.fraunhofer.de

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
#include "../include/sse_approx_logexp.h"
#include "../include/sse_math.hpp"

#include <stdio.h>
#include <cmath>
#include <assert.h>
#include <stdlib.h>

#define floatrand(x0,x1) ( ((float(random())/float(RAND_MAX))-x0) * ( x1 - x0 ) )

void test_mvp( size_t nx, size_t ny, float xmin, float xmax )
{
  float * x1 = new float[nx];
  float * x2 = new float[nx];
  float * y = new float[ny];
  float * M = new float[nx * ny];
  for ( size_t i = 0; i < nx; ++i ) x1[i] = x2[i] = 0.0f;
  for ( size_t i = 0; i < ny; ++i ) y[i] = floatrand(xmin,xmax);
  for ( size_t i = 0; i < nx * ny; ++i ) M[i] = floatrand(xmin,xmax);
  for ( size_t i = 0, k = 0; i < nx; ++i )
	for ( size_t j = 0; j < ny; ++j, ++k )
	  x1[i] += y[j] * M[k];
  sse_mvp_accum( x2, nx, y, ny, M, ny );
  for ( size_t i = 0; i < nx; ++i ) 
	{
	  float err = x2[i] - x1[i];
	  float relerr = std::abs(err) / x1[i];
	  assert( relerr < 1e-6 );
	}
  sse_mvp( x2, nx, y, ny, M, ny );
  for ( size_t i = 0; i < nx; ++i ) 
	{
	  float err = x2[i] - x1[i];
	  float relerr = std::abs(err) / x1[i];
	  assert( relerr < 1e-6 );
	}
}

int main()
{
  

#if 1

  for ( float x = -1000.0f; x < 50.0f; x += 0.1f )
	{
	  __m128 xxxx = _mm_set1_ps( x );
	  __m128 expxxxx = expf4( xxxx );
	  __m128 logexpxxxx = logf4( expf4( xxxx ) );
	  float errlogexp = ( *((float*)&logexpxxxx) - x ) / x;
	  float expx = std::exp(x);
	  float errexp = std::abs( ( *((float*)&expxxxx) - expx ) / expx );
	  assert( ( x < -60.0f && (*((float*)&expxxxx) ) < 1e-30f ) ||  errexp < 1e-5f );
	  assert( ( x < -80.0f ) ||  errlogexp < 1e-4f );
	}
  for ( float x = 0.1f; x < 10000.0f; x *= 1.1f )
  {
    float xx = x;
    __m128 xxxx = _mm_set1_ps( xx );
    __m128 logxxxx = logf4( xxxx );
    __m128 expxxxx = expf4( xxxx );
    __m128 logexpxxxx = logf4( expf4( xxxx ) );
    __m128 explogxxxx = expf4( logf4( xxxx ) );
    float logx = std::log(xx);
    float expx = std::exp(xx);
	float errlog = std::abs( ( *((float*)&logxxxx) - logx ) / logx );
	float errexp = std::abs( ( *((float*)&expxxxx) - expx ) / expx );
	float errlogexp = std::abs( ( *((float*)&logexpxxxx) - xx ) / xx );
	float errexplog = std::abs( ( *((float*)&logexpxxxx) - xx ) / xx );
	assert( ( abs( xx - 1.0f ) < 1e-5 && ( *((float*)&logxxxx) < 1e-5 ) ) ||  errlog < 1e-5f );
	if ( x < 95.0f )
	  {
		std::cout << " x " << x << " err log " << ( errlog / std::abs(logx) ) << std::endl;
		assert( errexp < 1e-5f );
		assert( errlogexp < 1e-5f );
		assert( errexplog < 1e-5f );
	  }
  }  

#endif

  test_mvp( 777, 223, -100.0f, 100.0f );
  
  return 0;
}
