/*
 * OpenCVImage.cpp
 *
 *  Created on: 04-Feb-2009
 *      Author: feisenkeil
 */

#include "OpenCVImage.hpp"

OpenCVImage::OpenCVImage() {
	capture = 0;
	startCapture();
}

OpenCVImage::~OpenCVImage() {
	if (capture!=0)
		stopCapture();
}


void OpenCVImage::startCapture(){

	/**
	 * Capture device, Parameter CV_CAP_ANY is an integer
	 */
	capture = cvCaptureFromCAM( CV_CAP_ANY );
	if( !capture ) {
		fprintf( stderr, "ERROR: capture is NULL \n" );
		getchar();
	}

}

void OpenCVImage::stopCapture(){
	cvReleaseCapture( &capture );
	capture=0;
}

float* OpenCVImage::queryImageFromCam(float* data){

	// Get one frame for allocation memory
	IplImage* frame = cvQueryFrame( capture );
	if( !frame ) {
		fprintf( stderr, "ERROR: frame is null...\n" );
		getchar();
	}

	// setting private variables
	h = frame->height;
	w = frame->width;
	size = frame->width * frame->height;


	static float* greyArray;
	if (data==0)
		greyArray = (float*) malloc(size*sizeof(float));
	else
		greyArray = data;

	for(int i = 0; i < size; i++){
		greyArray[i] = (float)(((unsigned char)(frame->imageData[i*3]))/255.0f);
	}
	return greyArray;
}


GPUImage OpenCVImage::getAsGPUImage(){

	float* greyFrame = this->queryImageFromCam();
	GPUImage im(w,h);
	im.copyFromHostMemory(greyFrame);
  free(greyFrame); // daniel: this was not done before, and greyFrame leaked every time
	return im;
}
