/*
 * OpenCVImage.hpp
 *
 *  Created on: 04-Feb-2009
 *      Author: feisenkeil
 */

#ifndef OPENCVIMAGE_HPP_
#define OPENCVIMAGE_HPP_

#include "cv.h"
#include "highgui.h"

#include <stdio.h>
#include <GPUImage.h>

/**
 * Class OpenCVImage provides methods to capture greyscaled pictures from webcam.
 * To grab pictures the OpenCV toolkit is used.
 */

class OpenCVImage {

private:

	/** width and hight */
	int w,h;

	/** Parameter for video capturing functions. */
	CvCapture* capture;

	/** number of grey values */
	int size;
	/**
	 * Returns float array representing grayValue picture captured by webcam
	 */
	float* queryImageFromCam(float* data = 0);

public:

	/**
	 * Constructor: Calls method startCapture() on declaration
	 */
	OpenCVImage();
	/**
	 * Destructor: Calls method stopCapture()
	 */
	virtual ~OpenCVImage();

	/**
	 * creates capturing interface (CvCapture* capture) for grabbing pictures
	 */
	void startCapture(void);
	/**
	 * deletes cepturing interface
	 */
	void stopCapture();

	/**
	 * grabs pictures from capturing interface and writes them to GPU. The returned result is a GPUImage
	 */
	GPUImage getAsGPUImage();
};

#endif /* OPENCVIMAGE_HPP_ */
