/*

RANDOM

  THIS MODULES PROVIDES UTILITIES FOR CREATING RANDOM NUMBERS IN VARIOUS DISTRIBUTIONS
  AND CONCURRENTLY

COPYRIGHT (C)

  FRAUNHOFER FIT-LIFE
  SCHLOSS BIRLINGHOVEN
  53754 ST. AUGUSTIN
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  Fraunhofer-Institut für Angewandte Informationstechnik FIT 
  http://fit.fraunhofer.de

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
/////////////////////////////////////////////////////////////////////////////
// The Software is provided "AS IS" and possibly with faults. 
// Intel disclaims any and all warranties and guarantees, express, implied or
// otherwise, arising, with respect to the software delivered hereunder,
// including but not limited to the warranty of merchantability, the warranty
// of fitness for a particular purpose, and any warranty of non-infringement
// of the intellectual property rights of any third party.
// Intel neither assumes nor authorizes any person to assume for it any other
// liability. Customer will use the software at its own risk. Intel will not
// be liable to customer for any direct or indirect damages incurred in using
// the software. In no event will Intel be liable for loss of profits, loss of
// use, loss of data, business interruption, nor for punitive, incidental,
// consequential, or special damages of any kind, even if advised of
// the possibility of such damages.
//
// Copyright (c) 2003 Intel Corporation
//
// Third-party brands and names are the property of their respective owners
//
///////////////////////////////////////////////////////////////////////////
// Random Number Generation for SSE / SSE2
// Source File
// Version 0.1
// Author Kipp Owens, Rajiv Parikh
////////////////////////////////////////////////////////////////////////

#ifndef RAND_SSE_H
#define RAND_SSE_H
#include "emmintrin.h"
//#define COMPATABILITY
//define this if you wish to return values similar to the standard rand();

class rand_sse
{
  
public:
  
  typedef unsigned int result_type;
  static const bool has_fixed_range = true;
  static const result_type min_value = 0;
#ifdef COMPATIBILITY
  static const result_type max_value  = 32768;
#else
  static const result_type max_value  = 2147483648;
#endif

  explicit rand_sse( )
  {
	seed( 1 );
  }

  explicit rand_sse( unsigned int seed  )
  {
	this->seed( seed );
  }
  
  inline void seed( unsigned int seed )
  {
	_cur_seed = _mm_set_epi32( seed, seed+1, seed, seed+1 );
  }

  result_type min() { return min_value; }
  result_type max() { return max_value; }

  inline unsigned int operator()()
  {
	unsigned int resultv;
	unsigned int* result = &resultv;
	__m128i cur_seed_split;
	 __m128i multiplier;
	 __m128i adder;
	 __m128i mod_mask;
	 __m128i sra_mask;
	 static const unsigned int mult[4] =
	  { 214013, 17405, 214013, 69069 };
	 static const unsigned int gadd[4] =
	  { 2531011, 10395331, 13737667, 1 };
	 static const unsigned int mask[4] =
	  { 0xFFFFFFFF, 0, 0xFFFFFFFF, 0 };
	 static const unsigned int masklo[4] =
	  { 0x00007FFF, 0x00007FFF, 0x00007FFF, 0x00007FFF };
	
	adder = _mm_load_si128( (__m128i*) gadd);
	multiplier = _mm_load_si128( (__m128i*) mult);
	mod_mask = _mm_load_si128( (__m128i*) mask);
	sra_mask = _mm_load_si128( (__m128i*) masklo);
	cur_seed_split = _mm_shuffle_epi32( _cur_seed, _MM_SHUFFLE( 2, 3, 0, 1 ) );
	
	_cur_seed = _mm_mul_epu32( _cur_seed, multiplier );
	multiplier = _mm_shuffle_epi32( multiplier, _MM_SHUFFLE( 2, 3, 0, 1 ) );
	cur_seed_split = _mm_mul_epu32( cur_seed_split, multiplier );
	_cur_seed = _mm_and_si128( _cur_seed, mod_mask);
	cur_seed_split = _mm_and_si128( cur_seed_split, mod_mask );
	cur_seed_split = _mm_shuffle_epi32( cur_seed_split, _MM_SHUFFLE( 2, 3, 0, 1 ) );
	_cur_seed = _mm_or_si128( _cur_seed, cur_seed_split );
	_cur_seed = _mm_add_epi32( _cur_seed, adder);
#ifdef COMPATABILITY
	 __m128i sseresult;
	sseresult = _mm_srai_epi32( _cur_seed, 16);
	sseresult = _mm_and_si128( sseresult, sra_mask );
	_mm_storeu_si128( (__m128i*) result, sseresult );
#endif
	_mm_storeu_si128( (__m128i*) result, _cur_seed);


	return resultv;

  }
  

private:

  __m128i _cur_seed;
  
};

#undef COMPATIBILITY

#endif
