/*

SHAPELEARNING

  THIS MODULE PROVIDES A FRAMEWORK FOR CONVOLUTIONAL BAYESIAN NETWORKS
  BY PROVIDING MODULES TO BUILD THE LAYERS, THE COMPOSITION NEEDS TO
  BE HANDLED BY THE USER

COPYRIGHT (C)

  FRAUNHOFER FIT-LIFE
  SCHLOSS BIRLINGHOVEN
  53754 ST. AUGUSTIN
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  Fraunhofer-Institut für Angewandte Informationstechnik FIT 
  http://fit.fraunhofer.de

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
#ifndef BATCH_NG_HPP
#define BATCH_NG_HPP

#ifdef BZ_DEBUG
using namespace std;
#endif

#include "SeariseProperties.hpp"
#include "blitz_properties.hpp"
#include <blitz/array.h>

#include <boost/scoped_ptr.hpp>
#include <boost/serialization/serialization.hpp>

template<typename T, typename Metric> 
class BatchNG : public HasProperties< SearisePropertyTypes, BatchNG<T,Metric> >
{
  
public:

  typedef BatchNG This;
  typedef HasProperties<SearisePropertyTypes, This> PBase;
  
  typedef blitz::Array<T,1> Matrix1;
  typedef blitz::Array<T,2> Matrix2;
  typedef blitz::Array<T,3> Matrix3;
  typedef blitz::Array<T,4> Matrix4;
  
  typedef blitz::Array<double,1> Matrix1d;
  typedef blitz::Array<double,2> Matrix2d;
  typedef blitz::Array<double,3> Matrix3d;
  typedef blitz::Array<double,4> Matrix4d;
  
  BatchNG(bool do_estimate_variances = true);
  virtual ~BatchNG();
  
  static void init_properties();
  void initialize(int k, int d1, int d2, int m);
  void begin_epoch();
  void process(const Matrix3 & data);
  void process(const Matrix3 & data, const Matrix2 & saliency);
  void end_epoch();
  
  Matrix1d get_histogram();
  Matrix3d get_histogram3();
  Matrix4 get_centers();
  Matrix4 get_variances();
  Matrix4d get_centerStat();
  Matrix4d get_squareStat();
  
  bool ready()
  {
	return _ready;
  }
    
private:


  Matrix1 compute_dists( const Matrix3 & data );
  Matrix1 compute_dists( const Matrix3 & data, const Matrix2 & weights );
  Matrix1 compute_ranking( const Matrix1 & dists );
  Matrix1 rand_ranking();
  
  T _lambda_i;
  T _lambda_f;
  int _itmax;
  int _minIter;
  int _maxIter;
  T _ctreshold;
  bool _ready;
  double _costs;
  double _lastcosts;
  int _k, _d1, _d2, _m;
  
  bool _initialized;
  bool _do_estimate_variances;
  bool _estimate_variances;
  int _it;
  T _lambda;
  
  
  Matrix4 _centers;
  Matrix4d _varianceStat;
  Matrix4d _centerStat;
  Matrix1d _weightStat;
  Matrix3d _weightStat3;
  Matrix1 _ranks;
  
};


#endif
