/*

SHAPELEARNING

  THIS MODULE PROVIDES A FRAMEWORK FOR CONVOLUTIONAL BAYESIAN NETWORKS
  BY PROVIDING MODULES TO BUILD THE LAYERS, THE COMPOSITION NEEDS TO
  BE HANDLED BY THE USER

COPYRIGHT (C)

  FRAUNHOFER FIT-LIFE
  SCHLOSS BIRLINGHOVEN
  53754 ST. AUGUSTIN
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  Fraunhofer-Institut für Angewandte Informationstechnik FIT 
  http://fit.fraunhofer.de

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
#pragma once

#include <blitz/array.h>

// O(N^2)-algorithm to determine the counting distribution of a multivariate binomial
template<typename T>
void bit_count_distrib( const blitz::Array<T,1> & mu, blitz::Array<T,1> & pN );

// O(N^2)-algorithm to determine the counting distribution of a multivariate binomial, skipping one bit
template<typename T>
T bit_count_distrib_skip_bit( const blitz::Array<T,1> & mu, blitz::Array<T,1> & pN, int skip );

// O(N^2)-algorithm to quickly generate the gate distribution from the counting distribution induced by the mask distributions, probably unstable
template<typename T>
T gating_distrib_fast(  const blitz::Array<T,1> & mu, const blitz::Array<T,1> & pN, blitz::Array<T,1> & pg );

// O(N^3)-algorithm to generate the gate distribution from the counting distribution induced by the mask distributions, stable
template<typename T>
T gating_distrib_slow(  const blitz::Array<T,1> & mu, const blitz::Array<T,1> & pN, blitz::Array<T,1> & pg );
