/*

SHAPELEARNING

  THIS MODULE PROVIDES A FRAMEWORK FOR CONVOLUTIONAL BAYESIAN NETWORKS
  BY PROVIDING MODULES TO BUILD THE LAYERS, THE COMPOSITION NEEDS TO
  BE HANDLED BY THE USER

COPYRIGHT (C)

  FRAUNHOFER FIT-LIFE
  SCHLOSS BIRLINGHOVEN
  53754 ST. AUGUSTIN
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  Fraunhofer-Institut für Angewandte Informationstechnik FIT 
  http://fit.fraunhofer.de

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
#ifndef MESSAGE_COMPONENT_HPP_P1CDJQ5L
#define MESSAGE_COMPONENT_HPP_P1CDJQ5L

#include "mmm2_belief_message_buffer.hpp"

template< typename T >
struct message_component
{
  typedef mmm2_belief_state<T> belief_state;
  typedef mmm2_belief_message_buffer<T> message_buffer;

  /// default construct on copy constructor, to support use in containers
  /// the const cast is ugly, but I really want to reference the same buffer
  /// if it is indeed by reference, to support proper resize semtantic of, f.e., std::vector< messsage_component<T> >
  message_component( const message_component<T> & other )
	: _buffer( other.owns_buffer() ? new message_buffer : const_cast<message_buffer*>( other._buffer ) )
	, _buffer_owned( other.owns_buffer() )
	, _message_index( other.message_index() )
  {
    if ( _buffer_owned )
	  {
		_buffer->occupy_slot( _message_index );
	  }
  }

  message_component()
  : _buffer( new message_buffer )
  , _buffer_owned( true )
  , _message_index( 0 )
  {
    _buffer->occupy_slot( message_index() );
  }

  ~message_component()
  {
    if ( _buffer_owned && _buffer )
	  {
		delete _buffer;
	  }
  }

  bool owns_buffer() const { return _buffer_owned; }

  void set_buffer( message_buffer & buffer, std::string name = "mrX" )
  {
    
	if ( _buffer == &buffer ) return;

    if ( _buffer )
    {
      if ( _buffer_owned )
		{
		  delete _buffer;
		}
      else
        _buffer->unoccupy_slot( _message_index );
    }
    
    _buffer = &buffer;
    _message_index = _buffer->free_slot();
    _buffer_owned = false;
    _buffer->occupy_slot( _message_index, name );
  }

  blitz::Array<unsigned int,2> & get_maxima( bool exclude_self = false )
  {
	_buffer->update_maxima( exclude_self ? message_index() : -1 );
	return buffer().get_maxima();
  }

  blitz::Array<T,3> & get_belief( bool exclude_self = false, int domain = belief_state::LINEAR )
  {
    _buffer->update( exclude_self ? message_index() : -1, domain == belief_state::LINEAR );
    switch ( domain )
    {
      case belief_state::LOG:  buffer().belief().to_log(); break;
      case belief_state::LINEAR:  buffer().belief().to_linear(true); break;
    }
    return buffer().belief().data;
  }

  void clear_msg()
  {
	get_msg().clear();
  } 

  void set_msg( const blitz::Array<T,3> & message, int domain = belief_state::LOG )
  {
    _buffer->set_msg( message_index(), message, domain );
  }
  
  belief_state & get_msg()
  {
    return _buffer->message( message_index() );
  } 
  
  mmm2_belief_message_buffer<T> & buffer()
  {
    return *_buffer;
  }

  size_t message_index() const { return _message_index; }

private:

  mmm2_belief_message_buffer<T> * _buffer;  
  bool _buffer_owned;
  size_t _message_index;

};


#endif /* end of include guard: MESSAGE_COMPONENT_HPP_P1CDJQ5L */
