/*

SHAPELEARNING

  THIS MODULE PROVIDES A FRAMEWORK FOR CONVOLUTIONAL BAYESIAN NETWORKS
  BY PROVIDING MODULES TO BUILD THE LAYERS, THE COMPOSITION NEEDS TO
  BE HANDLED BY THE USER

COPYRIGHT (C)

  FRAUNHOFER FIT-LIFE
  SCHLOSS BIRLINGHOVEN
  53754 ST. AUGUSTIN
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  Fraunhofer-Institut für Angewandte Informationstechnik FIT 
  http://fit.fraunhofer.de

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
#pragma once
#include "InverseRFIterator.hpp"
#include "ForwardRFIterator.hpp"
#include <boost/foreach.hpp>
#include <limits>
#include <iostream>
#include "stopwatch.hpp"

template<class C> struct TypeTraits
{	
  static const int num_statistics = 0;
};

template<class Derived> class MixtureModel2Base;

template<class Derived> struct has_upward_message_iterator : public boost::false_type {};

template<typename Derived, typename Enabled = void> 
struct update_gated_msg_delegate
{
  typedef typename TypeTraits<Derived>::num_type T;
  typedef blitz::Array<T,3> Matrix3;
  struct rf_sampling_context {};
  static void report( rf_sampling_context & ) {}
  static void init_interactions( MixtureModel2Base<Derived> &, Matrix3 &, Matrix3 &, InverseRFIterator & iter )	{};
  static void update_interactions( MixtureModel2Base<Derived> &, Matrix3 &, Matrix3&, InverseRFIterator & iter )	{};
  static void update_parent( MixtureModel2Base<Derived> &, Matrix3 &, Matrix3 &, Matrix3 &, ForwardRFIterator & ) {};
  static void build_upward_message_gated( MixtureModel2Base<Derived> &, Matrix3 &, Matrix3 &, T, InverseRFIterator & iter ) {};
  static void build_upward_message_with_saliency( MixtureModel2Base<Derived> &, Matrix3 &, Matrix3 &, InverseRFIterator & iter ) {};
  static void build_upward_message_with_saliency( MixtureModel2Base<Derived> &, IMatrix2 &, Matrix3 &, InverseRFIterator & iter ) {};
  static void build_downward_message( MixtureModel2Base<Derived> & model, Matrix3 & data, Matrix3 & r, Matrix3 & msg ) {};
  static void build_downward_message_sampled( MixtureModel2Base<Derived> & model, IMatrix2 & latents_sample, Matrix3 & msg ) {};
  static void sample_latents( MixtureModel2Base<Derived> & model, Matrix3 & data, IMatrix2 & latent_sample, Matrix3 lr0, ForwardRFIterator & iter ) {};
  static T foreground_probability( MixtureModel2Base<Derived> & model, Matrix1 & data, Matrix1 & r, int i, int j, int irf, int jrf ) 
  {
	return ( model._S.numElements() > 0 ) ? model._S( i, j ) : 1;
  };
  static T foreground_message( MixtureModel2Base<Derived> & model, Matrix1 & data, Matrix1 & r, int irf, int jrf ) {return 0;};
  static T background_message( MixtureModel2Base<Derived> & model, const Matrix1 & data, int i, int j ) {return 0;};
  static void sample_gates( MixtureModel2Base<Derived> & model, Matrix3 & data, IMatrix2 & latents_sample, InverseRFIterator & iter, bool cache_z = false, bool init = false ) {};
  static void sample_bits( MixtureModel2Base<Derived> & model, Matrix3 & data, IMatrix2 & latents_sample, ForwardRFIterator & iter ) {}
  static void sample_mask( MixtureModel2Base<Derived> & model, Matrix3 & data, IMatrix2 & latents_sample, InverseRFIterator & iter ) {};
  static void sample_data ( MixtureModel2Base<Derived> & model, Matrix3 & data, IMatrix2 & latents_sample, InverseRFIterator & iter ) {};
  static T evaluate_log_upward_at( Derived & derived, const typename TypeTraits<Derived>::num_type * x, int i1rf, int i2rf, int k ) { return 0;}
  static void sample_rf( MixtureModel2Base<Derived> & model, Matrix3 & data, IMatrix2 & latents_sample,  typename MixtureModel2Base<Derived>::position, rf_sampling_context & rf_samples ) {}
};

template<typename Derived>
struct update_gated_msg_delegate
< Derived, typename boost::enable_if< has_upward_message_iterator<Derived> >::type >
{
  typedef typename TypeTraits<Derived>::num_type T;
  typedef blitz::Array<T,3> Matrix3;
  typedef typename Derived::upward_message_iterator_t upward_message_iterator_t;
  typedef blitz::Array<bool,3> BMatrix3;
  typedef blitz::Array<bool,2> BMatrix2;
  typedef blitz::Array<bool,1> BMatrix1;
  typedef blitz::Array<unsigned int,2> IMatrix2;
  struct rf_sample
  {
	void resize( int d1, int d2 )
	{
	  gates.resize( d1, d2 );
	  parent_likelihoods.resize( d1, d2 );
	}	
	IMatrix2 gates;
	bool bit;
	int latent;
	Matrix2 parent_likelihoods;
  };
  typedef std::vector<rf_sample> rf_samples_t;
  struct rf_sampling_context
  {
	rf_sampling_context() : initialized(false) {}
	bool initialized;
	rf_samples_t samples;
	Matrix1 llrs;
	stopwatch init_timer, proposal_timer, choice_timer;
	void init( int k, int d1, int d2, bool have_bits )
	{
	  if ( initialized ) return;
	  size_t n_samples = have_bits ? ( k + 1 ) : ( k );
	  samples.resize( n_samples );
	  BOOST_FOREACH( rf_sample & sample, samples )
		sample.resize( d1, d2 );
	  llrs.resize( n_samples );
	  llrs = 0;
	  initialized = true;
	}
	~rf_sampling_context()
	{
	}
  };
  static void report( rf_sampling_context & ctx )
  {
	std::cout << " init times " << ctx.init_timer << std::endl;
	std::cout << " proposal times " << ctx.proposal_timer << std::endl;
	std::cout << " choice times " << ctx.choice_timer << std::endl;
  }
  static void init_interactions( MixtureModel2Base<Derived> &, Matrix3 &, Matrix3 &, InverseRFIterator & iter );
  static void update_interactions( MixtureModel2Base<Derived> &, Matrix3 &, Matrix3&, InverseRFIterator & iter );	
  static void update_parent( MixtureModel2Base<Derived> &, Matrix3 &, Matrix3 &, Matrix3 &, ForwardRFIterator & ) ;
  static void build_upward_message_gated( MixtureModel2Base<Derived> &, Matrix3 &, Matrix3 &, T, InverseRFIterator & iter ) ;
  static void build_upward_message_with_saliency( MixtureModel2Base<Derived> &, Matrix3 &, Matrix3 &, InverseRFIterator & iter );
  static void build_upward_message_with_saliency( MixtureModel2Base<Derived> &, IMatrix2 &, Matrix3 &, InverseRFIterator & iter ) {};
  static void build_downward_message( MixtureModel2Base<Derived> & model, Matrix3 & data, Matrix3 & r, Matrix3 & msg ) ;
  static void build_downward_message_sampled( MixtureModel2Base<Derived> & model, IMatrix2 & latents_sample, Matrix3 & msg ) ;
  static void sample_latents( MixtureModel2Base<Derived> & model, Matrix3 & data, IMatrix2 & latent_sample, Matrix3 lr0, ForwardRFIterator & iter ) ;
  static typename TypeTraits<Derived>::num_type foreground_probability( MixtureModel2Base<Derived> & model, Matrix1 & data, Matrix1 & r, int i, int j, int irf, int jrf );
  static typename TypeTraits<Derived>::num_type foreground_message( MixtureModel2Base<Derived> & model, Matrix1 & data, Matrix1 & r, int irf, int jrf );
  static typename TypeTraits<Derived>::num_type eval_likelihood( upward_message_iterator_t & iter )
  {
	typedef typename TypeTraits<Derived>::num_type T;
	T p = 0;
	T p0 = T(1) / iter.K;
	for ( ; iter.k < iter.K; iter.advance(1) )
	  p += p0 * iter.evaluate_linear();
	return p;
  }
  static typename TypeTraits<Derived>::num_type eval_likelihood( upward_message_iterator_t & iter, const Matrix1 & p0 )
  {
	typedef typename TypeTraits<Derived>::num_type T;
	T p = 0;
	for ( ; iter.k < iter.K; iter.advance(1) )
	  p += p0(iter.k) * iter.evaluate_linear();
	return p;	
  }
  static typename TypeTraits<Derived>::num_type background_message( MixtureModel2Base<Derived> & model, const Matrix1 & data, int i, int j );
  static void sample_gates( MixtureModel2Base<Derived> & model, Matrix3 & data, IMatrix2 & latents_sample, InverseRFIterator & iter, bool cache_z = false, bool init = false ) ;
  static void sample_bits( MixtureModel2Base<Derived> & model, Matrix3 & data, IMatrix2 & latents_sample, ForwardRFIterator & iter );
  static void sample_mask( MixtureModel2Base<Derived> & model, Matrix3 & data, IMatrix2 & latents_sample, InverseRFIterator & iter ) ;
  static void sample_data ( MixtureModel2Base<Derived> & model, Matrix3 & data, IMatrix2 & latents_sample, InverseRFIterator & iter ) ;
  static typename TypeTraits<Derived>::num_type evaluate_log_upward_at( Derived & derived, const typename TypeTraits<Derived>::num_type * x, int i1rf, int i2rf, int k ) ;
  static void sample_rf( MixtureModel2Base<Derived> & model, Matrix3 & data, 
						 IMatrix2 & latents_sample, 
						 typename MixtureModel2Base<Derived>::position, 
						 rf_sampling_context & context );
};

