/*

SHAPELEARNING

  THIS MODULE PROVIDES A FRAMEWORK FOR CONVOLUTIONAL BAYESIAN NETWORKS
  BY PROVIDING MODULES TO BUILD THE LAYERS, THE COMPOSITION NEEDS TO
  BE HANDLED BY THE USER

COPYRIGHT (C)

  FRAUNHOFER FIT-LIFE
  SCHLOSS BIRLINGHOVEN
  53754 ST. AUGUSTIN
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  Fraunhofer-Institut für Angewandte Informationstechnik FIT 
  http://fit.fraunhofer.de

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
#ifndef _SEGMENTATION_TOOLS_HPP_
#define _SEGMENTATION_TOOLS_HPP_

#include "mmm_tool.hpp"
#include <map>
#include <vector>

#define SEGMENTATION_RANGES(prefix,d)									\
  const blitz::Range prefix ## lo(0,d/2-1), prefix ## hi(d/2,d-1);		\
  const blitz::Range prefix ## 1 [] = { prefix ## lo, prefix ## lo,		\
										prefix ## hi, prefix ## hi };	\
  const blitz::Range prefix ## 2 [] = { prefix ## lo, prefix ## hi,		\
										prefix ## hi, prefix ## lo };

template<typename T>
void resizeweights ( 
  const blitz::Array<T,2> & weights, 
  blitz::Array<T,2> & result, int w, int h );

template<typename T>
blitz::Array<T,3> resize_multinomial_array ( 
  const blitz::Array<T,3> & in, int w, int h, 
  const blitz::Array<T,2> * weights = 0);

template<class T>
void generate_label_segmentation (
  const blitz::Array<T,3> & labels,
  int step, int D, 
  blitz::Array<T,3> & s,
  const blitz::Array<T,2> & weights,
  T lmaxlo = 0.5, T lmaxhi = 0.8, T suppression_factor = 0.01);

template<class T>
void segment_propagate ( 
  blitz::Array<T,3> const& in, blitz::Array<T,3> const& s,
  blitz::Array<T,3> & out, int step, int D );

template<class T>
void segment_propagate ( 
  blitz::Array<T,2> const& in, blitz::Array<T,3> const& s,
  blitz::Array<T,2> & out, int step, int D );

template<typename T>
blitz::Array<T,3> apply_labelmap (
  const blitz::Array<T,3> & out, const std::map<int,int> & labelmap, 
  const std::vector<int> & outindices, int K,
  T invalid_threshold = 0.3 );

template<typename T>
blitz::Array<T,3> 
expand_indexed ( 
  const blitz::Array<T,3> & out, 
  const std::vector<int> & outindices, int K );

template<typename T> 
void convertlabels ( 
  const blitz::Array<unsigned int,2>  & labels, 
  int w, int h,
  blitz::Array<T,3> & outdata, 
	std::vector<int> & outindices,
	blitz::Array<T,2> & weights,
	T bgweight,
	std::map<int,int> &
										);



#endif /* _SEGMENTATION_TOOLS_HPP_ */

