/*

SHAPELEARNING

  THIS MODULE PROVIDES A FRAMEWORK FOR CONVOLUTIONAL BAYESIAN NETWORKS
  BY PROVIDING MODULES TO BUILD THE LAYERS, THE COMPOSITION NEEDS TO
  BE HANDLED BY THE USER

COPYRIGHT (C)

  FRAUNHOFER FIT-LIFE
  SCHLOSS BIRLINGHOVEN
  53754 ST. AUGUSTIN
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  Fraunhofer-Institut für Angewandte Informationstechnik FIT 
  http://fit.fraunhofer.de

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
#ifndef _SHMM_HPP_
#define _SHMM_HPP_

#include "tfmm.hpp"
#include <boost/foreach.hpp>

/// this model is used for recognizing several kinds of sequences
/// each kind of sequence is modeled by a separate markov chain (the sub-chains)
/// an additional markov chain (the switch-chain) selects the active chain
/// this model is thus a kind of mixture of markov chains, where the latent variable
/// is again embedded in a markov chain
/// TODO: put implementation into cpp file, implement serialization
template<typename T>
class SwitchingMarkovChainModel
: public HasProperties< SearisePropertyTypes, SwitchingMarkovChainModel<T> >
{

public:  

  typedef SwitchingMarkovChainModel<T> This;
  typedef HasProperties< SearisePropertyTypes, This > PBase;

  SwitchingMarkovChainModel( int m, int k ) 
    : _k(k), _m(m), _new_data(false) {}  

  static void init_properties()
  {
    // FIXME: implement STL-container property handling by broadcasting sets and performing get on element 0 or throwing if empty (or return empty property?)
    // FIXME: implement pre/post set/get to enable updating before getting/setting sub-chain properties
  }

  Matrix3 s()
  {
    return _switch_chain.buffer.update();
  }

  void set_data( const Matrix3 & data )
  {    
    _data.reference(data);
    _new_data = true;
  }
  
  Matrix3 & data()
  {
    return _data;
  }
  
  void do_mp( bool variational = false )
  {
    // FIXME: implement like this:
    // 1. get s-prediction
    // 2. iterate: 
    //      a) update sub-chains
    //      b) update switch-chain
    // update sub-chains
  }

  void batchlearn_begin_epoch()
  {
    _switch_chain.batchlearn_begin_epoch();
    BOOST_FOREACH( Chain & chain, _sub_chains )
      chain.batchlearn_begin_epoch();
  }

  void end_mp_frame( bool learn = false )
  {
    // FIXME: if learning: collect statistics
    _switch_chain.end_mp_frame();
    BOOST_FOREACH( Chain & chain, _sub_chains )
      chain.end_mp_frame();
  }

  void batchlearn_end_epoch()
  {
    _switch_chain.batchlearn_begin_epoch();
    BOOST_FOREACH( Chain & chain, _sub_chains )
      chain.batchlearn_begin_epoch();
  }

  

private:

  update()
  {
    if ( _k != _sub_chains.size() )
      {
        _sub_chains.resize( _k );
      }
	  if ( _k != 0 && _m != cast<int>( _sub_chains.back().get_property("k") ) )
      {
        BOOST_FOREACH( Chain & chain, _sub_chains )
          {
            chain.set_property( "k", _m );
          }        
      }    
  }

  struct Chain 
  {
    TemporalForwardMarkovModel<T> tfmm;
    mmm2_belief_message_buffer<T> buffer;
  };
  std::vector< Chain > _sub_chains;
  Chain _switch_chain;
  Matrix3 _data;
  bool _new_data;
  unsigned int _m; // m is the cardinality of the input state space, and thus the state space of the sub-chains
  unsigned int _k; // k is the cardinality of the output state space, and thus the state space of the switch chain
  unsigned int _iterations;
};

#endif
