/*
Copyright INRIA 2011

Authors: Neil D.B. Bruce <Neil.Bruce@inria.fr>
Contact: Pierre Kornprobst <Pierre.Kornprobst@inria.fr>

This software is a computer program whose purpose is to estimate saliency.

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL-C
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.
*/

#include "CPUData.hpp"
#include "GPUImage.h"
#include <blitz/array.h>


class Fusion {
  
 private:
  
  int w,h,f,type,rangeDefined,scalingType;
  blitz::Array<float,3>  currentData;
  blitz::Array<float,3>  sortedData;
  blitz::Array<float,2>  currentSlice;
  blitz::Array<float,2>  combinedFusion;
  blitz::Array<float,1>  minVals;
  blitz::Array<float,1>  maxVals;
  blitz::Array<float,1>  weights;

 public: 

  Fusion();
  Fusion( int w, int h, int feats, int type, int scale_in);
  void setWeights(blitz::Array<float,1> newWeights);
  void setScaleType(int _in_type);
  void  init( int w_in, int h_in, int feats_in, int type_in, int scale_in);
  ~Fusion();

  void      readWeights(const char* filename);
  void      scaleCurrentData();
  CPUData<float> Fuse(CPUData<float> newData);
};


