/*
Copyright INRIA 2011

Authors: Neil D.B. Bruce <Neil.Bruce@inria.fr>
Contact: Pierre Kornprobst <Pierre.Kornprobst@inria.fr>

This software is a computer program whose purpose is to estimate saliency.

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL-C
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.
*/

#include "CPUData.hpp"
#include "GPUImage.h"
#include <blitz/array.h>


class Surprise {
  
 private:
  
  int w,h,f,bins,type,rangeDefined, priorDefined, scalingType, sw;
  float scaleFactor, oneMinusScale, decayFactor, oneMinusDecay, limitscale;
  blitz::Array<float,4>  probDensities; 
  blitz::Array<float,4>  normedDensities; 
  blitz::Array<float,4>  priorDensities; 
  blitz::Array<float,3>  currentData;
  blitz::Array<float,3>  catData;
  blitz::Array<float,3>  normCatData;
  blitz::Array<float,2>  catMask;
  blitz::Array<float,2>  catCounter;
  blitz::Array<int,3>    indices;
  blitz::Array<float,3>  marginalSurprise;	
  blitz::Array<float,2>  combinedSurprise;
  blitz::Array<float,2>  simpleDensities;
  blitz::Array<float,1>  minVals;
  blitz::Array<float,1>  maxVals;
  blitz::Array<float,2>  weightMatrix;
  
  void updateDensitiesCommon();

 public: 

  Surprise();
  Surprise  ( int w, int h, int feats, int bins, int type, 
			  float scale_in = 0.0, float decay_in = 0.0, int sw_in = 0, float limitscale_in = 1.0,
			  CPUData<float> weightMatrix_in = CPUData<float>());
  void  init( int w_in, int h_in, int feats_in, int bins_in, int type_in, 
			  float scale_in = 0.0, float decay_in = 0.0, int sw_in = 0, float limitscale_in = 1.0,
			  CPUData<float> weightMatrix_in = CPUData<float>());
  ~Surprise();


  void      scaleCurrentData();
  void      setCurrentData(CPUData<float> newData);
  void      setMask(CPUData<float> newData);
  void  	computeCombinedSurprise();
  void  	computeMarginalSurprise();
  void  	computeMarginalSurpriseSimple();
  void		updateDensitiesAccumulate();
  void		updateDensitiesSimple();
  void		updateDensitiesWeighted();
  void		updateDensitiesDecay();
  void		updateDensitiesBuffered();
  void		updateDensitiesCat();
  void          computeCatSurprise();
  void      updateNormalizedDensityApprox();
  CPUData<float>  returnCombinedSurprise();
  CPUData<float>  returnMarginalSurprise();
  blitz::Array<float,4>& returnNormedDensities();
  void            writeRange(const char* fileName);
  void            readRange(const char* fileName);
  void            setScaleType(int scaleType){scalingType = scaleType;};
  int             getScaleType(){return scalingType;};
  void            writePriorDensities(const char* fileName);
  void            readPriorDensities(const char* fileName);
  blitz::Array<int,3> returnIndex();
  CPUData<float>  discriminantSaliency(blitz::Array<float,4> otherSurprise);
};


