# daniel: optionally include local make config
-include ./local.mk
-include ../local.mk

include ../$(SUBDIR)flags.mk

# The build directory
BLD=build
# The NVCC compiler
NVCC=$(CUDA_INSTALL_PATH)/bin/nvcc

# Outputted libraries (shared & static)
# TODO: The shared library isn't usable for me
LIB=lib$(LIBNAME).a
SHLIB=lib$(LIBNAME).so

CHECK_LIBS+=-lcppunit 
NVCCFLAGS+=-Isrc
CHECK_SRC=$(wildcard tests/*.cpp)
CHECK_OUTPUT=./tests/checks
LIBS += -lcudart

# Outputs to be generated
CU_O=$(patsubst src-cu/%.cu, $(BLD)/%.cu.o, $(wildcard src-cu/*.cu))
ifndef CPP_SRC
	CPP_SRC=$(wildcard src/*.cpp)
endif
CPP_O=$(patsubst src/%.cpp, $(BLD)/%.cpp.o, $(CPP_SRC))

CPP_DEPEND = $(patsubst $(BLD)/%.cpp.o, $(BLD)/%.cpp.d, $(CPP_O))


# These are not really file/dirs targets
.PHONY: all clean check
# Do not remove intermediate files
.PRECIOUS: all $(SHLIB) $(LIB) $(CU_O) $(CU_CPP) $(CPP_O)

# By default build the libraries
all: $(BLD) $(LIB)

info:
	echo lib $(LIB)
	echo objects $(CPP_O)

$(BLD):
	mkdir $(BLD)

$(LIB): $(CPP_O) $(CU_O)
	$(AR) rcs $@ $^

$(SHLIB):
	echo "Not yet implemented"

$(BLD)/%.cpp.o: src/%.cpp
	$(CXX) $(CXXFLAGS) $(CPPFLAGS) -c -o $@ $<

$(BLD)/%.cu.o: src/%.cu.c
	$(CXX) $(CXXFLAGS) $(CPPFLAGS) -c -o $@ $<

$(BLD)/%.cu.o: src-cu/%.cu
	$(NVCC) $(NVCCFLAGS) $(shell echo $(CPPFLAGS) | sed 's/-msse[^ ]*//g') -c -o $@ $<

check: $(CHECK_OUTPUT)
	$<

tests/checks: $(BLD) $(CHECK_SRC) $(LIB)
	$(CXX) $(CXXFLAGS) $(CPPFLAGS) -Wall -Wextra -Isrc -o $@ $(CHECK_SRC) $(LIB) $(LIBS) $(LDFLAGS) $(CHECK_LIBS)

clean:
	rm -fr $(CPP_O) $(CU_CPP) $(CU_O) $(CPP_DEPEND) $(LIB) $(SHLIB)

# dependency tracking
MAKEDEPEND = gcc -M -MT $(subst .d,.o,$@) $(CXXFLAGS) $(CPPFLAGS) -o $@  $<
$(BLD)/%.cpp.d : src/%.cpp
	$(MAKEDEPEND)

ifneq ($(MAKECMDGOALS),clean)
-include $(CPP_DEPEND)
endif
