/*

COMMON

  THIS MODULE INCLUDES GENERAL SUPPORT FILES THAT DID NOT FIND THEIR OWN PLACE
  THIS INCLUDES HELPERS FOR DEALING WITH BLITZ ARRAS, CONCURRENCY, PROFILING
  GSL, UBLAS, SERIALIZATION, AND A SIMPLE YET EFFECTIVE MIXIN CLASS FOR PROVIDING
  ARBITRARY C++ CLASSES WITH ATTRIBUTES THAT CAN BE QUERIED IN TEXT FORM TO 
  EASE SCRIPTING

COPYRIGHT (C)

  FRAUNHOFER FIT-LIFE
  SCHLOSS BIRLINGHOVEN
  53754 ST. AUGUSTIN
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  Fraunhofer-Institut für Angewandte Informationstechnik FIT 
  http://fit.fraunhofer.de

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
#ifndef _PROPERTIES_HPP_
#define _PROPERTIES_HPP_

#include <typeinfo>
#include <iostream>
#include <sstream>
#include <map>
#include <string>
#include <list>
#include <stdexcept>

#include <boost/function.hpp>
#include <boost/format.hpp>
#include <boost/foreach.hpp>
#include <boost/tuple/tuple.hpp>
#include <boost/mpl/vector.hpp>

#include <boost/variant.hpp>

#include "variant_casts.hpp"

#define D_PROPS(X)

//TODO: 
//	signals and/or callbacks
//	hierarchies (loops?) -> recursive variant, including a hasproperties base over the property itself
//  operator[]
//  property iteration
//	serialization
//	helpful adaptors for setters/getters
//  tags, i.e.: dependant property (not to be serialized), others?

//minimal interface for introspective properties
//next: introspective method exports for brainless wrapping :)
template<class Types>
struct HasPropertiesBase
{
  typedef HasPropertiesBase< Types > This;
  typedef typename boost::mpl::push_front< Types, This * >::type ExtendedTypes;
  typedef typename boost::make_variant_over< ExtendedTypes >::type PropertyType;
  typedef boost::function2< void,This*,PropertyType > Setter;
  typedef boost::function1< PropertyType, const This* > Getter;
  typedef boost::function1< void, This* > SideEffect;

  struct PropertyDescriptor
  {
    PropertyDescriptor(Getter g = 0, Setter s = 0) : getter(g), setter(s) 
	{
	  D_PROPS(std::cout << "creating property descriptor" << std::endl;);
	}
	~PropertyDescriptor()
	{
	
	  D_PROPS(std::cout << "destroying property descriptor" << std::endl;);
	}
    Getter getter;
    Setter setter;
    std::string description;
  };

  typedef std::map< std::string, PropertyDescriptor > PropertyVTable;

  struct PropertiesInfo
  {
    PropertiesInfo() 
	{
	  D_PROPS(std::cout << "creating property info" << std::endl;);
	}
	~PropertiesInfo()
	{
	  D_PROPS(std::cout << "destroying properties info" << type << std::endl;);
	}
    PropertyVTable pdescs;
    std::string type;
  };

  typedef PropertiesInfo * PropertiesInfoHandle;

  template<class ostream>
  void dump(ostream & os)
  {
#if 0
    BOOST_FOREACH ( PropertiesInfoHandle pinfo, _pinfos )
    {
      os << "  Properties for type " << pinfo->type << ":\n";
      for ( typename PropertyVTable::const_iterator entry = pinfo->pdescs.begin(), eentry = pinfo->pdescs.end(); entry != eentry; ++entry )
      {
        const std::string & name = entry->first;
        const PropertyDescriptor & desc = entry->second;
        os << "    " << name << ": " << (desc.setter?"settable ":"") << (desc.getter?"gettable ":"");
        if ( desc.getter )
        {
          PropertyType value = desc.getter( this );
          std::string str = cast<std::string>( value );
          os << " value: " << str << " (" << value << ")" << std::endl;
        }
      }
    }
#endif
  }

  typedef boost::tuple< const PropertyDescriptor *, const PropertiesInfoHandle, const This *> ConstPInfoResult;;
  typedef boost::tuple< const PropertyDescriptor *, const PropertiesInfoHandle, This *> PInfoResult;;

  ConstPInfoResult
  find_property_descriptor( const std::string & key ) const
  {
	return const_cast<This*>(this)->find_property_descriptor( key );
  }

  PInfoResult
  find_property_descriptor( const std::string & key )
  {

    typedef typename PropertyVTable::const_iterator DescIter;
    This * instance = this;

    //resolve relative adressing
    size_t dotpos = 0, lastdotpos = -1;
    while ( ( dotpos = key.find_first_of( '.', lastdotpos + 1 ) ) != std::string::npos )
    {
      const std::string & iname = key.substr( lastdotpos + 1, dotpos - lastdotpos - 1 );

	  //	  try
		{
		  instance = boost::get<This*>( instance->get_property( iname ) );
		  D_PROPS(std::cout << "recursing to hasproperties instance " << iname << std::endl;);
		} 
		//	  catch(...)
		//		{
		//		  instance = 0;
		//		}
      if ( instance == 0 )
        throw std::runtime_error( boost::str( boost::format("could not resolve hasproperties instance %s ") % key.substr(0, dotpos) ) );
	  lastdotpos = dotpos;
    }

	const std::string & subkey = ( lastdotpos != 0 ) ? key.substr( lastdotpos + 1 ) : key;

	/// fixme: search the list backwards to allow property overrides
    BOOST_REVERSE_FOREACH ( const PropertiesInfoHandle & pinfo, instance->_pinfos )
    {
      DescIter desc = pinfo->pdescs.find(subkey);
      if ( desc != pinfo->pdescs.end() )
        return PInfoResult( &desc->second, pinfo, instance );
    }

    return PInfoResult(0,0,0);
  }

  void set_property(const std::string & key, PropertyType value)
  {
	D_PROPS(std::cout << "setting property " << key << std::endl;);
    const PropertyDescriptor * desc;
    const PropertiesInfo * pinfo;
    This * instance;
    boost::tie(desc,pinfo,instance) = find_property_descriptor( key );
    if ( desc )
    {
      if (desc->setter)
        desc->setter( instance, value );
      else
        throw(std::runtime_error(boost::str(boost::format("%s: no setter for property: %s")%name()%key)));
    }
    else
      throw(std::runtime_error(boost::str(boost::format("%s: invalid property: %s")%name()%key)));
  }

  PropertyType get_property(const std::string & key) const
  {
	D_PROPS(std::cout << "getting property " << key << std::endl;);
    const PropertyDescriptor * desc;
    const PropertiesInfo * pinfo;
    const This * instance;
    boost::tie(desc,pinfo,instance) = find_property_descriptor( key );
    if ( desc )
    {
      if ( desc->getter )
        return desc->getter( instance );
      else
        throw(std::runtime_error(boost::str(boost::format("%s: no getter for property: %s" ) % name() % key ) ) );
    }
    else
      throw( std::runtime_error( boost::str( boost::format( "%s: invalid property: %s" ) % name() %key ) ) );
  }

  virtual ~HasPropertiesBase() 
  {
	D_PROPS(std::cout << "destroying base " << std::endl;);
  }

  bool has_property(const std::string & key)
  {
    const PropertyDescriptor * desc;
    const PropertiesInfo * pinfo;
    const This * instance;
    boost::tie(desc,pinfo,instance) = find_property_descriptor( key );
	return desc != 0;
  }

  template<class Archive>
  void serialize( Archive & ar, const unsigned int version )
  {
	throw std::runtime_error("HasPropertiesBase is not serializable");
  }

protected:

  const std::string & name() const
  {
    return _pinfos.back()->type;
  }

  // FIXME: using shared_ptr results in some kind of threadless deadlock
  void add_pinfo( const PropertiesInfoHandle & pinfo )
  {
	D_PROPS(std::cout << "adding pinfo for type " << pinfo->type << std::endl;);
	_pinfos.push_back( pinfo );
  }

  std::list< PropertiesInfoHandle > _pinfos;
};


//CRTP-class for properties registration

template< class Types, class Derived > class HasProperties 
: public virtual HasPropertiesBase< Types >
{
public:

  typedef HasPropertiesBase< Types > Base;
  typedef typename Base::PropertyType PropertyType;
  typedef typename Base::PropertyDescriptor PropertyDescriptor;
  typedef typename Base::Setter Setter;
  typedef typename Base::Getter Getter;
  typedef typename Base::SideEffect SideEffect;
  typedef typename Base::PropertiesInfo PropertiesInfo;

  HasProperties()
  {
    checked_initialize_properties();
    Base::add_pinfo( _pinfop );
  }

  virtual ~HasProperties()
  {
	D_PROPS( std::cout << " destroying HasProperties for " << typeid(Derived).name() << std::endl; );
  }

//protected:

  template<class C, typename numT>
  struct IVarValueSetter
  {
    IVarValueSetter(numT C::* _ivar, numT _value)
      : ivar(_ivar), value(_value) {}
    numT C::* ivar;
    numT value;
    void operator()( Base * b )
    {
	  C * c = dynamic_cast<C*>(b);
	  if ( c == 0 )
		throw std::runtime_error("IVarValueSetter: could not down cast");
      c->* ivar = value;
    }
  };  

  template<class C, typename numT>
  static IVarValueSetter<C,numT> make_ivar_setter(numT C::* ivar, numT value)
  {
    return IVarValueSetter<C,numT>(ivar,value);
  }

  struct FunctionalSetter
  {
    typedef boost::function2<void,Derived*,PropertyType> F;
	F _f;
	FunctionalSetter( F f )  : _f(f) {}
    void operator()( Base * b, PropertyType v)
    {
	  Derived * c = dynamic_cast<Derived*>(b);
	  if ( c == 0 )
		throw std::runtime_error("FunctionalSetter: could not down cast");
	  _f( c, v );
	}	
  };

  struct FunctionalGetter
  {
    typedef boost::function1<PropertyType,const Derived*> F;
	F _f;
	FunctionalGetter( F f )  : _f(f) {}
    PropertyType operator()( const Base * b )
    {
	  const Derived * c = dynamic_cast<const Derived*>(b);
	  if ( c == 0 )
		throw std::runtime_error("FunctionalGetter: could not down cast");
	  return _f( c );
	}	
  };

  template<class C, typename numT>
  struct IVarPropertySetter
  {
    IVarPropertySetter(numT C::* _ivar, boost::function1<void,Base*> _sideffect = 0)
      : ivar(_ivar),sideffect(_sideffect) {}
    numT C::* ivar;
    boost::function1<void,Base*> sideffect;
    void operator()( Base * b, PropertyType v)
    {
	  C * c = dynamic_cast<C*>(b);
	  if ( c == 0 )
		throw std::runtime_error("IVarPropertySetter: could not down cast");
	  c->* ivar = cast<numT>(v);
	  if (sideffect)
		sideffect(c);
    }
  };


  template<class C, typename numT>
  static IVarPropertySetter<C,numT> make_ivar_setter (numT C::* ivar, boost::function1<void,Base*> sideeffect = 0 )
  {
    return IVarPropertySetter<C,numT>(ivar, sideeffect);
  }

  template< class SubSetter, typename numT >
  struct ConvertingSetterWrapper
  {
	typedef boost::function1<numT,numT> converter_t;
	ConvertingSetterWrapper( const SubSetter & setter, const converter_t & converter )
	  : _base_setter( setter ), _converter( converter )
	{
	}
	SubSetter _base_setter;
	converter_t _converter;
	void operator()( Base * b, PropertyType v)
	{
	  numT input_value = cast<numT>( v );
	  numT output_value = _converter( input_value );
	  _base_setter( b, output_value );
	}
  };

  template< class SubSetter, typename numT > 
  static ConvertingSetterWrapper< SubSetter, numT >
  make_converting_setter( const SubSetter & setter, const boost::function1<numT,numT> & converter )
  {
    return ConvertingSetterWrapper< SubSetter, numT >( setter, converter );
  }
  
  template<class C, typename numT>
  static ConvertingSetterWrapper< IVarPropertySetter<C,numT>, numT > 
  make_converting_ivar_setter ( numT C::* ivar, const boost::function1<numT,numT> & converter, 
                                boost::function1<void,Base*> sideeffect = 0 )
  {
    return make_converting_setter( make_ivar_setter( ivar, sideeffect ), converter );
  }

  template<class C, class M>
  struct MemberPropertyGetter
  {
    const std::string key;
    M C::* member;

    MemberPropertyGetter(const std::string & _key, M C::* _member) 
      : key(_key), member(_member) {}

    PropertyType operator()(const Base *c)
    {			
      return (const_cast<C*>(dynamic_cast<const C*>(c))->* member).get_property(key);
    }
  };
  template<class C, class M>
  static MemberPropertyGetter<C,M>  make_mprop_getter(const std::string & key, M C::* member)
  {
	return MemberPropertyGetter<C,M>( key, member );
  }

  template<class C, class M>
  struct MemberPropertySetter
  {
    const std::string key;
    M C::* member;

    MemberPropertySetter(const std::string & _key, M C::* _member) 
      : key(_key), member(_member) {}

    void operator()( Base * b, PropertyType v)
    {			
	  C * c = dynamic_cast<C*>(b);
	  if ( c == 0 )
		throw std::runtime_error("MemberPropertySetter::operator() could not down cast");
      ( c->* member ).set_property( key, v );
    }
  };
  
  struct MemFunSetter
  {
	typedef void ( Derived::* MemFunP )( PropertyType );

    MemFunP memfun;
	SideEffect sideeffect;

    MemFunSetter( MemFunP _memfun, SideEffect _sideeffect = 0 ) 
      : memfun(_memfun), sideeffect(_sideeffect) {}

    void operator()( Base * b, PropertyType v)
    {
	  Derived * c = dynamic_cast<Derived*>(b);
	  if ( c == 0 )
		throw std::runtime_error("MemFunSetter::operator(): could not down cast");
      ( c->* memfun )( v );
	  if ( sideeffect )
		sideeffect(c);
    }
  };
  
  struct MemFunGetter
  {

	typedef PropertyType ( Derived::* MemFunP )( );

    MemFunP memfun;

    MemFunGetter( MemFunP _memfun ) 
      : memfun(_memfun) 
	{
	}
	
    PropertyType operator()( const Base * b )
    {
	  const Derived * c = dynamic_cast<const Derived*>(b);
	  if ( c == 0 )
		throw std::runtime_error("MemFunGetter::operator(): could not down cast");
      return ( const_cast<Derived*>(c)->* memfun )( );
    }

  };
  
  template<class C, typename numT>
  struct IVarGetter
  {
    IVarGetter(numT C::* _ivar)
      : ivar(_ivar) {}
    numT C::* ivar;
    PropertyType operator()( const Base *b )
    {
	  const C * c = dynamic_cast<const C*>(b);
	  if ( c == 0 )
		throw std::runtime_error("IVarGetter: could not down cast");
      return c->* ivar;
    }
  };

  template<class C, typename numT>
  static IVarGetter<C,numT> make_ivar_getter(numT C::* ivar)
  {
    return IVarGetter<C,numT>(ivar);
  }

  template<typename numT, typename SubGetter>
  struct ConvertingGetterWrapper
  {
    typedef boost::function1<numT,numT> converter_t;
    ConvertingGetterWrapper( const SubGetter & getter, const converter_t & converter )
    : _wrapped_getter( getter ), _converter( converter )
    {
    }
    Getter _wrapped_getter;
    converter_t _converter;
    PropertyType operator()( const Base *b )
    {
      PropertyType base_value = _wrapped_getter(b);
      return _converter( cast<numT>( base_value ) );
    }
  };

  template<typename numT, class SubGetter>
  static ConvertingGetterWrapper< numT, SubGetter > 
  make_converting_getter( const SubGetter & getter, const boost::function1<numT,numT> & converter )
  {
    return ConvertingGetterWrapper< numT, SubGetter >( getter, converter );
  }  

  template<class C, typename numT>
  static ConvertingGetterWrapper< numT, IVarGetter< C, numT >  > 
  make_converting_ivar_getter(numT C::* ivar, const boost::function1<numT,numT> & converter )
  {
    return make_converting_getter< numT, IVarGetter<C,numT> > ( IVarGetter<C,numT>(ivar), converter );
  }

  struct ConstGetter
  {
    ConstGetter( const PropertyType & v ) : value(v) {}
    PropertyType value;
    PropertyType operator()( const Base *b )
    {
      return value;
    }
  };

  template<class C, typename T>
  struct PIVarGetter
  {
    PIVarGetter(T C::* _ivar)
      : ivar(_ivar) {}
    T C::* ivar;
    PropertyType operator()( const Base *b )
    {
	  const C * c = dynamic_cast<const C*>(b);
	  if ( c == 0 )
		throw std::runtime_error("PIVarGetter: could not down cast");
      T * v = const_cast<T*>( &(c->* ivar) );
	  return PropertyType(v);
    }
  };
  template<class C, typename numT>
  static PIVarGetter<C,numT> make_pivar_getter(numT C::* ivar)
  {
    return PIVarGetter<C,numT>(ivar);
  }

  template<class C, typename T>
  struct VIVarGetter
  {
    VIVarGetter(std::vector<T> C::* _ivar, size_t _index)
      : ivar(_ivar), index(_index) {}
	std::vector<T> C::* ivar;
	size_t index;
    PropertyType operator()( const Base *b )
    {
	  const C * c = dynamic_cast<const C*>(b);
	  if ( c == 0 )
		throw std::runtime_error("VIVarGetter: could not down cast");
      T v = (c->* ivar).at(index);
	  return PropertyType(v);
    }
  };

  template<class C, typename T>
  struct VIVarSetter
  {
    VIVarSetter(std::vector<T> C::* _ivar, size_t _index, boost::function1<void,Base*> _sideffect = 0)
      : ivar(_ivar), index(_index), sideffect(_sideffect) {}
    std::vector<T> C::* ivar;
	size_t index;
    boost::function1<void,Base*> sideffect;
    void operator()( Base * b, PropertyType v)
    {
	  C * c = dynamic_cast<C*>(b);
	  if ( c == 0 )
		throw std::runtime_error("VIVarSetter: could not down cast");
	  (c->* ivar).at(index) = cast<T>(v);
	  if (sideffect)
		sideffect(c);
    }
  };

  template<typename Tnum>
  static VIVarGetter<Derived,Tnum> make_vivar_getter(std::vector<Tnum> Derived::* ivar, size_t index)
  {
    return VIVarGetter<Derived,Tnum>(ivar, index);
  }

  template<typename Tnum>
  static VIVarSetter<Derived,Tnum> make_vivar_setter( std::vector<Tnum> Derived::* ivar, size_t index, boost::function1<void,Base*> sideeffect = 0 )
  {
    return VIVarSetter<Derived,Tnum>(ivar, index, sideeffect );
  }

  template<class C, typename T>
  struct VPIVarGetter
  {
    VPIVarGetter(std::vector<T> C::* _ivar, size_t _index)
      : ivar(_ivar), index(_index) {}
	std::vector<T> C::* ivar;
	size_t index;
    PropertyType operator()( const Base *b )
    {
	  const C * c = dynamic_cast<const C*>(b);
	  if ( c == 0 )
		throw std::runtime_error("VPIVarGetter: could not down cast");
      T * v = const_cast<T*>( &((c->* ivar).at(index)) );
	  return PropertyType(v);
    }
  };

  template<class C, typename T>
  static VPIVarGetter<C,T> make_vpivar_getter(std::vector<T> C::* ivar, size_t index)
  {
    return VPIVarGetter<C,T>(ivar, index);
  }

  static void add_property(const std::string & name, Getter g = 0, Setter s = 0)
  {
	  D_PROPS(std::cout << " adding property " << name << std::endl; );
	  if ( _pinfop->pdescs.count(name) )
	    throw std::runtime_error( boost::str( boost::format("redefining property %s for class %s") % name % typeid(Derived).name() ) );
    _pinfop->pdescs[name] = PropertyDescriptor(g,s);
  }
  
  //TODO: add automatic callbacks instead of having optional side-effects (i.e. will-get, and will/did-set)

  template<class C, typename numT>
  static void add_ivar_property_rw ( std::string name, numT C::* ivar, 
    boost::function1<void,Base*> sideffect = 0 )
  {
    add_property ( name, make_ivar_getter(ivar), make_ivar_setter(ivar,sideffect) );
  }

  template<class C, class M>
  static void add_member_property_ro ( M C::* member, const std::string & key, const std::string & key2 )
  {
    add_property (key, MemberPropertyGetter<C,M>(key2, member) );
  }

  template<class C, class M>
  static void add_member_property_rw ( M C::* member, const std::string & key, const std::string & key2 )
  {
    add_property ( key, MemberPropertyGetter<C,M>(key2, member), MemberPropertySetter<C,M>(key2, member) );
  }

  template<class C, typename numT>
  static void add_ivar_property_ro ( std::string name, numT C::* ivar )
  {
    add_property(name,make_ivar_getter(ivar));
  }


  template<class C, typename numT>
  static void add_converting_ivar_property_rw ( 
    std::string name, numT C::* ivar, 
    const boost::function1<numT,numT> & get_converter,
    const boost::function1<numT,numT> & set_converter,
    boost::function1<void,Base*> sideffect = 0 )
  {
    add_property ( name, make_converting_ivar_getter<C,numT>( ivar, get_converter), 
                         make_converting_ivar_setter<C,numT>( ivar, set_converter, sideffect) );
  }




  ///probably not thread safe, and not safe on old gccs (<3.0)
  void checked_initialize_properties()
  {
    if ( !_pinfop )
    {
	  D_PROPS(std::cout << " initializing properties for " << typeid(Derived).name() << std::endl;);
	  _pinfop = new PropertiesInfo;
	  _pinfop->type = typeid(Derived).name();
      dynamic_cast<Derived*>(this)->init_properties();
    }
  }

  ///stub
  static void init_properties() {}

private:

  static PropertiesInfo * _pinfop;

};

//static template member dfinitions...so how exactly does this conform to the ODR?? works tho...

template< typename Types, class Derived > 
typename HasProperties<Types,Derived>::PropertiesInfo * HasProperties< Types,Derived>::_pinfop;


#endif /* _PROPERTIES_HPP_ */
