#pragma once

#include "SeariseProperties.hpp"
#ifdef HAVE_NUMPY
extern "C" {
#include<numpy/arrayobject.h>
}
#include "numpy_support.hpp"
#endif
#include <Python.h>
#include <stdexcept>
#include <string>
#include <boost/format.hpp>

SeariseProperty property_from_pyobject( PyObject * pyvalue )
{
  SeariseProperty value = true;

  if ( PyString_Check( pyvalue ) )
  {
    value = std::string((char*)PyString_AsString( pyvalue ));
  }
#ifdef HAVE_NUMPY
  else if ( PyObject_TypeCheck( pyvalue, &(PyArray_Type) )  )
  {
    value = numpy_to_CPUData_converter::execute( pyvalue );
    /*
    if ( CPUData<float> * cdata = boost::get< CPUData<float> >( &value ) )
    std::cout << " got CPUdata<float> " << algorithm << "." << key << " : " 
    << cdata->dump_dims() << std::endl;
    if ( CPUData<double> * cdata = boost::get< CPUData<double> >( &value ) )
    std::cout << " got CPUdata<double> " << algorithm << "." << key << " : " 
    << cdata->dump_dims() << std::endl;
    */
  }
#endif
  else if ( PyFloat_Check( pyvalue ) )
  {
    value = float(PyFloat_AsDouble( pyvalue ));
  }
  else if ( PyLong_Check( pyvalue ) )
  {
    value = int(PyLong_AsLong( pyvalue ));
  }
  else if ( PyInt_Check( pyvalue ) )
  {
    value = int(PyInt_AsLong( pyvalue ));
  }
  else if ( PyBool_Check( pyvalue ) )
  {
    value = bool(PyObject_IsTrue( pyvalue ));
  }
  else 
  {
    throw std::runtime_error( str( boost::format( "dont know how to handle %s for properties" ) 
                                    % pyvalue->ob_type->tp_name   ) ) ;
  }
  return value;
}

PyObject * pyobject_from_property( SeariseProperty v )  
{
  if ( bool* pb = boost::get<bool>( &v ) )
	{
	  //std::cout << "Yeah, got a bool="<<*pb<<std::endl;
	  boost::python::object obj(*pb);
	  return object_base_initializer( obj );
	}
  else if ( int* pi = boost::get<int>( &v ) )
	{
	  //std::cout << "Yeah, got an int="<<*pi<<std::endl;
	  boost::python::object obj(*pi);
	  return object_base_initializer( obj );
	}
  else if ( std::string* pstr = boost::get<std::string>( &v ) )
	{
	  //std::cout << "Yeah, got a std::string="<<*pstr<<std::endl;
	  boost::python::object obj(*pstr);
	  return object_base_initializer( obj );
	}
  else if ( float* pf = boost::get<float>( &v ) )
	{
	  //std::cout << "Yeah, got a float="<<*pf<<std::endl;
	  boost::python::object obj(*pf);
	  return object_base_initializer( obj );
	} 
#ifdef HAVE_NUMPY
  else if ( CPUData<float> * data = boost::get< CPUData<float> > ( & v ) )
	{
      return cpudata2python( *data );
    }
  else if ( CPUData<double> * data = boost::get< CPUData<double> > ( & v ) )
	{
      return cpudata2python( *data );
    }
#endif		
  else
	throw(std::runtime_error(str(boost::format("could not resolve type of property" ) ) ) );
  
}