/*

COMMON

  THIS MODULE INCLUDES GENERAL SUPPORT FILES THAT DID NOT FIND THEIR OWN PLACE
  THIS INCLUDES HELPERS FOR DEALING WITH BLITZ ARRAS, CONCURRENCY, PROFILING
  GSL, UBLAS, SERIALIZATION, AND A SIMPLE YET EFFECTIVE MIXIN CLASS FOR PROVIDING
  ARBITRARY C++ CLASSES WITH ATTRIBUTES THAT CAN BE QUERIED IN TEXT FORM TO 
  EASE SCRIPTING

COPYRIGHT (C)

  FRAUNHOFER FIT-LIFE
  SCHLOSS BIRLINGHOVEN
  53754 ST. AUGUSTIN
  GERMANY

CONTACT INFORMATION AVAILABLE AT:

  Fraunhofer-Institut für Angewandte Informationstechnik FIT 
  http://fit.fraunhofer.de

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the 
CeCILL-C license as circulated by CEA, CNRS and INRIA at the following 
URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
#ifndef SERILAIZE_BLITZ_HPP
#define SERILAIZE_BLITZ_HPP

#include <blitz/array.h>
#include <blitz/tinyvec-et.h>
#include <boost/serialization/array.hpp>
#include <boost/format.hpp>
#include <iostream>

namespace boost {
namespace serialization {

template<class Archive, typename T, int R>
		void serialize(Archive & ar, blitz::TinyVector<T,R> & vec, const unsigned int version)
{
	for (int r = 0; r < R; ++r) 		
		ar & make_nvp(str(::boost::format("n%i")%r).c_str(),vec[r]);
}

template<class Archive, typename T, int R>
		void serialize(Archive & ar, blitz::Array<T,R> & array, const unsigned int version)
{
  blitz::TinyVector<int,R> shape = array.shape();
  ar & BOOST_SERIALIZATION_NVP(shape);
  
  blitz_check_if_valid( array, true );
    
  for (int r = 0; r < R; ++r)
  {
    if ( shape[r] != array.extent(r) )
    {
      array.resize(shape);
      break;
    }
  }
  
  ar & make_nvp("data",make_array((T*)array.data(),array.numElements()));
}

} // namespace serialization
} // namespace boost

template< typename T, int R >
bool blitz_is_valid( const blitz::Array<T,R> & array )
{
  for (int i = 0; i < array.numElements(); ++i)
    {
      double val = array.data()[i];
      if ( std::isnan(val) || std::isinf( val ) )
        return false;
    }
  return true;
}

template< typename T, int R >
void blitz_check_if_valid( blitz::Array<T,R> & array, bool null_invalids = false )
{
  bool valid = true;
  for ( typename blitz::Array<T,R>::iterator i = array.begin(), e = array.end(); i != e; ++i )
    {
      double val = *i;
#ifndef _WIN32
      if ( std::isnan(val) || std::isinf( val ) )
      {
		valid = false;
		//std::cout << " invalid value " << val << " @ " << i.position() << "/" << array.shape() << std::endl;
		if ( null_invalids ) *i = 0;
      }
#endif
    }
  if ( !valid )
	std::cout << " warning: nans or infs while serializing blitz array " << std::endl;
}


#endif
