# this is a local.mk template 
# a file like this can be placed in the top-level directory
# and will be included in every make file
# if you place such makefiles in sub-directories they will only
# be included when the respective component is built
# following is an example setup including all the important variables
# that you can set and a verbose explanation of their meaning

# CPPFLAGS contains any include paths for libraries in non-standard locations, such as often boost
# or also installations in /usr/local
CPPFLAGS += -I/usr/local/include/boost-1_37

# this is neccessary because usually boost postfixes their library names
# to find this out look in the place were your boost libs are
# the suffix is the bit between the actual libary name (like libbost_program_options)
# and the .a or .so.... suffix
LIBBOOSTSUFFIX=-gcc43-mt

# here provide your cuda installation path (/usr/local/cuda is actually
# the default and will be found anyhow)
CUDA_INSTALL_PATH=/usr/local/cuda

# these are flags for parts that are not absolutely neccesary
# set this if you have libreadline (usually on most linuxes)
# and want to have a nicer python shell in the framework
# with history and line editing
USE_READLINE=1

# set this if you have ffmpeg and headers (usually
# the linux package is called ffmpeg-dev or somesuch)
# this enables using videos as input to the framework
HAVE_FFMPEG=1

# set this if you have python and headers and want python scripting
# and console support
HAVE_PYTHON=1

# set this if you have gtk gtkmm gtkgl and gtkglmm and want to be able
# to visualize data directly from the framework (i.e. use the Visualizer
# algorithm or use the DataWindow class from your own code)
HAVE_GTK=1

# set this if you want to be able to input/output n-dimensional data
# arrays to/from the framework from within python
HAVE_NUMPY=1

# these are flags which are used to build the components when you run
# make in the top-level directory, for example pass -j2 if you want
# make to use two processes to speed up the process (this is not default
# because compiling uses a lot of memory, causing problems on systems
# with less than 4GB of memory)
FLAGS="-kj3"

# this flags includes MPI support. 
# Note: to truly make use of mpi, the flag --use-mpi=true has to be set
HAVE_MPI=1

# use opencv camera driver
HAVE_OPENCV=1

# use sse extensions
HAVE_SSE=1

# use openmp parallelization
HAVE_OPENMP=1

# find ffmpeg headers in subsirs (like avcodec...)
FFMPEG_IN_SUBDIRS=1

# build ueye camera driver
HAVE_UEYE=1

# make refcounts in CPUData atomic using libao
HAVE_AO=1
