#CXXFLAGS+=-fno-strict-aliasing

ifdef DEBUG
	CXXFLAGS += -O0 -fno-inline -g -Wall
	CFLAGS += -O0 -fno-inline -g -Wall
	CPPFLAGS += -DDEBUG
	NVCCFLAGS+=-D_DEBUG -g
else
	CXXFLAGS += -O3 -g -Wall -ffast-math
	CFLAGS += -O3 -g -Wall
endif

ifdef NO_CUDA
	CPPFLAGS += -DNO_CUDA
endif

ifdef HAVE_AO
	CPPFLAGS += -DHAVE_AO
endif

ifdef HAVE_OPENCV
	CPPFLAGS += -DHAVE_OPENCV
endif

ifdef HAVE_GLUT
	CPPFLAGS += -DHAVE_GLUT
	LDFLAGS  += -lglut
endif

ifdef NO_MEMOIZER
  CPPFLAGS += -DNO_MEMOIZER
endif

ifdef HAVE_UEYE
	CPPFLAGS += -DHAVE_UEYE -D__LINUX__ -I../Shell/src
endif

ifdef HAVE_FFMPEG
	CPPFLAGS += -DHAVE_FFMPEG -D__STDC_CONSTANT_MACROS
endif

# activate this flag if ffmpeg cannot find headers and libs
ifdef FFMPEG_IN_SUBDIRS
	CPPFLAGS += -DHAVE_FFMPEG_LIB_PREFIXES
endif

ifdef HAVE_BAUMER
	CPPFLAGS += -DHAVE_BAUMER
	CXXFLAGS += -D_GNULINUX
endif

ifdef HAVE_HASP
	CPPFLAGS += -DHAVE_HASP
endif

ifdef USE_BOF
  CPPFLAGS += -DUSE_BOF
endif
NVCCFLAGS += -DNO_LOGZ
CPPFLAGS += -DNO_LOGZ

ifdef HAVE_SSE
	CPPFLAGS += -DHAVE_SSE
	CPPFLAGS += -msse
endif

ifdef HAVE_SSE4
	CPPFLAGS += -DHAVE_SSE -DHAVE_SSE4
	CPPFLAGS += -msse -msse4.1
endif

ifdef HAVE_PYTHON
	CPPFLAGS += -DHAVE_PYTHON
endif

ifdef HAVE_GTK
	CPPFLAGS += -DHAVE_GTK
endif

ifdef HAVE_NUMPY
	CPPFLAGS += -DHAVE_NUMPY -DPY_ARRAY_UNIQUE_SYMBOL=_PY_ARRAY_UNIQUE_SYMBOL_ -DNO_IMPORT_ARRAY
endif

ifdef LTO
	CFLAGS += -flto
	CXXFLAGS += -flto
	LDFLAGS += -flto
#	NVCCFLAGS += -Xcompiler -flto
endif


ifdef USE_PROFILE
	CFLAGS +=  -fprofile-use -fprofile-correction
	CXXFLAGS +=  -fprofile-use -fprofile-correction
	LDFLAGS +=  -fprofile-use -fprofile-correction
endif

ifdef PROFILE
	CFLAGS += -pg -fprofile-arcs
	CXXFLAGS += -pg -fprofile-arcs
	LDFLAGS += -pg -fprofile-arcs
	NVCCFLAGS += -pg -Xcompiler -fprofile-arcs
endif

ifdef HAVE_OPENMP
	ifdef INSTRUMENT_OPENMP
		CC = kinst-ompp -rcfile ../opari.rc gcc
		CXX = kinst-ompp -rcfile ../opari.rc gcc
	endif
	ifdef DEBUG_OPENMP
		CPPFLAGS += -DDEBUG_OPENMP
	endif
    CFLAGS += -fopenmp
    CXXFLAGS += -fopenmp
    LDFLAGS += -fopenmp
    CPPFLAGS += -DHAVE_OPENMP=1
endif


ifdef HAVE_MPI
	CPPFLAGS += -DHAVE_MPI $(shell pkg-config --cflags mpich2-ch3)
	LDFLAGS += $(shell pkg-config --libs mpich2-ch3)
endif


CFLAGS += -fPIC
CXXFLAGS += -fPIC
NVCCFLAGS += -Xcompiler -fPIC

CPPFLAGS += -Isrc

# Set CUDA Toolkit default
ifndef CUDA_INSTALL_PATH
  CUDA_INSTALL_PATH=/usr/local/cuda
endif

#ifndef BOOST_INSTALL_PATH
#  BOOST_INSTALL_PATH = c:\Program Files (x86)\boost\boost_1_37
#endif
#ifndef HDF5_INSTALL_PATH
#  HDF5_INSTALL_PATH = c:\hdf5
#endif
#ifndef CPPUNIT_INSTALL_PATH
#  CPPUNIT_INSTALL_PATH = c:\Program Files (x86)\cppunit-1.12.0
#endif

#do some cuda flag selection whether or not device emulation is set
ifdef CUDEBUG
	NVCCFLAGS += -g -G
endif

ifdef CUDAEMU
	NVCCFLAGS += --device-emulation 
	CPPFLAGS += -D__DEVICE_EMULATION__
	LIBS += -lcublasemu
else
	LIBS += -lcublas
endif


# Set build flags for cuda
#CPPFLAGS += -I"$(BOOST_INSTALL_PATH)" 
#CPPFLAGS += -I"$(HDF5_INSTALL_PATH)\include"
#CPPFLAGS += -I"$(CPPUNIT_INSTALL_PATH)\include"
LDFLAGS  += -L$(CUDA_INSTALL_PATH)/lib64
LDFLAGS  += -L$(CUDA_INSTALL_PATH)/lib
#LDFLAGS  += -L"$(HDF5_INSTALL_PATH)\lib"
#LDFLAGS  += -L"$(BOOST_INSTALL_PATH)\lib"
#LDFLAGS  += -L"$(CPPUNIT_INSTALL_PATH)\lib"
CPPFLAGS  += -I$(CUDA_INSTALL_PATH)/include

# currently active global settings
CPPFLAGS += -DGUARD_GAMMA_FUNCTIONS_BELOW -DBZ_ALIGN_BLOCKS_ON_CACHELINE_BOUNDARY

# stop CImg from trying to instantiate display routines
CPPFLAGS += -Dcimg_display=0
