/*
 * Copyright (C) 1997 Todd Veldhuizen <tveldhui@oonumerics.org>
 * All rights reserved.  Please see <blitz/blitz.h> for terms and
 * conditions of use.
 *
 */

#ifndef BZ_MEMBLOCK_CC
#define BZ_MEMBLOCK_CC

#include <blitz/numtrait.h>

BZ_NAMESPACE(blitz)

#if 0

template<typename P_type>
void MemoryBlock<P_type>::deallocate()
{
  //FIXME: delegate to manager_, could be done using a write locked swsr unbounded buffer from fastflow, but I am scared to use fastflow as it might break windows?
#ifdef BZ_ALIGN_BLOCKS_ON_CACHELINE_BOUNDARY
  blitz::memblock_manager->free( reinterpret_cast<char*>( dataBlockAddress_ ), length_ * sizeof(T_type) );
#else
  blitz::memblock_manager->free( reinterpret_cast<char*>( data_ ), length_ * sizeof(T_type) );
#endif
}



template<typename P_type>
inline void MemoryBlock<P_type>::allocate(size_t length)
{
    TAU_TYPE_STRING(p1, "MemoryBlock<T>::allocate() [T="
        + CT(P_type) + "]");
    TAU_PROFILE(p1, "void ()", TAU_BLITZ);

	if ( !blitz::memblock_manager.get() )
	  blitz::memblock_manager.reset( new typename blitz::memblock_manager(true) );
	  
	manager_ = blitz::memblock_manager.get();

    size_t numBytes = length * sizeof(T_type);

#ifndef BZ_ALIGN_BLOCKS_ON_CACHELINE_BOUNDARY
	data_ = reinterpret_cast<T_type*>( blitz::memblock_manager->alloc( numBytes ) );
# ifdef HAVE_SSE
	assert( ( size_t(data_) & 15 ) == 0 );
# endif
#else

    if ( false /* always! numBytes < 1024 */ )
    {
	  dataBlockAddress_ = reinterpret_cast<T_type*>( blitz::memblock_manager->alloc( numBytes ) );
	  data_ = dataBlockAddress_;
    }
    else
    {
        // We're allocating a large array.  For performance reasons,
        // it's advantageous to force the array to start on a
        // cache line boundary.  We do this by allocating a little
        // more memory than necessary, then shifting the pointer
        // to the next cache line boundary.

        // Patches by Petter Urkedal to support types with nontrivial
        // constructors.

        const int cacheBlockSize = 128;    // Will work for 32, 16 also

        dataBlockAddress_ = reinterpret_cast<T_type*>
		  (	blitz::memblock_manager->alloc( numBytes + cacheBlockSize - 1 );

        // Shift to the next cache line boundary

        ptrdiff_t offset = ptrdiff_t(dataBlockAddress_) % cacheBlockSize;
        ptrdiff_t shift = (offset == 0) ? 0 : (cacheBlockSize - offset);
        data_ = reinterpret_cast<T_type*>
                (reinterpret_cast<char*>(dataBlockAddress_) + shift);

    }
#endif
}


#endif

BZ_NAMESPACE_END

#endif // BZ_MEMBLOCK_CC
