#ifndef __OMP_FINALIZER_HPP__
#define __OMP_FINALIZER_HPP__

#include "omp_macros.hpp"

#include <boost/format.hpp>

#ifdef HAVE_AO
#include "atomic_ops.h"
#endif

//#define LOG_FINALIZER

struct omp_finalizer
{
  omp_finalizer() : counter(0) 
  {
#ifdef HAVE_AO
	flags = new AO_t[OPENMP_MAX_THREADS];
#endif
	reset();
  }

  // dummy copy contructor for standard containers
  omp_finalizer( const omp_finalizer & ) : counter(0) 
  {
#ifdef HAVE_AO
	flags = new AO_t[OPENMP_MAX_THREADS];
#endif
	reset();
  }

  ~omp_finalizer()
  {
#ifdef HAVE_AO
	delete[] flags;
#endif
  }

  void reset()
  {
#ifdef HAVE_AO
	OMP_LOG( str( boost::format( " resetting finalizer %x " ) % this ) );
	AO_store_full ( &counter, 0 );
	for ( int i = 0; i < OPENMP_MAX_THREADS; ++i )
	  AO_store_full( ( volatile AO_t * ) ( flags + i ), 0 );
#endif
  }

  void sync()
  {
	if ( ! IN_OPENMP ) return;
#ifdef HAVE_AO
	timespec interval = { 0 };
	interval.tv_nsec = 1;
	OMP_LOG( str( boost::format( " syncing finalizer %x " ) % this ) );
	while ( 0 == AO_compare_and_swap_full( ( volatile AO_t * ) ( flags + OPENMP_THREAD_NUM ), 0, 1 ) )
	  nanosleep( &interval, NULL );
	OMP_LOG( str( boost::format( " synced finalizer %x " ) % this ) );
#else
	OPENMP(barrier);
#endif
  }

  bool operator()()
  {
	if ( ! IN_OPENMP ) return true;
#ifdef HAVE_OPENMP
#ifdef HAVE_AO
	int local_counter = AO_fetch_and_add1_full( &counter );
	int nthreads = omp_get_num_threads();
	OMP_LOG( str( boost::format( " finalizer %x thread %i/%i counter %i " )
				  % this % OPENMP_THREAD_NUM % nthreads % local_counter ) );
	if ( local_counter == nthreads - 1 )
	  {
		return true;
	  }
	return false;
#else
	bool retval = false;
	OMP_LOG( str( boost::format( " testing finalizer %x " ) % this ) );
	OPENMP( critical (omp_finalizer) )
	  {
		OMP_LOG( str( boost::format( " finalizing finalizer %x " ) % this ) );
		if ( ++counter == omp_get_num_threads() )
		  {
			counter = 0;
			retval = true;
		  }
	  }
	return retval;
#endif
#else
	return true;
#endif
  }
private:
#ifdef HAVE_AO
  AO_t * flags;
#endif
  volatile size_t counter;
};

#endif

