#ifndef __OMP_MACROS_HPP__
#define __OMP_MACROS_HPP__

#ifdef HAVE_OPENMP
#include <omp.h>
#endif

#ifdef HAVE_OPENMP
#include "omp_macros_functions.hpp"

#include <boost/format.hpp>

#ifdef DEBUG_OPENMP
#define OMP_LOG(x) omp_log(x)
#else
#define OMP_LOG(x)
#endif

#ifdef _WIN32
# ifndef PRAGMA
#  define PRAGMA(x) { OMP_LOG( #x ); }; __pragma(#x)
# endif
#else
# ifndef PRAGMA
#  define PRAGMA(x) { OMP_LOG( #x ); }; _Pragma(#x)
# endif
#endif

#define OPENMP( ... ) PRAGMA( omp __VA_ARGS__ )

#define WORK_RANGE(N) make_work_range( N )

#define IN_OPENMP omp_in_parallel()

#define OPENMP_NUM_THREADS omp_get_num_threads()

#define OPENMP_MAX_THREADS omp_get_max_threads()

#define OPENMP_THREAD_NUM omp_get_thread_num()

#define OPENMP_REQUIRE_SERIAL											\
  if ( omp_in_parallel() && omp_get_num_threads() > 1 )					\
	{																	\
	  OMP_LOG( str( boost::format( " %s(%i): parallel execution of function requiring serial execution " ) \
					% __FILE__ % __LINE__ ) );							\
	  exit(-1);															\
	}																	\
  
#else

#define IN_OPENMP false

#define OPENMP_NUM_THREADS 1

#define OPENMP_MAX_THREADS 1

#define OPENMP_THREAD_NUM 0

#define OMP_LOG(x)

#define OPENMP(...)

#define WORK_RANGE(N) blitz::Range(0,N-1)

#define OPENMP_REQUIRE_SERIAL

#endif


#endif
