/****************************************************************************/
/*    Copyright (C) 2010 by Dominik Endres                                  */
/*    dominik.endres@klinikum.uni-tuebingen.de                              */ 
/*                                                                          */
/*   This program is free software; you can redistribute it and#or modify   */
/*    it under the terms of the GNU General Public License as published by  */
/*    the Free Software Foundation; either version 2 of the License, or     */
/*   (at your option) any later version.                                    */
/*                                                                          */
/*    This program is distributed in the hope that it will be useful,       */
/*    but WITHOUT ANY WARRANTY; without even the implied warranty of        */
/*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         */
/*    GNU General Public License for more details.                          */
/*                                                                          */
/*   You should have received a copy of the GNU General Public License      */
/*    along with this program; if not, write to the                         */
/*    Free Software Foundation, Inc.,                                       */
/*    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             */
/****************************************************************************/

#ifndef OMP_GUARD_HPP_
#define OMP_GUARD_HPP
       
#include <omp.h>
/** This is a class for guard objects using OpenMP
*  It is adapted from the book
*  "Pattern-Oriented Software Architecture". */
class omp_guard {
    public:
	/** Acquire the lock and store a pointer to it */
	omp_guard (omp_lock_t &lock) : lock_ (&lock), owner_ (false)
	{
	  acquire ();
	}

	/** Set the lock explicitly */
	void acquire ()
	{
	  omp_set_lock (lock_);
	  owner_ = true;
	}
	/** Release the lock explicitly (owner thread only!) */
	void release ()
	{
	  if (owner_) {
	      owner_ = false;
	      omp_unset_lock (lock_);
	  }
	}
	/** Destruct guard object. release lock */
	~omp_guard ()
	{
	  release ();
	}
    private:
	omp_lock_t *lock_;  // pointer to our lock
	bool owner_;   // is this object the owner of the lock?
	// Disallow copies or assignment
	omp_guard (const omp_guard &);
	void operator= (const omp_guard &);
};
#endif /*OMP_GUARD_HPP*/