#include <cppunit/extensions/HelperMacros.h>
#include <cppunit/ui/text/TestRunner.h>

#include "dpmm.hpp"
#include "full_covariance_gaussian_model.hpp"

class GDPMMTest : public CppUnit::TestFixture 
{
  CPPUNIT_TEST_SUITE(GDPMMTest);

  CPPUNIT_TEST(test_instantiation);

  CPPUNIT_TEST_SUITE_END();

public:

  typedef dpmm< full_covariance_gaussian_model< float > > gdpmm;

  void test_instantiation()
  {
	gdpmm model;
	model.set_property("alpha0",0.5);
	model.set_property("T",3);
	model.set_property("prototype.D",10);
	model.set_property("prototype.m0",0);
	model.set_property("prototype.n0",10);
	model.set_property("prototype.beta0",0.5);
	model.set_property("prototype.S0",1);
	model.setPrior();
	blitz::Array<float,1> x(10);
	x = 0;

	model.beginLearning();

	model.infer( x, true );
	std::cout << " z " << model.latent() << std::endl;
	model.learn( x );
	model.updateParameterPosterior();

	model.infer( x, true );
	std::cout << " z " << model.latent() << std::endl;
	model.learn( x );
	model.updateParameterPosterior();

	model.infer( x, true );
	std::cout << " z " << model.latent() << std::endl;
	model.learn( x );
	model.updateParameterPosterior();

	model.infer( x, true );
	std::cout << " z " << model.latent() << std::endl;
	model.learn( x );
	model.updateParameterPosterior();

	model.posteriorToPrior();
  }

};

CPPUNIT_TEST_SUITE_REGISTRATION(GDPMMTest);

int main() 
{
  CppUnit::TextUi::TestRunner runner;
  CppUnit::TestFactoryRegistry& registry = CppUnit::TestFactoryRegistry::getRegistry();
  runner.addTest(registry.makeTest());
  return ! runner.run("", false);
}

